''' $Header
''' 
.de Sh
.br
.ne 5
.PP
\fB\\$1\fR
.PP
..
.de Sp
.if t .sp .5v
.if n .sp
..
.de Ip
.br
.ie \\n.$>=3 .ne \\$3
.el .ne 3
.IP "\\$1" \\$2
..
'''
'''     Set up \*(-- to give an unbreakable dash;
'''     string Tr holds user defined translation string.
'''     Bell System Logo is used as a dummy character.
'''
.tr \(bs-|\(bv\*(Tr
.ie n \{\
.ds -- \(bs-
.if (\n(.H=4u)&(1m=24u) .ds -- \(bs\h'-12u'\(bs\h'-12u'-\" diablo 10 pitch
.if (\n(.H=4u)&(1m=20u) .ds -- \(bs\h'-12u'\(bs\h'-8u'-\" diablo 12 pitch
.ds L" ""
.ds R" ""
.ds L' '
.ds R' '
'br\}
.el\{\
.ds -- \(em\|
.tr \*(Tr
.ds L" ``
.ds R" ''
.ds L' `
.ds R' '
'br\}
.TH GATED 8 LOCAL
.SH NAME
gated - gateway routing daemon
.SH SYNOPSIS
.B gated [-t[i][e][r][p][u][R][H]] [logfile]
.SH DESCRIPTION
.I Gated
is a routing daemon that handles multiple routing protocols and is meant
as a replacement for routed(8), egpup(8), and any routing daemon that
speaks the HELLO routing protocol.
.I Gated
currently handles the RIP, EGP, and HELLO routing protocols.  The
.I gated
process can be configured to perform all 3 routing protocols or any combination
of the three.
The configuration for
.I gated
is by default stored in the file \*(L"/etc/gated.conf\*(R", but can be changed
at compile time.
.PP
.I Gated
can be invoked with a number of debug flags and/or a log file.  When
debug flags are used and no log file is specified, the process does not
fork and detach itself from the controlling terminal.  If there is no
log file specified, all debug output is sent to the controlling
terminal; otherwise, the debugging output is sent to the log file
specified. 
The valid debugging flags are as follows:
.Ip -t 8
If used alone, log all error messages, route changes and packets sent and
received.  Using this flag alone turns on the i, e, r, and p debug flags
automatically.  When used with another flag, the -t has no effect and only
the accompanying flags are recognized.  Note that when using other flags,
you must use the -t with them.
.Ip i 8
Log all internal errors and interior routing errors.
.Ip e 8
Log all external errors due to EGP, exterior routing errors, and EGP state
changes.
.Ip r 8
Log all routing changes.
.Ip p 8
Trace all EGP packets sent and received.
.Ip u 8
Log all routing updates sent.
.Ip R
Trace all RIP packets received.
.Ip H
Trace all HELLO packets received.
.PP
The
.I gated
process always logs fatal errors.
If no log file is specified and no debugging flags are set, all messages
are sent to \*(L"/dev/null\*(R".
.SH "SIGNAL PROCESSING"
.PP
.I Gated
catches a number of signals and performs specific actions when
caught.  Currently
.I gated
does special processing with the SIGHUP and SIGINT signals.  When a
SIGHUP is sent to the 
.I gated
process and
.I gated
was invoked with debug flags and a log file, debugging is toggled off and
the log file is closed.  At this point the log file may be moved or removed.
The next SIGHUP to 
.I gated
will toggle the debugging on using the same debug flags in the command
line.  The log file which is specified in the command line is created
if necessary and the debug output is sent to that file.  The debug output
is appended to an already existing log file.  This is useful for having
rotating log files much like the syslog daemon.
.PP
Sending
.I gated
a SIGINT will cause a memory dump to be scheduled within the next sixty
seconds.  The memory dump will be written to the file /usr/tmp/gated_dump.
.I Gated
will finish processing any routing updates currently being worked on before
performing the memory dump.  The memory dump contains all relevant routing
information and interface information.  If the file /usr/tmp/gated_dump
already exists, the memory dump will be appended to the existing file.
.SH "CONFIGURATION FILE OPTIONS FOR HANDLING ROUTING PROTOCOLS"
.PP
In this section, the numerous configuration options are explained.
Each time the
.I gated
process is started, it reads the file \*(L"/etc/gated.conf\*(R" to
obtain its instructions on how routing will be managed with respect to
each protocol.  The configuration options are as follows:
.PP
RIP {yes | no | supplier | pointopoint | quiet| gateway #}
.PP
This tells the
.I gated
process how to perform the RIP routing protocol.  Only one of the above
RIP arguments is allowed after the keyword \*(L"RIP\*(R".  If more than
one is specified, only the first one is recognized.  The arguments:
.Ip yes 8
Perform the RIP protocol.  Process all incoming RIP packets and supply
RIP information every thirty seconds only if there are two or more
network interfaces.
.Ip no 8
Do not perform the RIP protocol.  Do no RIP processing.
.Ip supplier 8
Perform the RIP protocol.  Process all incoming RIP packets and force
the supplying of RIP information every thirty seconds no matter how
many network interfaces are present.
.Ip pointopoint 8
Perform the RIP protocol. Process all incoming RIP packets and force the
supplying of RIP information every thirty seconds no matter how many
network interfaces are present.  When this argument is specified, RIP
information will not be sent out in a broadcast packet.  The RIP information
will be sent directly to the gateways listed in the \*(L"sourceripgateways\*(R"
option described below.
.Ip quiet 8
Process all incoming RIP packets, but do not supply any RIP information no
matter how many network interfaces are present.
.Ip "gateway #" 8
Process all incoming RIP packets, supply RIP information every thirty seconds,
and announce the default route (0.0.0.0) with a metric of #.
The metric should be specified in a value that represents a RIP hopcount.
With this option
set, all other default routes coming from other RIP gateways will be ignored.
The default route is only announced when you are actively peering with at least one
of your EGP neighbors, therefore should only be used if you are
using EGP.
.PP
If no \*(L"RIP\*(R" clause is specified, RIP will not be performed.
.PP
HELLO {yes | no | supplier | pointopoint | quiet| gateway #}
.PP
This tells the
.I gated
process how to perform the HELLO routing protocol.
The arguments parallel the RIP arguments, but do have some minor differences.
Only one of the above
HELLO arguments is allowed after the keyword \*(L"HELLO\*(R".  If more than
one is specified, only the first one is recognized.  The arguments:
.Ip yes 8
Perform the HELLO protocol.  Process all incoming HELLO packets and supply
HELLO information every fifteen seconds only if there are two or more
network interfaces.
.Ip no 8
Do not perform the HELLO protocol.  Don't do HELLO processing.
.Ip supplier 8
Perform the HELLO protocol.  Process all incoming HELLO packets and force
the supplying of HELLO information every fifteen seconds no matter how
many network interfaces are present.
.Ip pointopoint 8
Perform the HELLO protocol. Process all incoming HELLO packets and force the
supplying of HELLO information every fifteen seconds no matter how many
network interfaces are present.  When this argument is specified, HELLO
information will not be sent out in a broadcast packet.  The HELLO information
will be sent directly to the gateways listed in
the \*(L"sourcehellogateways\*(R"
option described below.
.Ip quiet 8
Process all incoming HELLO packets, but do not supply any HELLO information no
matter how many network interfaces are present.
.Ip "gateway #" 8
Process all incoming HELLO packets, supply HELLO information every
fifteen seconds, and announce the default route (0.0.0.0) with a time
delay of #.  The time delay should be specified in milliseconds. 
The default route is only announced when you are actively peering with at least one
of your EGP neighbors, therefore should only be used if you are
using EGP.
.PP
If no \*(L"HELLO\*(R" clause is specified, HELLO will not be performed.
.PP
EGP {yes | no}
.PP
This clause allows the processing of EGP by
.I gated
to be turned on or off.
.Ip no 8
Do not perform any EGP processing.
.Ip yes 8
Perform all EGP operations.
.PP
Please note that by default, EGP processing will take place.  Therefore, if
no \*(L"EGP\*(R" clause is specified, all EGP operations will take place.
.PP
autonomoussystem #
.PP
If performing the EGP protocol, you must use this clause to specify your
autonomous system number (#).  If not specified,
.I gated
will exit and give a fatal error message.
.PP
egpmaxacquire #
.PP
If performing the EGP protocol, this clause specifies the number of EGP peers
you will be performing EGP with.  This cannot be 0 and
.I gated
will exit on such a condition.
.PP
egpneighbor gateway
.PP
If performing the EGP protocol, this clause specifies who you will be
performing EGP with.  \*(L"gateway\*(R" can be either a symbolic name in
/etc/hosts or it can be in dot (a.b.c.d) notation.  Dot notation is
recommended.  Each EGP neighbor will be acquired in the order listed in
the configuration file.
.SH "CONFIGURATION FILE OPTIONS FOR HANDLING ROUTING INFORMATION"
.PP
These next configuration file options deal with how routing knowledge is
dealt with.  This includes incoming knowledge as well as the knowledge we
will give out.
.PP
trustedripgateways gateway [gateway] [gateway] .....
.br
trustedhellogateways gateway [gateway] [gateway] .....
.PP
With these clauses specified, you will only listen to RIP or HELLO
information respectively, from these RIP or HELLO gateways respectively.
\*(L"gateway\*(R" can be either a symbolic name from /etc/hosts or it can
be in dot format (a.b.c.d).  Again, dot format is recommended to eliminate
confusion.  Please note that the propagation of your knowledge is not
restricted in any way by this clause.
.PP
sourceripgateways gateway [gateway] [gateway] .....
.br
sourcehellogateways gateway [gateway] [gateway] .....
.PP
You will send RIP or HELLO information directly to the gateways specified.
If \*(L"pointopoint\*(R" is specified in the \*(L"RIP\*(R" or \*(L"HELLO\*(R"
clauses mentioned above, you will only send RIP or HELLO information to the
specified gateways.  You will NOT send out any information using the
broadcast address.  If \*(L"pointopoint\*(R" is not specified in those
clauses and you are a supplier of RIP or HELLO information, you will send
information to the specified gateways as well as broadcasting it using a
broadcast address.
.PP
noripoutinterface intfaddr [intfaddr] [intfaddr] .....
.br
nohellooutinterface intfaddr [intfaddr] [intfaddr] .....
.br
noripfrominterface intfaddr [intfaddr] [intfaddr] .....
.br
nohellofrominterface intfaddr [intfaddr] [intfaddr] .....
.PP
Turning on and off protocols on a per interface basis is accomplished with
the above clauses.  \*(L"no{rip|hello}frominterface\*(R" means that no
RIP or HELLO information will be accepted coming into the listed interfaces
from another gateway. \*(L"no{rip|hello}outinterface\*(R" means that no
RIP or HELLO knowledge will be sent out of the listed interfaces.
\*(L"intfaddr\*(R" should be in dot notation (a.b.c.d).
.PP
passiveinterfaces intfaddr [intfaddr] [intfaddr] .....
.PP
In order to dynamically determine if an interface is properly functioning,
.I gated
will time out an interface when no RIP or HELLO packets are being
received on that particular interface.
IMP interfaces send a RIP or HELLO packet to themselves to determine
if the interface is properly functioning.  Interfaces that have timed out
automatically have their routes re-installed when routing information
is again received over the interface.
The above clause stops 
.I gated
from timing out the listed interfaces.  The interfaces listed will always
be considered up and working.  If you are not a RIP or HELLO
supplier, all interfaces will NOT be aged and the \*(L"passiveinterfaces\*(R"
automatically applies to all interfaces.
.PP
interfacemetric intfaddr metric#
.PP
This feature allows you to specify an interface metric for the listed
interface.  On systems that support interface metrics, this clause will
override the kernel's metric.  On systems that do not have support for
an interface metric, this feature allows you to use an interface metric.
The interface metric is added to the true metric of each route that comes in
via routing information from the listed interface.  The interface metric is
also added to the true metric of any information sent out via the listed
interface.  You will need one clause for each interface that you want to
specify an interface metric for.
.PP
reconstmetric intfaddr metric#
.PP
This is a first attempt to throw hooks for fallback routing into
.I gated.
If the above clause is used, any RIP information coming into the listed
interface will have the metrics of the routes contained in the RIP information
set to the specified \*(L"metric#\*(R".  Metric Reconstitution should not
be used
lightly as it could be a major contributor in the formation of routing
loops.  USE THIS WITH EXTREME CAUTION.  Any route that has a
metric of infinity will not be reconstituted and left as infinity.
.PP
fixedmetric intfaddr proto {rip|hello} metric#
.PP
This is another attempt to throw hooks for fallback routing in
.I gated.
If the above clause is used, all routing information sent out the
specified interface will have a metric of \*(L"metric#\*(R".  For RIP
specify the metric as a RIP hopcount from 0 to infinity.  For HELLO
specify the metric as a HELLO delay in milliseconds from 0 to infinity. 
Any route that has a metric of infinity will be left as infinity.
Fixed metrics should also be USED WITH EXTREME CAUTION!
.PP
donotlisten net intf addr [addr] ... proto {rip|hello}
.br
donotlistenhost host intf addr [addr] ... proto {rip|hello}
.PP
This clause reads as follows: keyword \*(L"donotlisten\*(R" followed by
a network number which should be in dot format followed by keyword
\*(L"intf\*(R".  Then there should be a list of interfaces in dot format
followed by the keyword \*(L"proto\*(R" followed by \*(L"rip\*(R" or
\*(L"hello\*(R".
.PP
This means that any information regarding \*(L"net\*(R" coming in via
the specified protocols AND from the specified interfaces will be ignored.
You can also put the keyword \*(L"all\*(R" after the keyword \*(L"intf\*(R"
to specify all interfaces on the machine.
For example:
.PP
donotlisten 10.0.0.0 intf 128.84.253.200 proto rip
.PP
means that any RIP information about net 10 coming in via interface
128.84.253.200 will be ignored.  You must have one clause for each net
that you want to put this restriction on.
.PP
donotlisten 26.0.0.0 intf all proto rip hello
.PP
means that any RIP and HELLO information about net 26 coming in via any
interface will be ignored.
.PP
\*(L"donotlistenhost\*(R" can be described the same way as above except
that you provide a host address instead of a network address.  Restrictions
of the nature described above are applied to the specified
host route learned of by the specified routing protocol.
.PP
listen net gateway addr [addr] ... proto {rip|hello}
.br
listenhost host gateway addr [addr] ... proto {rip|hello}
.PP
This clause reads as follows: keyword \*(L"listen\*(R" followed by
a network number which should be in dot format followed by keyword
\*(L"gateway\*(R".  Then there should be a list of gateways in dot format
followed by the keyword \*(L"proto\*(R" followed by \*(L"rip\*(R" or
\*(L"hello\*(R".
.PP
This means to only listen to information about network \*(L"net\*(R"
by the specified protocol(s) only from the listed \*(L"gateways\*(R".
Ignore all other information from any other gateway regarding this network.
For example:
.PP
listen 128.84.0.0 gateway 128.84.253.3 proto hello
.PP
means that any HELLO information about net 128.84 coming in via gateway
128.84.253.3 will be accepted.
Any other information about 128.84 from any other gateway will be rejected.
You must have one clause for each net
that you want to put this restriction on.
.PP
listenhost 26.0.0.15 gateway 128.84.253.3 proto rip
.PP
means that any information about host 26.0.0.15 must come via RIP and from
gateway 128.84.253.3.  All other information regarding this host will be
ignored.
.PP
announce net intf addr [addr] ... proto type [egpmetric #]
.br
announcehost host intf addr ... proto type [egpmetric #]
.br
noannounce net intf addr [addr] ... proto type [egpmetric #]
.br
noannouncehost host intf addr ... proto type [egpmetric #]
.PP
These clauses allow you to place restrictions on the networks and hosts
you announce
and by what protocol they are announced.  The \*(L"announce{host}\*(R" and
\*(L"noannounce{host}\*(R" clauses may not be used together
on the same interface.
You can either have
one or the other per interface.  The differences in the two are this; with the
\*(L"announce{host}\*(R" clause, you will only announce the nets or hosts
that have an
associated \*(L"announce{host}\*(R" clause with the appropriate protocol.  With
the \*(L"noannounce{host}\*(R" clause, you will announce everything,
EXCEPT those
nets or hosts that have an associated \*(L"noannounce{host}\*(R" clause.
So you have a
choice of announcing only what is on your announce list or everything
except those nets on your noannounce list on a per interface basis.
.PP
The arguments are the same as in the \*(L"donotlisten\*(R" clause except
you can specify \*(L"egp\*(R" in the \*(L"proto\*(R" field.
\*(L"type\*(R" can either be rip, hello, egp, or any combination of the three.
When egp is specified in the \*(L"proto\*(R" field,
you must specify an egp metric.  This is the metric at which
you will announce via EGP the listed net.
.PP
Please note that these are not static route entries.  These restrictions
will only apply if the net or host is learned of via one of the
routing protocols.
If a restricted network suddenly becomes unreachable and goes away,
announcement of this net will of course stop until it is learned of again.
Some Examples:
.PP
announce 128.84 intf all proto rip hello egp egpmetric 0
.br
announce 10.0.0.0 intf all proto rip
.br
announce 0.0.0.0 intf 128.84.253.200 proto rip
.br
announce 35.0.0.0 intf all proto rip egp egpmetric 3
.PP
With only these four \*(L"announce\*(R" clauses in the configuration file,
this
.I gated
process will only announce these four nets.
It will announce 128.84 via RIP and HELLO to all interfaces and announce
it via EGP with a metric of 0.  Net 10 will be announced via RIP to all
interfaces.  Net 0 (default) will be announced by RIP out interface
128.84.253.200 only.  Net 35 will be announced via RIP to all interfaces
and announced via EGP with a metric of 3.  These are the only nets that
will be broadcast by this gateway.  Once you have one \*(L"announce\*(R"
clause, only the nets with \*(L"announce\*(R" clauses will be broadcast;
this includes local subnets.  Once an \*(L"announce{host}\*(R" or
\*(L"noannounce{host}\*(R" has an \*(L"all\*(R" specified after an
\*(L"intf\*(R", that clause is applied globally and you lose the
option of having per interface restrictions.  If no routing announcement
restrictions
are desired, just don't include any \*(L"announce\*(R" clauses in your
configuration file.  All information gained will then be propagated
out. Please note that this does not affect what information is being
listened to.  Any net that does not have an \*(L"announce\*(R" clause
is still added to the kernel routing tables, but it is not announced
anywhere else by your gateway.  To stop nets from being added to the
kernel check out the \*(L"donotlisten\*(R" clause discussed earlier.
.PP
announce 128.84 intf 128.59.2.1 proto rip
.br
noannounce 128.84 intf 128.59.1.1 proto rip
.PP
The above clauses mean that on interface 128.59.2.1 only information
about 128.84 will be announced via RIP, but on interface 128.59.1.1,
all information except 128.84 via RIP will be announced.
.PP
noannounce 128.84 intf all proto rip hello egp egpmetric 0
.br
noannounce 10.0.0.0 intf all proto hello
.PP
These clauses mean that all nets except these two that are restricted
will be propagated.  Specifically, net 128.84 will not be announced out
any interface with any protocols.  Knowledge of 128.84 is not sent
anywhere.  Net 10 will not be announced via HELLO to any interface.
This also means that net 10 will be announced to every interface via
RIP.  This net will also be broadcast via EGP with a metric specified
in the \*(L"defaultegpmetric\*(R" clause.
.PP
defaultegpmetric #
.PP
This is a default EGP metric to use when you have no routing restrictions.
Normally, with no routing restrictions,
.I gated
announces all networks learned via HELLO or RIP via EGP with this
specified default EGP metric.  If this clause is not used, the
default EGP metric is set to 255, which would make any EGP advertised
route of this nature be ignored.
When there are no routing restrictions, any network that you have a
direct interface to is announced via EGP with a metric of 0.
Note that this does not include subnets, but only the non-subnetted
network.
.PP
defaultgateway gateway proto {active|passive}
.PP
This is the initial default gateway that is installed in your kernel
during startup and initialization.  If you are running EGP, as soon as
you receive an update from your EGP neighbor, this route is deleted.
If you are not running EGP, but running RIP and \*(L"active\*(R" is
specified, this default route will
be deleted and overwritten by any other default it hears from any
other RIP gateway.  If no default routes are being propagated
throughout your local network, this default route will stay in your
kernel's routing table.
If \*(L"passive\*(R" is specified, the default route will be added to
the kernel and will NOT be overwritten by any other default route
broadcast.  You will also NOT propagate the default route with the
\*(L"passive\*(R" option set.
.PP
\*(L"gateway\*(R" should be an address in dot notation.
\*(L"proto\*(R" should be either rip, egp, or hello.  The \*(L"proto\*(R"
field just initializes what protocol the route was learned by.  In this
case it is unused, but the field kept in for consistency.
.PP
net netaddr gateway addr metric hopcnt {rip|egp|hello}
.br
host hostaddr gateway addr metric hopcnt {rip|egp|hello}
.PP
The following clauses install a static route to net \*(L"netaddr\*(R" or
host \*(L"hostaddr\*(R"
through gateway \*(L"addr\*(R" at a metric of \*(L"hopcnt\*(R" and
learned via either RIP, HELLO, or EGP.  As usual, dot notation is
recommended for the addresses.  This route will be installed in the
kernel's routing table and will never be affected by any other
gateway's RIP or HELLO announcements.  The protocol with which it
was learned by is important when it comes to announcing via EGP.
If the protocol is rip or hello and there are no routing restrictions, then this
route will be announced by EGP with a metric of \*(L"defaultegpmetric\*(R".
If the protocol is egp and there are no routing restrictions, then this route
will be announced by EGP with a metric of \*(L"hopcnt\*(R".
.PP
egpnetsreachable net [net] [net] .....
.PP
This option was left in as a \*(L"soft restriction\*(R".  It cannot be used
when the \*(L"announce\*(R" or \*(L"noannounce\*(R" clauses are used.
Normally, with no restrictions,
.I gated
announces all routes learned from RIP and HELLO via EGP.
The \*(L"egpnetsreachable\*(R" clause restricts EGP announcement to
those nets listed in the clause.  The metric used for the HELLO and
RIP learned routes is the value given in the \*(L"defaultegpmetric\*(R"
clause.  If this clause does not specify a value, the value is set to
255.  With the \*(L"egpnetsreachable\*(R" clause, you can't set individual
unique EGP metrics for each net.  The \*(L"defaultegpmetric\*(R" is used
straight across the board except, of course, for any direct interface.  In
this case, a metric of 0 is used.
.PP
.SH "NOTES ON CONFIGURATION OPTIONS"
.I Gated
stores its Process ID in the file \*(L"/etc/gated.pid\*(R".  If running
EGP, supplying the default route (RIP gateway), and you lose all of
your EGP neighbors, you will stop broadcasting the default route until
your EGP neighbors regain contact.
If routing restrictions are used,
.I gated
logs all invalid networks using syslog at log level LOG_WARNING
and facility LOG_DAEMON.
.PP
With the complexity of the current network topology and with many back door
paths to networks, the use of routing restrictions is recommended.
With the current routing strategies, it is easy for illegal or invalid
networks to penetrate into the ARPAnet Core or the NSFnet backbone.
Using routing restrictions does take a little more maintenance time and
routing restrictions are not the long term answer, but for now, in order
to be good internet players, we must use them.
.PP
.SH "NOTES ON IMPLEMENTATION SPECIFICS"
.I Gated
stores all metrics internally in milliseconds.  The RIP metric is
mapped to a millisecond based metric and processed.  This is so that
we keep the granularity of the HELLO protocol time delay intact.
.PP
In the gated.conf file, all references to POINT-TO-POINT interfaces
must use the DESTINATION address.  This is exactly opposite the old
versions which used the source address of the PTP link.  This is
the only change made to the gated.conf syntax.  Otherwise, all
old gated.conf files should be compatible.
.PP
A two minute hold down is in effect for all of the protocols.
.PP
Changes can be made to the interfaces and gated will notice them
without having to restart the process.  If you change an address,
netmask, subnetmask, broadcast address, or interface metric, you
must first \*(L"ifconfig\*(R" the interface down and then back up
again.  You may have to wait about 30 seconds before gated realizes
the interface is down.  Gated checks the interfaces every 30 seconds.
You need not bring the interface down/up for just flag changes.
.PP
RIP does propagates and listen to host routes.  This was done to handle
PTP links more consistently.  The RIP_TRACE commands are also supported
in this version.
.PP
Subnet interfaces are supported.  This means that subnets will not get
propagated where they shouldn't be.  For example, if a gateway is
gatewaying two class B networks, the subnet routes for each respective
class B net are not propagated into the opposite class B net.  Just
the class B network number is propagated to the appropriate place.
.PP
Gated listens to host and network REDIRECTs and tries to take an action
on the REDIRECT for its own internal tables that parallels the kernel's
action.  In this way, the redirect routine in gated parallels the
Berkeley kernel redirect routine as closely as possible.
Unlike the Berkeley kernel, gated times out routes learned of via
a REDIRECT after six minutes.  The route is then deleted from
the kernel routing tables.  This helps in keeping the routing tables
on your gateway more consistent and clean.  Any route that was
learned of via a REDIRECT is NOT announced by any routing protocol.
.PP
.SH FILES
.Ip /etc/gated.conf 20
The configuration file.
.Ip /etc/gated.pid 20
The process ID is stored here.
.Ip /usr/tmp/gated_dump 20
The memory dump file.
.Ip /etc/gated 20
The GATED process itself.
.PP
.SH "SEE ALSO"
routed(8), RFC827 (EGP formal specs.),
.br
RFC891 (HELLO formal specs.), RFC911 (EGP under UNIX),
.br
The conf/ directory in the distribution package for
.br
sample configuration files.
.SH "AUTHORS"
Mark Fedor
.br
Cornell Theory Center
.br
265 Olin Hall
.br
Cornell University
.br
Ithaca, NY  14853
.br
607/255-8686
.br
fedor@tcgould.tn.cornell.edu
.br
{ucbvax,uunet,decvax}!tcgould.tn.cornell.edu!fedor
.sp
Jeffrey C Honig
.br
Cornell Theory Center
.br
265 Olin Hall
.br
Cornell University
.br
Ithaca, NY  14853
.br
607/255-8686
.br
jch@tcgould.tn.cornell.edu
.br
{ucbvax,uunet,decvax}!tcgould.tn.cornell.edu!jch
.br
.SH CREDITS
This program was derived from Paul Kirton's EGP for UNIX,
UC at Berkeley's routed(8), and HELLO routines by Mike Petry at the
University of Maryland.
.PP
Special thanks to Craig Partridge, craig@nnsc.nsf.net, for linting GATED,
profiling and providing performance enhancements.
.PP
Also, thanks to all the BETA-TESTERS and the NSFNET backbone crew who
put up with occasional nasty conditions brought on by the development of
this program.
