
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: defs.h,v 1.3.1.6 87/12/11 16:10:32 jch Locked $
 *
 */

/* defs.h
 *
 * Compiler switches and miscellaneous definitions.
 */

/* compiler switches */

#define INSTALL 1		/* 0 => debugging - dont install routes in 
					kernel. Reference in main() */

#define SAMENET 0		/* 1 => all egp neighbors must have the same
					shared net in common. Referenced in
					init_egpngh() */

#define ALLOWNONNEIGHBORS 1	/* 1 => allows non-neighbors not on shared net
					to conduct egp for testing,
					referenced in init_egp(), init_egp2(),
					also require SAMENET = 0, and 
					DEFAULTIF to specify source interface
					*/

#ifndef	LOG_FACILITY
#	define	LOG_FACILITY LOG_DAEMON	/*	Insure that syslog
						facility is defined. 
						Referenced in main()
						*/
#endif not LOG_FACILITY

/* initialization file */

#define EGPINITFILE	"/etc/gated.conf"
#define PIDFILE		"/etc/gated.pid"
#define DUMPFILE	"/usr/tmp/gated_dump"
#define VERSION		version

extern char *version;

/* general definitions for GATED user process */

#define TRUE	 1
#define FALSE	 0
#define ERROR	-1			/* used in rt_mknr() and rt_NRnets()*/
#define NOERROR -2			/* used in egppoll() */

#ifndef NULL
#define NULL	 0
#endif

#define LOOPBACKNET	(127 << 24)  /* loopback network */

#define MAXHOSTNAMELENGTH 64		/*used in init_egpngh & rt_dumb_init*/

#undef  MAXPACKETSIZE
#define EGPMAXPACKETSIZE 2048
#define HELLOMAXPACKETSIZE 1440
#define RIPPACKETSIZE 512

#if EGPMAXPACKETSIZE > RIPMAXPACKETSIZE
#  if HELLOMAXPACKETSIZE > EGPMAXPACKETSIZE
#    define MAXPACKETSIZE HELLOMAXPACKETSIZE
#  else
#    define MAXPACKETSIZE EGPMAXPACKETSIZE
#  endif
#else
#  if HELLOMAXPACKETSIZE > RIPMAXPACKETSIZE
#    define MAXPACKETSIZE HELLOMAXPACKETSIZE
#  else
#    define MAXPACKETSIZE RIPMAXPACKETSIZE
#  endif
#endif

#if ALLOWNONNEIGHBORS
#define DEFAULTIF "10.3.0.96"	/* source interface (in quoted dot notation)
				   to use for non-neighbor EGP peers,
				   host specific, for testing only */
#endif

/* macros to select internet address given pointer to a struct sockaddr */

/* result is u_long */
#define sock_inaddr(x) (((struct sockaddr_in *)(x))->sin_addr)

/* result is struct in_addr */
#define in_addr_ofs(x) (((struct sockaddr_in *)(x))->sin_addr)


/* additional definitions to netinet/in.h */

#ifndef IPPROTO_EGP
#define IPPROTO_EGP 8
#endif

/* definitions from C-gateway */

#define reg register
#define ext extern

#define	AMSK	0200		/* Mask values used to decide on which */
#define	AVAL	0000		/* class of address we have */
#define	BMSK	0300
#define	BVAL	0200
#define	CMSK	0340		/* The associated macros take an arg */
#define	CVAL	0300		/* of the form in_addr.i_aaddr.i_anet */

#define	in_isa(x)	(((x) & AMSK) == AVAL)
#define	in_isb(x)	(((x) & BMSK) == BVAL)
#define	in_isc(x)	(((x) & CMSK) == CVAL)

/* definitions from routed/defs.h */

#define equal(a1, a2) \
	(bcmp((caddr_t)(a1), (caddr_t)(a2), sizeof (struct sockaddr)) == 0)


/* system definitions */

extern	char *sys_errlist[];
extern	int errno;


/* external definitions */

extern FILE *ftrace;
extern int tracing;			 /* trace packets and route changes */
extern int savetrace;			 /* save tracing flags */
extern int are_tracing;			 /* are we tracing? */
extern char *logfile;			 /* the log file */
extern int n_interfaces;		 /* # internet interfaces */
extern int n_remote_nets;		 /* # remote nets via internal 
						non-routing gateways */
extern struct rthash nethash[];  /* net routes */
extern struct rthash hosthash[]; /* host routes */
extern struct restricthash rt_restrict[]; /* restrict control routes */
extern struct interface *ifnet;	   /* direct internet interface list */
extern int terminate;		   /* terminate EGP process - set by
					egpallcease(); tested by 
					egpstunacq() and egpacq() */
extern int	s;		/* socket for ioctl calls installing routes,
				   set in main() */
extern int	rip_socket;	/* rip socket for sending rip packets */
extern int	hello_socket;	/* hello socket for sending hello packets */
extern int	egp_socket;	/* egp socket for sending egp packets */
extern int	icmp_socket;	/* icmp socket for listening icmp packets */
extern int	install;	/* if TRUE install route in kernelcall kernel,
				 * it is set by main() after kernel routes
				 * initially read and tested in table2.c */
extern int  rt_default_status;	/* NULL (no default) | INSTALLED (install 
				   needs to be true also) | NOTINSTALLED */
extern u_short	mysystem;		/* autonomous system number */
extern	int	nneigh;			/* number of trusted neighbors in 
						egpnn[] */
extern	int	maxacq;		/* maximum number neighbors to be acquired */
extern  int	n_acquired;	/* number neighbors acquired */
extern	int	egpsleep;	/* No. seconds between egpjob wakeups.
				   Time computed when neigh. (re)acquired 
				   or dropped */
extern	struct egpngh *egpngh;	/* start of linked list of egp neighbor state
				   tables */

extern	u_short	egprid_h;	/* sequence number of received egp packet
				   in host byte order - all ids in internal
				   tables are in host byte order */
extern  int	rt_maxage;	/* maximum allowed age of any route since last
				   updated by an NR message */
extern	int	maxpollint;	/* maximum poll interval of acquired neighbors
				   set in egpstime(), used in rt_NRupdate() */
extern	unsigned	long	gatedtime;	/* time of day in seconds of current
						   interrupt */
extern	char	*strtime;	/* time of day as an ASCII string */
extern  int     sched_a_dump;   /* flag to schedule a dump */

extern	int	conv_factor;		/* conversion factor for RIP, HELLO to
					EGP metrics */
extern	int	doing_rip;		/* Are we running RIP protocols? */
extern	int	doing_hello;		/* Are we running HELLO protocols? */
extern	int	doing_egp;		/* Are we running EGP protocols? */
extern	int	rip_pointopoint;	/* Are we ONLY doing pointopoint RIP? */
extern	int	hello_pointopoint;	/* ONLY doing pointopoint HELLO? */
extern	int	rip_supplier;		/* Are we broadcasting RIP protocols? */
extern	int	hello_supplier;		/* Are we broadcasting HELLO info? */
extern	int	rip_gateway;		/* Are we sending a RIP default? */
extern	int	hello_gateway;		/* Are we sending a HELLO default? */
extern	int	rip_default;		/* Default metric for RIP default */
extern	int	hello_default;		/* Default metric for HELLO default */
extern	int	announcethesenets;	/* Announce restriction list? */
extern	int	donotannounce;		/* Forbid restriction list? */
extern	int	glob_announcethesenets;	/* Global announce restriction list? */
extern	int	glob_donotannounce;	/* Global forbid restriction list? */
extern  int	donotlisten;		/* is there a listen Black list? */
extern  int	islisten;		/* a source listen Black list? */
extern  int	nopeers;		/* Did we lose our EGP peers? */
extern struct sockaddr_in hello_dfltnet;

struct advlist {
	struct advlist *next;
	struct in_addr destnet;
	};
extern struct advlist *adlist;
extern struct advlist *srcriplist;
extern struct advlist *srchellolist;
extern struct advlist *trustedripperlist;
extern struct advlist *trustedhelloerlist;


/* function type declarations */

char *malloc();
char *inet_ntoa();
struct in_addr gd_inet_makeaddr();
int timeout();
int egpallcease();
struct rt_entry *rt_lookup();
struct	interface *if_withnet();
struct rt_entry *rt_find();
struct	interface *if_ifwithaddr();
struct restrictlist *control_lookup();
extern char *strcpy();
extern u_long inet_wholenetof();
