
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: egp.h,v 1.3.1.6 87/12/11 16:11:21 jch Locked $
 *
 */

/* egp.h
 *
 * Uses: defs.h
 *
 * Structure egpngh stores state information for an EGP neighbor. There is 
 * one such structure allocated at initialization for each of the trusted EGP
 * neighbors read from the initialization file. The egpngh structures are in a
 * singly linked list pointed to by external variable "egpngh". 
 * The major states of a neighbor are ACQUIRED, ACQUIRE_SENT, NEIGHBOR and 
 * CEASE_SENT. In the NEIGHBOR state there are three reachability substates 
 * according to whether the neighbor and this gateway both consider each other
 * up (BOTH_UP), this gateway considers its neighbor down (NG_DOWN), or the 
 * neighbor has reported that it considers this gateway down (ME_DOWN).
 */


struct egpngh {
	struct	egpngh	 *ng_next;	/* next state table of linked list */
/*	struct	interface *ng_ifp;	/* pointer to shared net interface 
					   table */
	int	ng_state;
	int	ng_reach;	/* reachability substate */
	int	ng_flags;
	u_char	ng_status;	/* Info saved for cease retransmission */
	struct in_addr	ng_myaddr;	/* My inet address for common net */
	struct in_addr	ng_addr;	/* Neighbor's inet address */
	u_short	ng_sid;		/* seq.# next acq., hello or poll to send */
	u_short ng_rid;		/* seq.# last acq., hello or poll recvd */

				/* Acquire and hello info */
	int	ng_retry;	/* For states ACQUIRE_SENT and CEASE_SENT,
				  No. retries to acquire or cease neighbor.
				  For NEIGHBOR state, no. consecutive times
					neighbor is unreachable */
	int	ng_hint;	/* Send acquire, hello or cease interval 
					in seconds */
	long	ng_htime;	/* Time for next acquire, hello, or cease */
	int	ng_responses;	/* Shift register of responses for determining
				reachability, each set bit corresponds to a
				response, each zero to no response */
	int	ng_rcmd;	/* No. acq., hello and poll commands recvd
				   since lats check */

				/* NR Poll info */
	int	ng_snpoll;	/* No. sends of this NR poll id */
	int	ng_spint;	/* Send NR poll interval (in seconds) */
	long	ng_stime;	/* Time to send next NR poll */
	int	ng_runsol;	/* No. unsolicited NR msgs received for 
				   previous id */
	int	ng_noupdate;	/* # successive polls (new id) which did not
				   receive a valid update */
	int	ng_rnpoll;	/* No. polls received before next poll time */
	int	ng_rpint;	/* Rcv NR poll interval (in seconds) */
	long	ng_rtime;	/* Time to rcv next NR poll */
	struct in_addr	ng_paddr;	/* poll address */
	};

#define egpng	struct egpngh

/* States */
#define EMPTY		0
#define UNACQUIRED	1
#define ACQUIRE_SENT	2
#define NEIGHBOR	3
#define CEASE_SENT	4
#define BAD		5		/* unused */

/* reachability substates for state NEIGHBOR */
#define BOTH_UP	0
#define NG_DOWN	1	/* according to me my peer is unreachable */
#define ME_DOWN	2	/* according to my peer I am unreachable */

/* flags */
#define NG_BAD	1	/* change to state BAD when cease complete */

/* Basic EGP packet */
#define egpkt	struct egppkt

egpkt	{	u_char	egp_ver;	/* Version # */
		u_char	egp_type;	/* Opcode */
		u_char	egp_code;
		u_char	egp_status;
		u_short	egp_chksum;
		u_short	egp_system;	/* Autonomous system */
		u_short	egp_id;
		};

/* EGP neighbor acquisition packet */
struct egpacq	{
	egpkt	ea_pkt;
	u_short	ea_hint;	/* Hello interval in seconds */
	u_short	ea_pint;	/* NR poll interval in seconds */
	};

/* EGP NR poll packet */
struct egppoll	{
	egpkt	ep_pkt;
	u_short	ep_unused;
	struct in_addr	ep_net;		/* Source net */
	};

/* EGP NR Message packet */
struct egpnr	{
	egpkt	en_pkt;
	u_char	en_igw;		/* No. internal gateways */
	u_char	en_egw;		/* No. external gateways */
	struct in_addr	en_net;		/* shared net */
	};

#define NRMAXNETUNIT 9		/* maximum size per net in octets of net part
				of NR message */
/* EGP Error packet */
struct egperr	{
	egpkt	ee_pkt;
	u_short ee_rsn;
	u_char 	ee_egphd[12];	/* First 12 bytes of bad egp pkt */
};

#define EGPLEN	(sizeof(egpkt))
#define EGPVER	2

/* EGP Types */
#define EGPNR		1
#define	EGPPOLL		2
#define EGPACQ		3
#define EGPHELLO	5
#define	EGPERR		8

/* Neighbor Acquisition Codes */
#define NAREQ	0	/* Neighbor acq. request */
#define NACONF	1	/* Neighbor acq. confirmation */
#define NAREFUS	2	/* Neighbor acq. refuse */
#define NACEASE	3	/* Neighbor cease */
#define NACACK	4	/* Neighbor cease ack */

/* Neighbour Acquisition Message Status Info */
#define UNSPEC		0
#define	ACTIVE		1
#define	PASSIVE		2
#define	NORESOURCE	3
#define	ADMINPROHIB	4
#define	GODOWN		5
#define	PARAMPROB	6
#define	PROTOVIOL	7

/* Neighbor Hello Codes */
#define HELLO	0
#define HEARDU	1

/* Reachability, poll and update status */
#define INDETERMINATE	0
#define UP		1
#define DOWN		2
#define UNSOLICITED   128

/* Error reason status */
#define	EUNSPEC		0
#define EBADHEAD	1
#define	EBADDATA	2
#define ENOREACH	3
#define	EXSPOLL		4
#define ENORESPONSE	5
