
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: egp_param.h,v 1.3.1.6 87/12/11 16:11:28 jch Exp $
 *
 */

/* egp_param.h
 *
 * Defines various egp parameters
 */

/* Retry counts */

#define NACQ_RETRY	5 	/* Max. No. acquire retransmits before 
				   switch to longer interval */
#define NCEASE_RETRY	3 	/* Max. No. cease retransmits */
#define NPOLL		2	/* Max. No. NR polls to send or receive
				   with same id */

/* Acquire interval constants */

/* MINHELLOINT below is used for neigh. acquire retransmit interval when
 * in state UNACQUIRED or ACQUIRE_SENT or cease retry interval when not
 * acquired
 */
#define LONGACQINT	240	/* Neigh. acquire retransmit interval (sec)
				   when no response after NACQ_RETRY or
				   after ceased */
#define	ACQDELAY	3600	/* Delay (seconds) before try to reacquire
				   neighbor that is misbehaving */

/* Hello interval constants */

#define MINHELLOINT	30	/* Minimum interval for sending and
				   receiving hellos */
#define MAXHELLOINT	120	/* Maximum hello interval, sec. */
#define HELLOMARGIN	2	/* Margin in hello interval to allow for delay
				   variation in the network */
/* Poll interval constants */

#define MINPOLLINT	120	/* Minimum interval for sending and receiving
				   polls */
#define MAXPOLLINT  	480	/* Maximum poll interval, sec. */

/* repoll interval is set to the hello interval for the particular neighbor */

/* Reachability test constants */

#define NCOMMANDS	4	/* No. commands sent on which reachability is
				   based */
#define NRESPONSES	2	/* No. responses expected per NCOMMANDs sent,
				   if down, > NRESPONSES => up,
				   if up, < NRESPONSES => down */
#define NUNREACH	60	/* No. consecutive times neighbor is 
				   unreachbable before ceased */
#define MAXNOUPDATE	3	/* Maximum # successive polls (new id) for
				   which no update was received before cease
				   and try to acquire an alternative :/

/* Command reception rate constants */

#define	CHKCMDTIME	480	/* No. seconds betw. check for recv too many
				   acq., hello or poll commands */
#define NMAXCMD		20	/* Max. # acq., hello and poll commands
				   allowed during CHKCMDTIME seconds */
