
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *          were developed and originally written at the University
 *	    of Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: ext.c,v 1.3.1.6 87/12/11 16:11:34 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: ext.c,v 1.3.1.6 87/12/11 16:11:34 jch Exp $";
#endif	not lint

#include "include.h"

/*
 * Socket declarations for each routing protocol and ioctl's
 */
int egp_socket;			/* egp raw socket descriptor */
int rip_socket;			/* rip raw socket descriptor */
int hello_socket;		/* hello raw socket descriptor */
int icmp_socket;		/* icmp raw socket descriptor */
int s;				/* socket for kernel ioctl calls */

/*
 * Routing tables and restriction table.
 */
struct rthash nethash[ROUTEHASHSIZ];
struct rthash hosthash[ROUTEHASHSIZ];
struct restricthash rt_restrict[ROUTEHASHSIZ];

/*
 * EGP specific variables
 */
u_short	mysystem;		/* autonomous system number */
int	nneigh;			/* number of trusted egp neighbors */
int	maxacq;			/* maximum number neighbors to be acquired */
int	n_acquired;		/* number neighbors acquired */
int	egpsleep;		/* seconds between egpjob() wakeups */
struct	egpngh	*egpngh;	/* start egp neighbor state table linked list */
int	nopeers = 0;		/* has there been an EGP peer loss? */
u_short	egprid_h;		/* sequence number of received egp packet
				   in host byte order - all ids in internal
				   tables are in host byte order */
/*
 * configuratrion specification variables.
 */
int	conv_factor;		/* default EGP metric with no restrictions */
int	doing_rip;		/* Are we running RIP protocols? */
int	doing_hello;		/* Are we running HELLO protocols? */
int	doing_egp;		/* Are we running EGP protocols? */
int	rip_pointopoint;	/* Are we ONLY doing pointopoint RIP? */
int	hello_pointopoint;	/* Are we ONLY doing pointopoint HELLO? */
int	rip_supplier;		/* Are we broadcasting RIP protocols? */
int	hello_supplier;		/* Are we broadcasting HELLO protocols? */
int	rip_gateway;		/* Are we broadcasting a RIP default? */
int	hello_gateway;		/* Are we broadcasting a HELLO default? */
int	rip_default;		/* Default metric for RIP default */
int	hello_default;		/* Default metric for HELLO default */
int	announcethesenets;	/* is there an announce restrict list? */
int	donotannounce;		/* is there a forbid restrict list? */
int	glob_announcethesenets;	/* is there a global announce restrict list? */
int	glob_donotannounce;	/* is there a global forbid restrict list? */
int	donotlisten;		/* is there a listen black list? */
int	islisten;		/* is there a source listen black list? */

/*
 * Miscellaneous variables.
 */
struct interface *ifnet;	/* direct internet interface list */
int n_interfaces;		/* # internet interfaces */
int n_remote_nets;		/* # remote nets via internal static routes */
int tracing;			/* log errors, route changes &/or packets */
int savetrace;			/* save trace flags */
int are_tracing;		/* are we tracing? */
char *logfile;			/* the log file */
int terminate;			/* terminate EGP process */
int install;			/* if TRUE install route in kernel */
int rt_default_status;		/* NULL | INSTALLED | NOTINSTALLED */
int rt_maxage;			/* maximum allowed age of any route */
int maxpollint;			/* max poll interval of acquired neighbors */
unsigned long gatedtime;	/* time of day in secs of current interrupt */
char	*strtime;		/* time of day as an ASCII string */
int sched_a_dump;		/* flag to schedule an information dump */
FILE *ftrace = NULL;
struct advlist *adlist;		/* nets to go into NR update */
struct advlist *srcriplist;	/* gateways to speak RIP directly to */
struct advlist *srchellolist;	/* gateways to speak HELLO directly to */
struct advlist *trustedripperlist;	/* trusted RIP gateways */
struct advlist *trustedhelloerlist;	/* trusted HELLO gateways */

/*
 * HELLO protocol, default route specification.
 */
struct sockaddr_in hello_dfltnet;

