/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines were
 *	    developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 */

#ifndef	lint
static char *rcsid = "$Header: hello.c,v 1.3.1.20 88/02/11 17:32:04 jch Exp $";
#endif	not lint

/*
 *	Hello output routines were taken from Mike Petry (petry@trantor.umd.edu)
 *	Also, hello input routines were written by Bill Nesheim, Cornell
 *	CS Dept,  Currently at nesheim@think.com
 */

#include "include.h"

helloin(hellobuf)
char *hellobuf;
{
  register int dsize, index, i;
  struct hellohdr *hi;
  struct m_hdr *mh;
  struct type1pair *t1;
  struct ip *ip;
  struct sockaddr_in gateway;
  struct rt_entry *rt, *exrt;
  struct interface *ifp, *ifpc;
  int change = FALSE;
  int fromproto;
  int latched;			/* Fuzzball has this route in hold down */
  u_short hello_cksum(), cksum;
    
  ip = (struct ip *)hellobuf;
    
  /* drop the ip header	*/
  dsize = ip->ip_len  /* - (ip->ip_hl<<2)  data length here, NOT total */;
  index = ip->ip_hl << 2;
  hi = (struct hellohdr *) &hellobuf[index];
  index +=  /* sizeof (struct hellohdr) */ 10;

  /* ignore if not from a trusted HELLOer */
  if (trustedhelloerlist != (struct advlist *)NULL) {
    register struct advlist *ad;      /* list of trusted HELLOers */
    register int OK = 0;

    for(ad = trustedhelloerlist; ad; ad = ad->next)
      if (ip->ip_src.s_addr == ad->destnet.s_addr) {
        OK++;
        break;
      }
    if (OK == 0)
      return;
  }

  /* Do we share a net with the sender? */
  bzero((char *)&gateway, sizeof (gateway));
  gateway.sin_family = AF_INET;
  gateway.sin_addr = ip->ip_src;
  if ((ifp = if_withnet(&gateway)) == NULL) {
    TRACE_EXT("helloin: gw %s no shared net?\n", inet_ntoa(ip->ip_src));
    return;
  }
  if (ifp->int_flags & IFF_NOHELLOIN)
	return;

  /*
   * Are we talking to ourselves?
   *
   * if_ifwithaddr() handles PTP links also.  If packet came
   * from other end of a PTP, let it fall through for further
   * processing.  We shouldn't ever hear our own RIPs on a PTP link.
   */
  ifpc = if_ifwithaddr(&gateway);
  if (ifpc) {
    rt = rt_find(&gateway);
    if (rt == 0 || (rt->rt_state & RTS_INTERFACE) == 0)
      addrouteforif(ifpc);
    else
      rt->rt_timer = 0;
    if ((ifpc->int_flags & IFF_POINTOPOINT) == 0)
      return;
  }
  /*
   * update the interface timer on interface the packet came in on.
   */
  if ((rt = rt_find(&gateway)) && (rt->rt_state & (RTS_INTERFACE|RTS_REMOTE)))
    rt->rt_timer = 0;
  else  /* re-install route for interface */
    addrouteforif(ifpc);

  fromproto = if_updateactivegw(ifp, gateway.sin_addr.s_addr, RTPROTO_HELLO);
  /* check the hello checksum */

  cksum = hello_cksum((u_short *) hi, dsize);
  if (cksum == hi->h_cksum) {
    
    /* message is made up of one or more sub messages */
    dsize -= /* sizeof (struct hellohdr) */ 10; 
    while (dsize > 0) {
      mh = (struct m_hdr *) &hellobuf[index];
      index += sizeof (struct m_hdr);
      dsize -= sizeof (struct m_hdr);
      switch (mh->m_type) {
        case 0:
          index += mh->m_count * sizeof (struct type0pair);
          dsize -= mh->m_count * sizeof (struct type0pair);
          /* not interested in type 0 messages */
          break;
	case 1:
          t1 = (struct type1pair *)&hellobuf[index];
          for (i = 0; i < mh->m_count; i++, t1++) {
            int delay = ntohs(t1->d1_delay);
            struct sockaddr_in dst;

            bzero((char *)&dst, sizeof (dst));
            dst.sin_family = AF_INET;
            /* dogone alignment problems! */
            bcopy((char *)&t1->d1_dst, (char *)&dst.sin_addr,
                    sizeof (struct in_addr));

            index += sizeof (struct type1pair);
            dsize -= sizeof (struct type1pair);

	    /*
	     * Fuzzballs set the top bit of the delay field when the
	     * delay jumps too high too fast, and latches the route.
	     * They unlatch the route when the metric by that route
	     * goes down some, or when the hold-down-latch period
	     * expires (in which case they start saying infinity)
	     * 
	     * The current (Dec 1987) idea is to insulate the RIP
	     * world from a Fuzzball hold down and to avoid making
	     * any routing changes during a hold down.  
	     * 
	     * If the route in hold down is not being used, the
	     * new route is ignored, even if it is more attractive.
	     * 
	     * If we are using the route, we will not update the
	     * routing table with the new metric, but continue to
	     * use the last non-hold down metric.  We will also not
	     * update the hello window.  We will however, reset the
	     * timer so that the route is not expired during hold down.
	     */
	    if ((u_short)delay & 0x8000) {	/* latched */
		delay = (int)((u_short)delay & 0x7fff);
		latched = TRUE;
	    } else {
		latched = FALSE;
	    }
	/*
	 *	Add the interface metric converted to a HELLO delay.
	 */
            delay += mapmetric(RIP_TO_HELLO,ifp->int_metric);

            TRACE_HEL("HELLO: %s time delay %d %s\n",
				inet_ntoa(dst.sin_addr), delay,
				latched ? "latched" : "");

            /* check for internal route */
            rt = rt_lookup((int)INTERIOR, &dst);
            if (rt == NULL) {	/* new route */
              int replaced = 0;
              struct rt_entry rttmp;

              if (delay >= DELAY_INFINITY)
                continue;
              /*
               * If the route is in Fuzzball hold down, ignore
               * it.  We don't want to use it until it stabilizes.
               */
              if (latched)
                continue;
              /*  is this new net acceptable? */
              bzero((char *)&rttmp, sizeof(rttmp));
              rttmp.rt_listenlist = control_lookup(RT_NOLISTEN, &dst);
              rttmp.rt_srclisten = control_lookup(RT_SRCLISTEN, &dst);
              if (is_valid_in(&rttmp, RTPROTO_HELLO, ifp, &gateway) == 0)
                continue;
              /* 
               * delete old external route(s), if any,
               * and add to internal table.
               */
              while (exrt = rt_lookup((int)EXTERIOR, &dst)) {
                replaced++;
                rt_delete(exrt, KERNEL_INTR);
              }
              TRACE_EXT("HELLO_%s %s delay %d \n",
                           (replaced) ? "REPLACE" : "ADD",
                           inet_ntoa(dst.sin_addr), delay);
              (void) rt_add((int)INTERIOR,&dst,&gateway,delay,0,RTPROTO_HELLO,
                            fromproto);
              change = TRUE;
            }
            else {	/* update existing route */
              if ((rt->rt_flags & RTF_GATEWAY) == 0)
                continue;	/* don't mess with non-gw's */
              if (rt->rt_state & (RTS_INTERFACE | RTS_PASSIVE))
                continue;
              if (is_valid_in(rt, RTPROTO_HELLO, ifp, &gateway) == 0)
                continue;

              if (exrt = rt_lookup((int)EXTERIOR, &dst)) {
                TRACE_INT("GATED WARNING: route to %s in both tables!\n",
                              inet_ntoa(dst.sin_addr));
                do {
                  rt_delete(exrt, KERNEL_INTR);
                } while (exrt = rt_lookup((int)EXTERIOR, &dst));
                TRACE_EXT("GATED-HELLO: Route in exterior table deleted\n");
              }
              if (delay >= DELAY_INFINITY) {
                /* destination now unreachable */
                if (equal (&rt->rt_router, &gateway)) {
		  if ( latched ) {
                    TRACE_ACTION(LATCHED, rt);
                    change = TRUE;
                    rt->rt_timer = 0;
                  } else {
                    if (rt->rt_metric < DELAY_INFINITY) {
                      TRACE_ACTION(UNREACHABLE, rt);
                      change = TRUE;
                      rt->rt_metric = DELAY_INFINITY;
                      rt->rt_timer = EXPIRE_TIME;
                    }
                    add_win(&rt->rt_hwindow, DELAY_INFINITY);
                  }
                }
                continue;
              }
              if (equal (&rt->rt_router, &gateway)) {
		if (latched) {
                  TRACE_ACTION(LATCHED, rt);
                  change = TRUE;
		} else {
                  if (METRIC_DIFF(rt->rt_metric,delay) >= HELLO_HYST(rt->rt_metric)) {
                    if (rt_change(rt, &gateway, delay, RTPROTO_HELLO, fromproto)) {
                      change = TRUE;
                    }
		  }
                  add_win(&rt->rt_hwindow, delay);
		}
                rt->rt_timer = 0;
              }
              else {
                /*
                 * if the current metric is INFINITY, then we
                 * will only listen to our current gateway.
                 * yes, I know this is a hold down. Yes, I
                 * know it's yucky.  If nothing is heard from our
                 * gateway within 120 seconds, the route is
                 * deleted.
                 */
                if (rt->rt_metric >= DELAY_INFINITY)
                  continue;
		/*
		 * If the route is in Fuzzball hold down, ignore
		 * it.  We don't want to use it until it stabilizes.
		 */
		if (latched)
		  continue;
                /*
                 * use the new gateway.
                 */
                if (((delay < rt->rt_metric) &&
                    (METRIC_DIFF(delay,rt->rt_metric) >= HELLO_HYST(rt->rt_metric))) ||
                    (rt->rt_timer > MINPOLLINT &&
                    !(rt->rt_state & RTS_CHANGED))) {
                   if (rt_change(rt, &gateway, delay, RTPROTO_HELLO, fromproto))
                     change = TRUE;
                   add_win(&rt->rt_hwindow, delay);
                   rt->rt_timer = 0;
                }
              }
            }
          } /* for each advertized net */

          break;
        default:
          syslog(LOG_ERR, "helloin: invalid type %d\n", mh->m_type);
      } /* switch (mh->m_type) */
    } /* while dsize */
  }	/* checksum */	
    
  if ((change && tracing & TR_RT) || tracing & TR_HELLO)
    printf("hello_update: above %d routes from %s updated %s\n",
                   mh->m_count,inet_ntoa(ip->ip_src), strtime);
} 

u_short
hello_cksum(cksumaddr, len)
u_short *cksumaddr;
int len;
{
  register int nleft = len - 2 ;
  register u_short *w = cksumaddr + 1 ;
  register int sum = 0;

  while (nleft > 0) {
    sum += *w++ ;
    if (sum & 0x10000) {
      sum &= 0xffff ;
      sum++ ;
    }
    nleft -= 2;
  }

  if (nleft == -1)
  sum += *(u_char *)w;

  return (~sum);
}

hellojob()
{
  struct interface *ifp;
  struct sockaddr_in *sin,*din;
  register struct advlist *ad;

  if (!(hello_pointopoint)) {
    for (ifp = ifnet; ifp != 0; ifp = ifp->int_next) {
      if ((ifp->int_flags & IFF_PASSIVE) ||
          ((ifp->int_flags & IFF_UP) == 0) ||
          (ifp->int_flags & IFF_NOHELLOOUT))
        continue;
      din = (ifp->int_flags & IFF_BROADCAST) ?
                (struct sockaddr_in *)&ifp->int_broadaddr :
                (ifp->int_flags & IFF_POINTOPOINT) ?
                    (struct sockaddr_in *)&ifp->int_dstaddr :
                    (struct sockaddr_in *)&ifp->int_addr;
      sin = (struct sockaddr_in *)&ifp->int_addr;
      hellotsend(din->sin_addr, sin->sin_addr, ifp);
    }
  }
  for (ad = srchellolist; ad; ad = ad->next) {
    struct sockaddr_in tmpdst;

    bzero((char *)&tmpdst, sizeof(tmpdst));
    tmpdst.sin_family = AF_INET;
    tmpdst.sin_addr = ad->destnet;
    if ((ifp = if_withnet(&tmpdst)) <= (struct interface *)0) {
      TRACE_INT("Source HELLO gateway %s not on same net\n",
                     inet_ntoa(tmpdst.sin_addr));
      continue;
    }
    if (((ifp->int_flags & IFF_UP) == 0) ||
        (ifp->int_flags & IFF_NOHELLOOUT))
      continue;
    sin = (struct sockaddr_in *)&ifp->int_addr;
    hellotsend(tmpdst.sin_addr, sin->sin_addr, ifp);
  }
}

hellotsend( dst, src, ifp)
	struct in_addr dst, src;  /* dest. and source internet address */
	struct interface *ifp;
{
  struct rt_entry *rt;
  struct rthash *rh;
  struct type1pair *t1;
  struct sockaddr_in sdst;
  struct m_hdr *mh;
  u_short hello_cksum();
  char hpkt[HELLOMAXPACKETSIZE];
  int index, save_index;
  int gateway_default;
  
  /*
   * Advertise only nets that are reachable by an interface other then
   * the one we are going to use (ifp).  if that is the case, announce
   * with a metric of INFINITY.
   */

  index = /* sizeof (struct hellohdr) */ 10;

  mh = (struct m_hdr *) &hpkt[index];
  save_index = index += sizeof (struct m_hdr);
  mh->m_type = 1;
  mh->m_count = 0;

  for (rh = nethash; rh < &nethash[ROUTEHASHSIZ]; rh++)
    for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw ) {
      if ((rt->rt_state & RTS_INTERIOR) == 0)  /* only interior routes */
        continue;
      if (is_valid(rt, RTPROTO_HELLO, ifp) == 0)
        continue;

#ifdef	notdef
      if ((gd_inet_netof(sock_inaddr(&rt->rt_dst)) == 0) ||
          (rt->rt_proto & RTPROTO_REDIRECT))
#else
      if (rt->rt_proto & RTPROTO_REDIRECT)
#endif
        continue;

      /* No subnets... */
      if (rt->rt_state & RTS_SUBNET)
        continue;

      /*
       * Check if this packet is full and send it if so.
       */
      if ( index + sizeof(struct type1pair) >= HELLOMAXPACKETSIZE ) {
        hello_send(dst, src, (u_char *)hpkt, index, mh->m_count);
        index = save_index;	/* Reset to start of network info */
	mh->m_count = 0;
      }

      t1 = (struct type1pair *)&hpkt[index];

      /*
       * set the metric.
       */
      gateway_default = FALSE;
      if (hello_gateway &&
          (sock_inaddr(&rt->rt_dst).s_addr == hello_dfltnet.sin_addr.s_addr) &&
          (rt->rt_state & RTS_NOTINSTALL)) {
        t1->d1_delay = hello_default;
        gateway_default = TRUE;
      } else /* same interface ?, do split-horizon */
        if ((rt->rt_ifp == ifp) &&
           (rt->rt_fromproto & (RTPROTO_HELLO|RTPROTO_DIRECT|RTPROTO_KERNEL)) &&
           ((rt->rt_state & RTS_STATIC) == 0))
          t1->d1_delay = DELAY_INFINITY;
      else {
        if (rt->rt_metric >= TDHOPCNT_INFINITY) {
          t1->d1_delay = DELAY_INFINITY;
        } else {
	  /* Add a 100ms as the delay through this gateway */
          t1->d1_delay = rt->rt_metric + 100;
        }
      }

      /*
       *  Add the interface metric after converting to a HELLO delay
       */
      t1->d1_delay += mapmetric(RIP_TO_HELLO,ifp->int_metric);

      /*
       *  Make sure the interface for the route is up and metric is less
       * than or equal to infinity.
       */
      if ((t1->d1_delay > DELAY_INFINITY) ||
          ((rt->rt_ifp->int_flags & IFF_UP) == 0))
        t1->d1_delay = DELAY_INFINITY;

      /*
       * If a HELLO fixed metric is defined for this interface
       * use that if the metric is not infinity.
       */
      if ( (ifp->int_flags & IFF_HELLOFIXEDMETRIC) &&
           (t1->d1_delay < DELAY_INFINITY) && !gateway_default) {
        t1->d1_delay = ifp->int_hellofixedmetric;
      }

      t1->d1_delay = htons(t1->d1_delay);

      TRACE_UPD("HELLO_SND: net %s time delay %d\n",
                inet_ntoa( sock_inaddr(&rt->rt_dst)), ntohs(t1->d1_delay));
      t1->d1_offset = htons(0);		/* should be signed clock offset */
      sdst.sin_addr.s_addr =
                         htonl((u_long)gd_inet_netof(sock_inaddr(&rt->rt_dst)));
      bcopy((char *)&sdst.sin_addr,(char *)&t1->d1_dst,sizeof(struct in_addr));
      index += sizeof (struct type1pair);
      mh->m_count++;
    }
  hello_send(dst, src, (u_char *)hpkt, index, mh->m_count);
}

/*
 * hello_send():
 * 	Fill in the hello header and checksum, then send the packet.
 */

hello_send(dst, src, hello, length, n_nets)
     struct in_addr dst, src;  /* destination and source internet address */
     u_char	*hello;		/* pointer to start of hello packet */
     int	length;		/* length in octets of hello packet */
     int	n_nets;		/* number of nets in this update for debugging */
{
  struct sockaddr_in hellosaddr;
  struct hellohdr *hi;
  struct interface   *ifp;
  struct tm *gmt;
  struct timeval tp;
  struct tm *gmttime();
  int	error = FALSE;
  int	sendhello = TRUE;		/*  for debugging */

  hi = (struct hellohdr *) hello;
    
  (void) gettimeofday(&tp, (struct timezone *)0);
  gmt = gmtime(&tp.tv_sec);
    
  /*
   * set the date field in the HELLO header.  Be very careful here as
   * the last two bits (14&15) should be set so the Fuzzware doesn't use
   * this packet to synchronize its Master Clock.  Using bitwise OR's
   * instead of addition just to be safe when dealing with h_date which
   * is an unsigned short.
   */
  bzero((char *)&hi->h_date, sizeof(hi->h_date));
#ifdef notdef
  hi->h_date = htons((u_short)(((gmt->tm_year - 72) & 037) |
                     ((gmt->tm_mday & 037) << 5) |
                     (((gmt->tm_mon + 1) & 037) << 10) |
                     ((1 & 037) << 15)));
#else notdef
  hi->h_date = htons((u_short)0xC000);
#endif notdef
  /*
   * milliseconds since midnight UT of current day
   */
  hi->h_time = htonl((u_long)((gmt->tm_sec + gmt->tm_min * 60 + gmt->tm_hour
                        * 3600) * 1000 + tp.tv_usec / 1000));
  /*
   * 16 bit field used in rt calculation,  0 for ethernets
   */
  hi->h_tstp = 0;
  hi->h_cksum = 0;
  hi->h_cksum = hello_cksum((u_short *)hello, length);

  bzero((char *)&hellosaddr, sizeof (hellosaddr));
  hellosaddr.sin_family = AF_INET;

  /* find interface to send packet on */
  hellosaddr.sin_addr = src;
  ifp = if_withnet(&hellosaddr);
  if (ifp == NULL) {
    syslog(LOG_ERR, "hello_send: no interface for source address %s\n",
              inet_ntoa( src));
    error = TRUE;
  }
  else {
    extern int hello_socket;
    hellosaddr.sin_addr = dst;

    if (sendhello) {
      if (sendto(hello_socket, (char *)hello, length, 0,
                     &hellosaddr, sizeof(hellosaddr)) < 0) {
        TRACE_EXT("hello_send: sendto: %s %d bytes\n", inet_ntoa(dst), length);
	perror("hello_send: sendto:");
	syslog(LOG_ERR, "hello_send: sendto() error sending %d bytes to %s: %m",
               length, inet_ntoa(dst));
        error = TRUE;
      }
    }
  }
  if (!error) {
    TRACE_HELLOPKT(HELLO SENT, src, dst, (char *)hello, length, n_nets);
  } else {
    TRACE_HELLOPKT(HELLO *NOT* SENT, src, dst, (char *)hello, length, n_nets);
  }
  return;
}
