
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: if.c,v 1.3.1.17 88/01/09 18:58:40 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: if.c,v 1.3.1.17 88/01/09 18:58:40 jch Exp $";
#endif	not lint

/*
 * if.c
 *
 * Functions: if_withnet, if_check, if_print
 */

#include "include.h"

/*
 * Find the interface on the network of the specified address.
 */
struct interface *
if_withnet(withnetaddr)
	register struct sockaddr_in *withnetaddr;
{
  register struct interface *ifp;
  register u_long net1, tmp, net2;

  if (withnetaddr->sin_family != AF_INET)
    return (0);

  /* get network part of withnetaddr */
  tmp = ntohl(withnetaddr->sin_addr.s_addr);
  net1 = inet_wholenetof(withnetaddr->sin_addr);
  for (ifp = ifnet; ifp; ifp = ifp->int_next) {
    if ((ifp->int_netmask & net1) == ifp->int_net) {
      net1 = tmp & ifp->int_subnetmask;
      break;
    }
  }
  /* search for ifp */
  for (ifp = ifnet; ifp; ifp = ifp->int_next) {
    if (ifp->int_flags & IFF_POINTOPOINT) {
      tmp = ntohl(in_addr_ofs(&ifp->int_dstaddr).s_addr);
      net2 = inet_wholenetof(in_addr_ofs(&ifp->int_dstaddr));
      if ((ifp->int_netmask & net2) == ifp->int_net)
         net2 = tmp & ifp->int_subnetmask;
    }
    else {
      tmp = ntohl(in_addr_ofs(&ifp->int_addr).s_addr);
      net2 = inet_wholenetof(in_addr_ofs(&ifp->int_addr));
      if ((ifp->int_netmask & net2) == ifp->int_net)
        net2 = tmp & ifp->int_subnetmask;
    }
    if (net1 == net2)
      break;
  }
  return(ifp);
}

#ifdef notdef
/* used for DEBUGing */
if_print()
{
  register struct interface *ifp;

  for (ifp = ifnet; ifp; ifp = ifp->int_next)
    printf("interface %s int_addr %s\n", ifp->int_name,
               inet_ntoa(sock_inaddr(&ifp->int_addr)));
}
#endif

/*
 * if_check() checks the current status of all interfaces
 * If any interface has changed status, the the interface values
 * are re-read from the kernel and re-set.
 */

if_check()
{
  register struct interface *ifp;
  struct ifreq ifrequest;
  int  if_change = FALSE;
  struct sockaddr_in *sin;
  u_long a;
  int info_sock;

  if ((info_sock = getsocket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    p_error("if_check: no socket ");
    return;
  }
  for (ifp = ifnet; ifp != NULL; ifp = ifp->int_next) {
    /* get interface status flags */
    (void) strcpy(ifrequest.ifr_name, ifp->int_name);
    if (ioctl(info_sock, SIOCGIFFLAGS, (char *)&ifrequest))
      p_error("if_check: ioctl SIOCGIFFLAGS");
    else {
      if ((ifrequest.ifr_flags & IFF_UP) != (ifp->int_flags & IFF_UP)) {
        if_change = TRUE;
        if (ifrequest.ifr_flags & IFF_UP) {
          ifp->int_flags = IFF_INTERFACE | ifrequest.ifr_flags;

#if BSD43 || BETA_BSD43 || UTX32_2_X || RT43
          (void) strcpy(ifrequest.ifr_name, ifp->int_name);
          if (ioctl(info_sock, SIOCGIFMETRIC, (char *)&ifrequest) < 0)
            p_error("if_check: ioctl SIOCGIFMETRIC:\n");
          else
            ifp->int_metric = (ifrequest.ifr_metric >= 0) ?
                                 ifrequest.ifr_metric : 0;
#else
          ifp->int_metric =  0;
#endif
          if (ifp->int_flags & IFF_POINTOPOINT) {
            (void) strcpy(ifrequest.ifr_name, ifp->int_name);
            if (ioctl(info_sock, SIOCGIFDSTADDR, (char *)&ifrequest) < 0)
              p_error("if_check: ioctl SIOCGIFDSTADDR:\n");
            else
              ifp->int_dstaddr = ifrequest.ifr_dstaddr;
          }
          (void) strcpy(ifrequest.ifr_name, ifp->int_name);
          if (ioctl(info_sock, SIOCGIFADDR, (char *)&ifrequest) < 0)
            p_error("if_check: ioctl SIOCGIFADDR:\n");
          else
            ifp->int_addr = ifrequest.ifr_addr;
          if (ifp->int_flags & IFF_BROADCAST) {
#ifdef SIOCGIFBRDADDR
            (void) strcpy(ifrequest.ifr_name, ifp->int_name);
            if (ioctl(info_sock, SIOCGIFBRDADDR, (char *)&ifrequest) < 0)
              p_error("if_check: ioctl SIOCGIFBRDADDR:\n");
            else
#ifdef SUN3_3PLUS
              ifp->int_broadaddr = ifrequest.ifr_addr;
#else
              ifp->int_broadaddr = ifrequest.ifr_broadaddr;
#endif
#else !SIOCGIFBRDADDR
            ifp->int_broadaddr = ifp->int_addr;
            sin = (struct sockaddr_in *)&ifp->int_addr;
            a = ntohl(sin->sin_addr.s_addr);
            sin = (struct sockaddr_in *)&ifp->int_broadaddr;
            if (IN_CLASSA(a))
              sin->sin_addr.s_addr = htonl(a & IN_CLASSA_NET);
            else if (IN_CLASSB(a))
              sin->sin_addr.s_addr = htonl(a & IN_CLASSB_NET);
            else
              sin->sin_addr.s_addr = htonl(a & IN_CLASSC_NET);
#endif SIOCGIFBRDADDR
          }
          (void) strcpy(ifrequest.ifr_name, ifp->int_name);
#ifdef SIOCGIFNETMASK
          if (ioctl(info_sock, SIOCGIFNETMASK, (char *)&ifrequest) < 0)
            p_error("if_check: ioctl SIOCGIFNETMASK:\n");
          else {
            sin = (struct sockaddr_in *)&ifrequest.ifr_addr;
            ifp->int_subnetmask = ntohl(sin->sin_addr.s_addr);
            sin = (struct sockaddr_in *)&ifp->int_addr;
            a = ntohl(sin->sin_addr.s_addr);
            if (IN_CLASSA(a))
              ifp->int_netmask = IN_CLASSA_NET;
            else if (IN_CLASSB(a))
              ifp->int_netmask = IN_CLASSB_NET;
            else
              ifp->int_netmask = IN_CLASSC_NET;
            ifp->int_net = a & ifp->int_netmask;
            ifp->int_subnet = a & ifp->int_subnetmask;
          }
#else !SIOCGIFNETMASK
          sin = (struct sockaddr_in *)&ifp->int_addr;
          a = ntohl(sin->sin_addr.s_addr);
          if (IN_CLASSA(a))
            ifp->int_subnetmask = IN_CLASSA_NET;
          else if (IN_CLASSB(a))
            ifp->int_subnetmask = IN_CLASSB_NET;
          else
            ifp->int_subnetmask = IN_CLASSC_NET;
          sin = (struct sockaddr_in *)&ifp->int_addr;
          a = ntohl(sin->sin_addr.s_addr);
          if (IN_CLASSA(a))
            ifp->int_netmask = IN_CLASSA_NET;
          else if (IN_CLASSB(a))
            ifp->int_netmask = IN_CLASSB_NET;
          else
            ifp->int_netmask = IN_CLASSC_NET;
          ifp->int_net = a & ifp->int_netmask;
          ifp->int_subnet = a & ifp->int_subnetmask;
#endif SIOCGIFNETMASK
          syslog(LOG_NOTICE, "interface %s now up\n", ifp->int_name);
          TRACE_INT("if_check: Updated info for interface %s\n",
                                ifp->int_name);
          TRACE_INT("int_addr %s Hex: %x\nint_metric %d\n",
                      inet_ntoa(sock_inaddr(&ifp->int_addr)),
                      ntohl(sock_inaddr(&ifp->int_addr).s_addr),
                      ifp->int_metric);
	  if (ifp->int_flags & IFF_BROADCAST)
            TRACE_INT("int_broadaddr %s Hex: %x\n",
                       inet_ntoa(sock_inaddr(&ifp->int_broadaddr)),
                       ntohl(sock_inaddr(&ifp->int_broadaddr).s_addr));
	  if (ifp->int_flags & IFF_POINTOPOINT)
            TRACE_INT("int_dstaddr %s Hex: %x\n",
                       inet_ntoa(sock_inaddr(&ifp->int_dstaddr)),
                       ntohl(sock_inaddr(&ifp->int_dstaddr).s_addr));
          TRACE_INT("int_netmask %x\nint_subnetmask %x\nint_net %x\n",
                      ifp->int_netmask, ifp->int_subnetmask, ifp->int_net);
          TRACE_INT("int_subnet %x\n", ifp->int_subnet);
        }
        else {
          syslog(LOG_NOTICE, "interface %s now down\n", ifp->int_name);
          ifp->int_flags = IFF_INTERFACE | ifrequest.ifr_flags;
        }
      }
    }
  }
  if (if_change) {
    register struct rt_entry *rt;
    register struct rthash *rh;

    for (rh = nethash; rh < &nethash[ROUTEHASHSIZ]; rh++) 
      for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
        if ((rt->rt_state & RTS_INTERIOR) == 0)
          continue;
        if (rt->rt_ifp->int_flags & IFF_UP)
          rt->rt_flags |= RTF_UP;
        else
          rt->rt_flags &= ~RTF_UP;
      }
  }
  (void) close(info_sock);
}

/*
 * Find the interface with address addr.
 */

struct interface *
if_ifwithaddr(withaddraddr)
	struct sockaddr *withaddraddr;
{
  register struct interface *ifp;

#define	same(a1, a2) \
	(bcmp((caddr_t)((a1)->sa_data), (caddr_t)((a2)->sa_data), 14) == 0)

  for (ifp = ifnet; ifp; ifp = ifp->int_next) {
    if (ifp->int_flags & IFF_REMOTE)
      continue;
    if (ifp->int_addr.sa_family != withaddraddr->sa_family)
      continue;
    if (ifp->int_flags & IFF_POINTOPOINT) {
      if (same(&ifp->int_dstaddr, withaddraddr))
        break;
      else
        continue;
    }
    if (same(&ifp->int_addr, withaddraddr))
      break;
    if ((ifp->int_flags & IFF_BROADCAST) &&
        same(&ifp->int_broadaddr, withaddraddr))
      break;
  }
  return (ifp);
}

/*
 * update the active gw list on argument interface.
 */

if_updateactivegw(ifptr, actgw_addr, gw_proto)
	struct interface *ifptr;
	u_long actgw_addr;
	int gw_proto;
{
  struct active_gw *agp, *tmpactgw;
  int found_gw = 0;

  for (agp = ifptr->int_active_gw; agp; agp = agp->next) {
    if (actgw_addr == agp->addr) {
      found_gw++;
      break;
    }
    if (agp->next == NULL)
      break;
  }
  if (found_gw != 0) {
    agp->timer = 0;
    agp->proto |= gw_proto;
    return(agp->proto);
  }
  /*
   * this active gateway wasn't recorded yet!  Add it.
   * we have agp pointing to the last element, so no need to
   * traverse again.
   */
  tmpactgw = (struct active_gw *)malloc((unsigned)sizeof(struct active_gw));
  if (tmpactgw <= (struct active_gw *)0) {
    syslog(LOG_WARNING, "if_updateactivegw: out of memory");
    return(0);
  }
  tmpactgw->proto = gw_proto;
  tmpactgw->addr = actgw_addr;
  tmpactgw->timer = 0;
  if (agp == NULL) {		/* first one */
    ifptr->int_active_gw = tmpactgw;
    tmpactgw->back = ifptr->int_active_gw;
  }
  else {
    agp->next = tmpactgw;
    tmpactgw->back = agp;
  }
  tmpactgw->next = NULL;
  tmpactgw = NULL;		/* just to be safe */
  return (gw_proto);
}
