
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: init.c,v 1.3.1.17 88/01/09 19:03:29 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: init.c,v 1.3.1.17 88/01/09 19:03:29 jch Exp $";
#endif	not lint

/*
 * init.c
 *
 * Functions: init_if, init_sock, getsocket, init_egpngh, getnetorhostname,
 *	      init_egp, init_egp2, rip_init, get_rip_socket
 */
/* Initialization routines */


#include "include.h"

extern u_long inet_isnetwork();
extern u_long inet_addr();

/* init_if() determines addresses and names of internet interfaces configured.
 * Results returned in global linked list of interface tables.
 * The interface names are required for later ioctl calls re interface status.
 *
 * External variables:
 * ifnet - pointer to interface list
 * n_interfaces - number of direct internet interfaces (set here) 
 */ 

init_if()
{
  int	i, j, init_ifs, n_intf;
  struct in_addr initifaddr;
  int	bufsize;
  struct ifreq  *ifreq_buf;	/* buffer for receiving interace
				 * configuration info from ioctl */
  struct ifconf ifconf_req;
  struct interface *ifp;
  struct ifreq if_info;
  u_long a;
  struct sockaddr_in *sin;

/*
 * Determine internet addresses of all internet interfaces, except
 * ignore loopback interface.
 */

/*
 * First must open a temporary socket to get a valid socket descriptor for
 * ioctl call.
 */

  if ((init_ifs = getsocket(AF_INET, SOCK_DGRAM, 0)) < 0)
    quit();

  /*
   * get interface configuration.
   * allocate buffer for config. if
   * insufficient double size
   */
  bufsize = 10 * sizeof(struct ifreq) + 1; /* ioctl assumes > size ifreq */
  for (;;) {
    ifreq_buf = (struct ifreq *)malloc((unsigned)bufsize);
    if (ifreq_buf == 0) {
      syslog(LOG_ERR, "init_if: malloc: out of memory\n");
      quit();
    }
    ifconf_req.ifc_len = bufsize; /*sizeof(ifreq_buf);*/
    ifconf_req.ifc_req = ifreq_buf;

    if (ioctl(init_ifs, SIOCGIFCONF, (char *)&ifconf_req) ) {
      p_error("init_if: ioctl SIOCGIFCONF:\n");
      quit();
    }
    /*
     * if spare buffer space for at least
     * one more interface all found
     */
    if ((bufsize - ifconf_req.ifc_len) > sizeof (struct ifreq)) 
      break;
    /* else double buffer size and retry*/
    free((char *)ifreq_buf);
    if (bufsize > 40 * sizeof(struct ifreq)) {
      syslog(LOG_ERR, "init_if: more than 39 interfaces\n");
      quit();
    }
    bufsize <<= 1;
  }
  n_intf = ifconf_req.ifc_len/sizeof(struct ifreq); /* number of interfaces */
  j = 0;	/* internet interface index */
  for (i = 0; i < n_intf; i++) {
    if (ifreq_buf[i].ifr_addr.sa_family != AF_INET)
      continue;
    initifaddr.s_addr = ((struct sockaddr_in *)&ifreq_buf[i].ifr_addr)
						       ->sin_addr.s_addr;
    TRACE_INT("init_if: interface %s net 0x%x\n", 
                      ifreq_buf[i].ifr_name, gd_inet_netof(initifaddr));
    if ((unsigned)gd_inet_netof(initifaddr) == (unsigned)LOOPBACKNET)
      continue;	/* ignore loopback interface */
    ifp = (struct interface *)malloc(sizeof (struct interface));
    if (ifp == 0) {
      syslog(LOG_ERR, "init_if: malloc: out of memory\n");
      quit();
    }
    /* save name for future ioctl calls */
    ifp->int_name = malloc((unsigned)strlen(ifreq_buf[i].ifr_name) + 1);
    if (ifp->int_name == 0) {
      syslog(LOG_ERR, "init_if: malloc: out of memory\n");
      quit();
    }
    /* Get interface flags */
    (void) strcpy(if_info.ifr_name, ifreq_buf[i].ifr_name);
    if (ioctl(init_ifs, SIOCGIFFLAGS, (char *)&if_info)) {
      p_error("init_if:ioctl SIOCGIFFLAGS:\n");
      quit();
    }
    ifp->int_flags = IFF_INTERFACE | if_info.ifr_flags;
    /*
     * if interface is marked down, we will include it and try again
     * later.
     */
    if (ifp->int_flags & IFF_UP == 0) {
      ifp->int_metric =  0;
      if (ifp->int_flags & IFF_POINTOPOINT)
        bzero((char *)&ifp->int_dstaddr, sizeof(ifp->int_dstaddr));
      else
        bzero((char *)&ifp->int_addr, sizeof(ifp->int_addr));
      if (ifp->int_flags & IFF_BROADCAST)
        bzero((char *)&ifp->int_broadaddr, sizeof(ifp->int_broadaddr));
      ifp->int_subnetmask = ntohl((u_long)0);
      ifp->int_netmask = ntohl((u_long)0);
      ifp->int_subnet = ntohl((u_long)0);
      ifp->int_net = ntohl((u_long)0);
      ifp->int_active_gw = NULL;
      ifp->int_reconstmetric = -1;
    }
    else {
      /* get interface metric */
      (void) strcpy(if_info.ifr_name, ifreq_buf[i].ifr_name);
#if BSD43 || BETA_BSD43 || UTX32_2_X || RT43
      if (ioctl(init_ifs, SIOCGIFMETRIC, (char *)&if_info) < 0) {
        p_error("init_if: ioctl SIOCGIFMETRIC:\n");
        quit();
      }
      ifp->int_metric = (if_info.ifr_metric >= 0) ? if_info.ifr_metric : 0;
#else
      ifp->int_metric =  0;
#endif
      if (ifp->int_flags & IFF_POINTOPOINT) {
        if (ioctl(init_ifs, SIOCGIFDSTADDR, (char *)&if_info) < 0) {
          p_error("ioctl (get dstaddr):");
          quit();
        }
        ifp->int_dstaddr = if_info.ifr_dstaddr;
      }
      if (ifp->int_flags & IFF_BROADCAST) {
#ifdef SIOCGIFBRDADDR
        if (ioctl(init_ifs, SIOCGIFBRDADDR, (char *)&if_info) < 0) {
          p_error("ioctl (get broadaddr):");
          quit();
        }
#ifdef SUN3_3PLUS
        ifp->int_broadaddr = if_info.ifr_addr;
#else
        ifp->int_broadaddr = if_info.ifr_broadaddr;
#endif
#else !SIOCGIFBRDADDR
        /*
         *  Assume 4.2BSD with INADDR_ANY == 0
         */
        ifp->int_broadaddr = ifreq_buf[i].ifr_addr;
        sin = (struct sockaddr_in *)&ifreq_buf[i].ifr_addr;
        a = ntohl(sin->sin_addr.s_addr);
        sin = (struct sockaddr_in *)&ifp->int_broadaddr;
        if (IN_CLASSA(a))
          sin->sin_addr.s_addr = htonl(a & IN_CLASSA_NET);
        else if (IN_CLASSB(a))
          sin->sin_addr.s_addr = htonl(a & IN_CLASSB_NET);
        else
          sin->sin_addr.s_addr = htonl(a & IN_CLASSC_NET);
#endif SIOCGIFBRDADDR
      }
      /* get netmask */
      (void) strcpy(ifp->int_name, ifreq_buf[i].ifr_name);
      ifp->int_addr = ifreq_buf[i].ifr_addr;
#ifdef SIOCGIFNETMASK 
      if (ioctl(init_ifs, SIOCGIFNETMASK, (char *)&ifreq_buf[i]) < 0) {
        perror("ioctl (get netmask)");
        continue;
      }
      sin = (struct sockaddr_in *)&ifreq_buf[i].ifr_addr;
      ifp->int_subnetmask = ntohl(sin->sin_addr.s_addr);
#else !SIOCGIFNETMASK
      sin = (struct sockaddr_in *)&ifp->int_addr;
      a = ntohl(sin->sin_addr.s_addr);
      if (IN_CLASSA(a))
        ifp->int_subnetmask = IN_CLASSA_NET;
      else if (IN_CLASSB(a))
        ifp->int_subnetmask = IN_CLASSB_NET;
      else
        ifp->int_subnetmask = IN_CLASSC_NET;
#endif SIOCGIFNETMASK
      sin = (struct sockaddr_in *)&ifp->int_addr;
      a = ntohl(sin->sin_addr.s_addr);
      if (IN_CLASSA(a))
        ifp->int_netmask = IN_CLASSA_NET;
      else if (IN_CLASSB(a))
        ifp->int_netmask = IN_CLASSB_NET;
      else
        ifp->int_netmask = IN_CLASSC_NET;
      if (ifp->int_subnetmask == 0)
	  ifp->int_subnetmask = ifp->int_netmask;
      ifp->int_net = a & ifp->int_netmask;
      ifp->int_subnet = a & ifp->int_subnetmask;
      if (ifp->int_subnetmask != ifp->int_netmask)
        ifp->int_flags |= IFF_SUBNET;
      ifp->int_active_gw = NULL;
      ifp->int_reconstmetric = -1;
    }
		
    ifp->int_next = ifnet;
    ifnet = ifp;

    TRACE_INT("init_if: interface name: %s, address %s, metric %d\n", 
                    ifp->int_name, inet_ntoa(initifaddr),
                    ifp->int_metric);
    if (doing_rip && (ifp->int_flags & IFF_POINTOPOINT) && (rip_supplier < 0)) {
      rip_supplier = 1;
      TRACE_INT("init_if: PointoPoint rip supplier to: %s\n", ifp->int_name);
    }
    /*
     * if multiple interfaces with same net number exit as the routing tables
     * assume at most one interface per net.
     * NOT TRUE!  Point-to-point`s are OK.
     */
    for (ifp = ifnet->int_next; ifp != 0; ifp = ifp->int_next)
       if ((ifnet->int_subnet == ifp->int_subnet) &&
           ((ifnet->int_flags|ifp->int_flags) & IFF_POINTOPOINT) == 0) {
         syslog(LOG_ERR, "init_if: Exit as multiple non PTP");
         syslog(LOG_ERR, "interfaces to net of interface %s\n",
                 inet_ntoa(initifaddr));
         quit();
       }
    j++;
  }
  n_interfaces = j;	/* number internet interfaces */
  if (n_interfaces == 0) {
    syslog(LOG_ERR, "if_init: Exit no interfaces\n");
    quit();
  }
  if (doing_rip && (n_interfaces > 1) && (rip_supplier < 0)) {
    rip_supplier = 1;
    syslog(LOG_NOTICE, "if_init: Acting as RIP supplier to our direct nets\n");
  }
  free((char *)ifreq_buf);
  (void) close(init_ifs);
}


/*
 * getsocket gets a socket, retries later if no buffers at present
 */

getsocket(domain, type, protocol)
	int domain, type, protocol;
{
  int retry, getsocks;

  retry = 2;		/* if no buffers a retry might work */
  while ((getsocks = socket(domain, type, protocol)) < 0 && retry--) {
    p_error("getsocket:socket");
    sleep(5);
  }
  if (getsocks < 0)
    return (-1);
  return (getsocks);
}

/*
 * init_egpngh() reads autonomous system number and names of EGP neighbors 
 * from EGPINITFILE .
 * The format is: # comment
 *		  autonomoussystem value
 *		  egpneighbor name
 *		  egpmaxacquire value
 * where name is either a symbolic name in /etc/hosts or an internet address
 * in dot notation and value is the maximum number of EGP neighbors to acquire.
 * All EGP neighbors must be on the same net.
 * A state table is allocated for the neighbor and added to a linked list
 * terminated by a NULL pointer.
 *
 * External variables:
 *   mysystem - autonomous sytem number
 *   maxacq - maximum number of EGP neighbors to acquire
 *   nneigh - # egp neighbors
 *   egpngh - pointer to start of linked list of egp neighbor state tables.
 */

init_egpngh(fp)
	FILE *fp;
{
  struct sockaddr_in  egpnghaddr;
  char name[MAXHOSTNAMELENGTH+1];
  int	error = FALSE, c;
  struct egpngh	*ngp = NULL, *egpngh_last = NULL;

  nneigh = 0;
  egpngh = NULL;
  bzero((char *)&egpnghaddr, sizeof(egpnghaddr));

  rewind(fp);
  while(fscanf(fp, "%s", name) != EOF) {	/* read first word of line and
						   compare to key words */
    if (strcmp(name, "autonomoussystem") == 0) {
#if UTX32_1_X || UTX32_2_X || sun || RT43
      char sysnum[MAXHOSTNAMELENGTH+1];

      if (fscanf(fp, "%s", sysnum) != 1)
        error = TRUE;
      else {
        mysystem = atoi(sysnum);
        TRACE_INT("init_egpngh: autonomoussystem = %d\n", mysystem);
      }
#else
      if (fscanf(fp, " %d", &mysystem) != 1)
        error = TRUE;
      else
        TRACE_INT("init_egpngh: autonomoussystem = %d\n", mysystem);
#endif
    }
    else if (strcmp(name, "egpmaxacquire") == 0) {
#if UTX32_1_X || UTX32_2_X || sun || RT43
      char maxnum[MAXHOSTNAMELENGTH+1];

      if (fscanf(fp, "%s", maxnum) != 1)
        error = TRUE;
      else {
        maxacq = atoi(maxnum);
        TRACE_INT("init_egpngh: egpmaxacquire = %d\n", maxacq);
      }
#else
      if (fscanf(fp, " %d", &maxacq) != 1)
        error = TRUE;
      else
        TRACE_INT("init_egpngh: egpmaxacquire = %d\n", maxacq);
#endif
    }
    else if (strcmp(name, "egpneighbor") == 0) {
      if (fscanf(fp, "%s", name) != 1)
        error = TRUE;
      else if (getnetorhostname("host", name, &egpnghaddr)) {
        ngp = (struct egpngh *)malloc( sizeof(struct egpngh));
        if (ngp == NULL) {
          syslog(LOG_ERR, "init_egpngh: malloc: out of memory\n");
          quit();
        }
	/*
         * add to end of egp neighbor table linked
         * list so order same as in EGPINITFILE,
         * although simpler to add to front.
         */
        if (egpngh == NULL)
          egpngh = ngp;   /* first neighbor */
        else
          egpngh_last->ng_next = ngp;
        egpngh_last = ngp;
        ngp->ng_next = NULL;
        ngp->ng_addr = egpnghaddr.sin_addr;
        nneigh++;
        TRACE_INT("init_egpngh: EGP neighbor %s\n",
                       inet_ntoa(egpnghaddr.sin_addr));
#if SAMENET
        if (nneigh > 1) {
          if (gd_inet_netof(ngp->ng_addr) != gd_inet_netof(egpngh->ng_addr)) {
            syslog(LOG_ERR, "init_egpngh: EGP neighbors on");
            syslog(LOG_ERR, " different nets\n");
            error = TRUE;
          }
        }
#endif
      }       /* end else if getnetorhostname... */
      else {
        syslog(LOG_ERR, "init_egpngh: invalid neighbor name or");
        syslog(LOG_ERR, " address %s\n", name);
        error = TRUE;
      }
    }    /* end else egpneighbor */
    do c = fgetc(fp); while (c != '\n' && c != EOF); /* next line */
  }			/* end while */
  if (mysystem == 0) {
    syslog(LOG_ERR, "init_egpngh: autonomous system # 0\n");
    error = TRUE;
  }
  if ((maxacq == 0) && (nneigh > 0)) {
    syslog(LOG_ERR, "init_egpngh: egpmaxacquire = 0\n");
    error = TRUE;
  }
  if (maxacq > nneigh) {
    syslog(LOG_ERR, "init_egpngh: egpmaxacquire = %d > # neighbors = %d\n",
                         maxacq, nneigh);
    error = TRUE;
  }
  if (nneigh == 0) {
    syslog(LOG_ERR, "init_egpngh: no EGP neighbors.\n");
    error = TRUE;
  }
  if (error) {
    syslog(LOG_ERR, "init_egpngh: %s: initialization error\n", EGPINITFILE);
    quit();
  }
}

/*
 * Given host or net name or internet address in dot notation assign the
 * internet address in byte format.
 * source is ../routed/startup.c with minor changes to detect syntax errors.
 *
 * Unfortunately the library routine inet_addr() does not detect mal formed
 * addresses that have characters or byte values > 255.
 */

getnetorhostname(type, name, sin)
	char *type, *name;
	struct sockaddr_in *sin;
{
  if (strcmp(type, "net") == 0) {
    struct netent *np;
    u_long n;

#if UTX32_2_X || UTX32_1_X
    if ((n = inet_isnetwork(name)) == (u_long) -1) {
#else
    if ((n = inet_isnetwork(name)) == -1) {
#endif
      if (np = getnetbyname(name)) {
        if (np->n_addrtype != AF_INET)
          return (0);
        n = np->n_net;
      }
      else
        return(0);
    }
    if (n < 128)
      n <<= IN_CLASSA_NSHIFT;
    else if (n < 65536)
      n <<= IN_CLASSB_NSHIFT;
    else
      n <<= IN_CLASSC_NSHIFT;
    sin->sin_family = AF_INET;
    sin->sin_addr = gd_inet_makeaddr((int)n, 0, FALSE);
    return (1);
  }				
  if (strcmp(type, "host") == 0) {
    struct hostent *hp;

#if UTX32_2_X || UTX32_1_X
    if ((sin->sin_addr.s_addr = inet_addr(name)) == (u_long) -1) {
#else
    if ((sin->sin_addr.s_addr = inet_addr(name)) == -1) {
#endif
      if (hp = gethostbyname(name)) {
        bcopy(hp->h_addr, (char *)&sin->sin_addr, hp->h_length);
        sin->sin_family = hp->h_addrtype;
        return (1);
      }
      return(0);
    }
    sin->sin_addr.s_addr = inet_addr(name);
    sin->sin_family = AF_INET;
    return (1);
  }
  return (0);
}

/*
 * init_egp() initializes the state tables for each potential EGP neighbor
 */

init_egp()
{
  reg	egpng		*ngp;
  struct	interface	*ifp;
  struct	sockaddr_in	dst;

  egpsleep = MINHELLOINT + HELLOMARGIN;
  rt_maxage = RT_MINAGE;		/* minimum time before routes are
                                           deleted when not updated */
  for (ngp = egpngh; ngp != NULL; ngp = ngp->ng_next) {
    ngp->ng_state = UNACQUIRED;
    ngp->ng_flags = 0;
    ngp->ng_sid = 1;
    ngp->ng_retry = 0;
    ngp->ng_hint = MINHELLOINT + HELLOMARGIN;
    ngp->ng_htime = gatedtime;
    ngp->ng_rcmd = 0;

    /* Check that I have a direct net to neighbor */
    dst.sin_family = AF_INET;
    dst.sin_addr = ngp->ng_addr;
    if ((ifp = if_withnet(&dst) ) != NULL)
      ngp->ng_myaddr = sock_inaddr(&ifp->int_addr);
    else {
      TRACE_INT("init_egp: no direct net to neighbor %s\n",
                             inet_ntoa(ngp->ng_addr));
      if (ALLOWNONNEIGHBORS)
        ngp->ng_state = EMPTY;
      else
        quit();
    }
  }

/* Debug */
#if ALLOWNONNEIGHBORS
  init_egp2();
#endif
}

/*
 * init_egp2() is for testing only, allows peers which do not share a direct 
 * net. The source interface is defined by DEFAULTIF in defs.h
 */
#if ALLOWNONNEIGHBORS

init_egp2()
{
  reg	egpng	*ngp;

  TRACE_INT("init_egp2 called\n");
  for (ngp = egpngh; ngp != NULL; ngp = ngp->ng_next) {
    if (ngp->ng_state != EMPTY)
      continue;
    ngp->ng_state = UNACQUIRED;
    ngp->ng_myaddr.s_addr = inet_addr(DEFAULTIF);
  }
}
#endif

/*
 * initialize all the RIP stuff...
 */

rip_init()
{
  int ripinits;

  sp = getservbyname("router", "udp");
  if (sp == NULL) {
    syslog(LOG_ERR,"No service for router available\n");
    exit(1);
  }
  addr.sin_family = AF_INET;
  addr.sin_port = sp->s_port;
  ripinits = get_rip_socket(AF_INET, SOCK_DGRAM, &addr);
  if (ripinits < 0)
    exit(1);
  return(ripinits);
}

get_rip_socket(domain, type, sin)
	int domain, type;
	struct sockaddr_in *sin;
{
  int ripsocks, on = 1;

  if ((ripsocks = socket(domain, type, 0)) < 0) {
    p_error("get_rip_socket: socket");
    return (-1);
  }
#ifdef SO_BROADCAST
  if (setsockopt(ripsocks, SOL_SOCKET, SO_BROADCAST, (char *)&on, sizeof (on))
                        < 0) {
    (void) close(ripsocks);
    return (-1);
  }
#endif SO_BROADCAST
#ifdef SO_RCVBUF
  on = 48*1024;
  if (setsockopt(ripsocks, SOL_SOCKET, SO_RCVBUF, (char *)&on, sizeof (on))
                        < 0) {
    syslog(LOG_ERR, "setsockopt SO_RCVBUF: %m");
  }
#endif SO_RCVBUF
  if (bind(ripsocks, sin, sizeof (*sin)) < 0) {
    p_error("get_rip_socket: bind");
    (void) close(ripsocks);
    return (-1);
  }
  return (ripsocks);
}
