/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 */

#ifndef	lint
static char *rcsid = "$Header: main.c,v 1.3.1.21 88/02/11 21:52:00 jch Exp $";
#endif	not lint

/* main.c
 *
 * Main function of the EGP user process. Includes packet reception,
 * ICMP handler and timer interrupt handler.
 *
 * Functions: main, recvpkt, icmpin, timeout, getod, quit, p_error
 */
/*
 * Software Overview:
 *
 * At start up, the controlling function, main(), first sets tracing options
 * based on starting command arguments. It then calls the initialization
 * functions described in the next section, sets up signal handlers for
 * termination (SIGTERM) and timer interrupt (SIGALRM) signals, calls
 * timeout() to start periodic interrupt processing, and finally waits in a 
 * loop to receive incoming EGP or ICMP redirect packets
 *
 * When an EGP packet is received, egpin() is called to handle it. It in turn
 * calls a separate function for each EGP command type, which sends the
 * appropriate response. When an ICMP packet is received icmpin() is called.
 *
 * The timer interrupt routine, timeout(), calls egpjob() to perform periodic
 * EGP processing such as reachability determination, command sending and
 * retransmission. It in turn calls separate functions to format each command
 * type. Timeout() also periodically calls rt_time() to increment route ages 
 * and delete routes when too old.
 */

#include "include.h"

struct rip *ripmsg = (struct rip *)rip_packet;
extern int setdumpflg();
extern int chgtrace();

main(argc, argv)
	int   argc;
	char *argv[];
{
  struct sigvec vec, ovec;
  int	selectbits,
        forever = TRUE,
        error = FALSE,
        arg_n = 0;
  char	*cp;
  FILE  *fp;

  getod();		/* start time */

  /* check arguments for turning on tracing and a trace file */

  tracing = savetrace = are_tracing = 0;
  logfile = NULL;
  while (--argc > 0 && !error) {
    argv++;
    arg_n++;
    switch (arg_n) {
      case 1:
        cp = *argv;
        if (*cp++ != '-') {
          if (argc > 1)
            error = TRUE;
          else
            logfile = *argv;
        }
        else
          if (*cp++ != 't')
            error = TRUE;
          else
            if (*cp == '\0')
              savetrace = TR_INT | TR_EXT | TR_RT | TR_PKT;
            else {
              while (*cp != '\0')
                switch (*cp++) {
                  case 'i':
                    savetrace |= TR_INT;
                    break;
                  case 'e':
                    savetrace |= TR_EXT;
                    break;
                  case 'r':
                    savetrace |= TR_RT;
                    break;
                  case 'p':
                    savetrace |= TR_PKT;
                    break;
                  case 'u':
                    savetrace |= TR_UPDATE;
                    break;
                  case 'R':
                    savetrace |= TR_RIP;
                    break;
                  case 'H':
                    savetrace |= TR_HELLO;
                    break;
                  default:
                    error = TRUE;
                }
            }
        break;
      case 2:
        logfile = *argv;
        break;
      default:
        error = TRUE;
    }
  }
  if (error) {
    fprintf(stderr, "Usage: gated [-t[i][e][r][p][u][R][H]] [logfile]\n");
    exit(1);
  }
  if ((savetrace == 0) || (logfile != NULL)) {
    int t;

    if (fork()) {
      exit(0);
    }
    for (t = 0; t < 20; t++) {
      (void) close(t);
    }
    (void) open("/dev/null", O_RDONLY);
    (void) dup2(0, 1);
    (void) dup2(0, 2);
    t = open("/dev/tty", O_RDWR);
    if (t >= 0) {
      (void) ioctl(t, TIOCNOTTY, (char *)NULL);
      (void) close(t);
    }
  }

#if BSD43 || BETA_BSD43 || UTX32_1_X || UTX32_2_X || RT43
  openlog("gated", LOG_PID|LOG_CONS|LOG_NDELAY, LOG_FACILITY);
  (void) setlogmask(LOG_UPTO(LOG_NOTICE));
#else
  openlog("gated", LOG_PID);
#endif

  (void) traceon(logfile, GEN_TRACE);
  if (logfile != NULL) {
    TRACE_TRC("File \"%s\" logs trace and error messages for ", logfile);
    TRACE_TRC("the GATED user process, \"gated\".\n");
    TRACE_TRC("Tracing levels turned on: ");
    TRACE_INT("i ");
    TRACE_EXT("e ");
    TRACE_RT("r ");
    TRACE_PKT("p ");
    TRACE_UPD("u ");
    TRACE_RIP("R ");
    TRACE_HEL("H ");
    TRACE_TRC("\n\n");
  }
  if (tracing) {
    are_tracing++;
  }

  syslog(LOG_NOTICE, "Start gated version %s at %s\n", VERSION, strtime);
  TRACE_TRC("Start gated version %s at %s\n", VERSION, strtime);
		
  fp = fopen(EGPINITFILE, "r");	/* open initialization file */
  if (fp == NULL) {
    syslog(LOG_WARNING, "main: initialization file %s missing\n", EGPINITFILE);
    TRACE_TRC("main: initialization file %s missing\n", EGPINITFILE);
    quit();
  }
  setnetent(1);
  sethostent(1);
  rt_init();		/* initialize route hash tables */
  init_options(fp);	/* initialize protocol options */
  init_if();  		/* initialize interface tables */
  rt_resinit(fp);	/* initialize routing restriction tables. */
  rt_NRadvise_init(fp);	/* initialize interior routes to be EGP advised */

  /*
   * only need one socket per protocol type in 4.3bsd
   */
  icmp_socket = getsocket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
  if (doing_egp)
    egp_socket = getsocket(AF_INET, SOCK_RAW, IPPROTO_EGP);
  if (doing_hello)
    hello_socket = getsocket(AF_INET, SOCK_RAW, IPPROTO_HELLO);
  if (doing_rip)
    rip_socket = rip_init();

  s = icmp_socket;   	/* socket desc. for ioctl calls to install routes */
  if (doing_egp) {
    init_egpngh(fp);	/* read egp neighbor list */
    init_egp();		/* initialize EGP neighbor tables */
  }
  rt_readkernel();	/* Initailize routing tables with kernel's */
  rt_ifinit();		/* initialize interior routes for direct nets */
  if (rip_gateway || hello_gateway)
    rt_default("GATEWAY");

  if (doing_rip && rip_supplier <= 0) {
    rip_supplier = 0;
    syslog(LOG_NOTICE, "ONLY LISTENING to RIP information....\n");
    TRACE_INT("ONLY LISTENING to RIP information....\n");
  }
  if (doing_rip) {
    ripmsg->rip_cmd = RIPCMD_REQUEST;
    ripmsg->rip_vers = RIPVERSION;
    ripmsg->rip_nets[0].rip_dst.sa_family = AF_UNSPEC;
    ripmsg->rip_nets[0].rip_metric = RIPHOPCNT_INFINITY;
    ripmsg->rip_nets[0].rip_dst.sa_family = htons(AF_UNSPEC);
    ripmsg->rip_nets[0].rip_metric = htonl((u_long)RIPHOPCNT_INFINITY);
    toall(sendripmsg);
  }
  install = TRUE;	/* install routes from now on */

  if (install) {
    TRACE_TRC("\n***Routes are being installed in kernel\n\n");
  } else {
    TRACE_TRC("\n***Routes are not being installed in kernel\n\n");
  }

  rt_dumbinit(fp);	/* initialize static routes gateways */
  (void) fclose(fp);

  fp = fopen(PIDFILE, "w");
  if (fp != NULL) {
    fprintf(fp, "%d\n", getpid());
    (void) fclose(fp);
  }

  /* setup abort signal */
  vec.sv_handler = egpallcease;
  vec.sv_mask = SIGIO | SIGALRM;
  if (error = sigvec(SIGTERM, &vec, &ovec)) {
    p_error("main: sigvec SIGTERM");
    quit();
  }
  vec.sv_handler = timeout;
  vec.sv_mask = SIGIO | SIGTERM;
  if (error = sigvec(SIGALRM, &vec, &ovec)) {
    p_error("main: sigvec SIGALRM");
    quit();
  }
  (void) signal(SIGINT, setdumpflg);
  (void) signal(SIGHUP, chgtrace);

  TRACE_RT("main: commence EGP-HELLO-RIP route updates:\n\n");

  timeout();

  /* wait to receive HELLO, RIP, EGP, ICMP or IMP messages */

  selectbits = 0;			/* select descriptor mask */
  if (doing_egp)
    selectbits |= 1 << egp_socket;	/* EGP socket */
  selectbits |= 1 << icmp_socket;	/* ICMP socket */
  if (doing_hello)
    selectbits |= 1 << hello_socket;	/* HELLO socket */
  if (doing_rip)
    selectbits |= 1 << rip_socket; 	/* RIP socket */
	
  endnetent();
  endhostent();

  while (forever) {
    int ibits;
    register int n;

    ibits = selectbits;
    n = select(20, (struct fd_set *)&ibits, (struct fd_set *)0,
                          (struct fd_set *)0, (struct timeval *)0);

    if (n < 0) {
      if (errno != EINTR) {
        p_error("main: select");
        quit();
      }
      continue;
    }
    if (doing_egp && (ibits & (1 << egp_socket)))
      recvpkt(egp_socket, IPPROTO_EGP);
    if (ibits & (1 << icmp_socket))
      recvpkt(icmp_socket, IPPROTO_ICMP);
    if (doing_hello && (ibits & (1 << hello_socket)))
      recvpkt(hello_socket, IPPROTO_HELLO);
    if (doing_rip && (ibits & (1 << rip_socket)))
      recvpkt(rip_socket, IPPROTO_RIP);
  }
}

recvpkt(sock, protocol)
	int sock, protocol;
{
  char packet[MAXPACKETSIZE+1];  	/* packet buffer */
  struct sockaddr_in from;
  int fromlen = sizeof(from), count, omask;

  getod();			/* current time */
  count = recvfrom(sock, packet, sizeof(packet), 0, &from, &fromlen);
  if (count <= 0) {
    if (count < 0 && errno != EINTR)
      p_error("recvpkt:recvfrom");
    return;
  }
  if (fromlen != sizeof (struct sockaddr_in)) {
    syslog(LOG_ERR, "recvpkt: fromlen %d invalid\n\n", fromlen);
    return;
  }
  if (count > sizeof(packet)) {
    syslog(LOG_ERR, "recvfrom: packet discarded, length %d > %d",
                       count, sizeof(packet));
    syslog(LOG_ERR, ", from addr %s\n\n", inet_ntoa(from.sin_addr));
    return;
  }

#define	mask(s)	(1<<((s)-1))

  omask = sigblock(mask(SIGALRM | SIGTERM));

  switch (protocol) {
    case IPPROTO_EGP:
      if (tracing & TR_PKT) 
        tracercv(sock, protocol, packet, count, &from);
      egpin(packet);
      break;
    case IPPROTO_ICMP:
      icmpin(packet);
      break;
    case IPPROTO_HELLO:
      helloin(packet);
      break;
    case IPPROTO_RIP:
      ripin(&from, count, packet);
      break;
    case IMPLINK_IP:
      break;  		/* not implemented - need care with different
                         * address family for imp?
                         */
  }
  (void) sigsetmask(omask);
}

/*
 * icmpin() handles ICMP redirect messages.
 */

icmpin(icmp_buf)
	char	*icmp_buf;
{
  struct	icmp	*icmppkt;
  struct	sockaddr_in	gateway,	/* gateway address */
                                dest;		/* destination addr */
#if BSD43 || UTX32_2_X || RT43
  int hlen;
  struct sockaddr_in who_from;
  struct	ip *ip;

  /*
   * Strip off the IP header on the ICMP message.
   */
  ip = (struct ip *)icmp_buf;
  hlen = ip->ip_hl << 2;
  icmppkt = (struct icmp *)(icmp_buf + hlen);

  bzero( (char *)&who_from, sizeof( who_from));
  who_from.sin_family = AF_INET;
  who_from.sin_addr = ip->ip_src;
#else
  icmppkt = (struct icmp *)icmp_buf;
#endif
  /*
   * filter ICMP network redirects
   */
  if (icmppkt->icmp_type != ICMP_REDIRECT)
    return;
  else
    if ((icmppkt->icmp_code != ICMP_REDIRECT_NET) &&
        (icmppkt->icmp_code != ICMP_REDIRECT_HOST))
      return;
    else {
      bzero((char *)&gateway, sizeof(gateway));
      gateway.sin_family = AF_INET;
      gateway.sin_addr = icmppkt->icmp_gwaddr;
      bzero((char *)&dest, sizeof(dest));
      dest.sin_family = AF_INET;
      dest.sin_addr = icmppkt->icmp_ip.ip_dst;
      if (icmppkt->icmp_code == ICMP_REDIRECT_NET)
        dest.sin_addr = gd_inet_makeaddr(gd_inet_netof(dest.sin_addr),0,TRUE);
#if BSD43 || UTX32_2_X || RT43
      rt_redirect( &dest, &gateway, &who_from, icmppkt->icmp_code);
#else
      rt_redirect( &dest, &gateway, NULL, icmppkt->icmp_code);
#endif
    }
  return;
}

/*
 * timer control for periodic route-age and interface processing.
 * timeout() is called when the periodic interrupt timer expires.
 */

timeout()
{
  static unsigned long
      		next_egpjob = 0,
		next_rttime = 0,
		next_ripjob = 0,
		next_iftime = 0,
		next_hellojob = 0;
  register unsigned long interval;
  struct itimerval  value, ovalue;

  getod();
#ifdef debug
  printf("gatedtime: %u\n", gatedtime);
  printf("next_egpjob: %u\n", next_egpjob);
  printf("next_ripjob: %u\n", next_ripjob);
  printf("next_hellojob: %u\n", next_hellojob);
  printf("next_rttime: %u\n", next_rttime);
  printf("next_iftime: %u\n", next_iftime);
#endif  

  if (next_rttime == 0)	{		/* initialization */
    next_rttime = gatedtime + RT_TIMERRATE;
  }
  interval = gatedtime+1000;	/* some large value */

  if (doing_egp && gatedtime >= next_egpjob) {
    egpjob();
    next_egpjob = gatedtime + egpsleep;
  }
  if (doing_egp && interval > next_egpjob)
      interval = next_egpjob;
  if (doing_rip && gatedtime >= next_ripjob) {
    if (rip_supplier)
      toall(supply);
    next_ripjob = gatedtime + RIP_INTERVAL;
  }
  if ( doing_rip && interval > next_ripjob)
      interval = next_ripjob;
  if (doing_hello && gatedtime >= next_hellojob) {
    if (hello_supplier)
      hellojob();
    next_hellojob = gatedtime + HELLO_TIMERRATE;
  }
  if (doing_hello && interval > next_hellojob)
      interval = next_hellojob;
  if (gatedtime >= next_rttime) {
    rt_time();
    next_rttime = gatedtime + RT_TIMERRATE;
    if (sched_a_dump == 1) {
      dumpinfo();
      sched_a_dump = 0;
    }
  }
  if (interval > next_rttime)
      interval = next_rttime;
  if (gatedtime >= next_iftime) {
    if_check();
    next_iftime = gatedtime + CHECK_INTERVAL;
  }
  if (interval > next_iftime)
      interval = next_iftime;

#ifdef debug
  printf("gatedtime: %u\n", gatedtime);
  printf("interval %u\n", interval);
  printf("next_egpjob: %u\n", next_egpjob);
  printf("next_ripjob: %u\n", next_ripjob);
  printf("next_hellojob: %u\n", next_hellojob);
  printf("next_rttime: %u\n", next_rttime);
  printf("next_iftime: %u\n", next_iftime);
  printf("doing_egp %d, doing_hello %d, doing_rip %d\n",
	 doing_egp, doing_hello, doing_rip);
  printf("sleeping for %u seconds\n", interval-gatedtime);

#endif  
  interval -= gatedtime;
  value.it_interval.tv_sec = 0;		/* no auto timer reload */
  value.it_interval.tv_usec = 0;
  value.it_value.tv_sec = interval;
  value.it_value.tv_usec = 0;
  (void) setitimer(ITIMER_REAL, &value, &ovalue);
  (void) fflush(stdout);		/* periodically flush output */
  return;
}

/*
 * get time of day in seconds and as an ASCII string.
 * Called at each interrupt and saved in external variables.
 */

extern	unsigned long	gatedtime;
extern	char	*strtime;

getod()
{
  struct timeval tp;
  struct timezone tzp;

  if (gettimeofday (&tp, &tzp))
    p_error("getod: gettimeofday");
  gatedtime = tp.tv_sec;		/* time in seconds */
  strtime = ctime(&gatedtime);		/* time as an ASCII string */
  return;
}


/* exit gated */

quit()
{
  if (rt_default_status == NOTINSTALLED)
    rt_default("ADD");
  syslog(LOG_NOTICE, "Exit gated at %s\n\n", strtime);
  exit(1);
}

/*
 * Print error message to system logger.
 *
 * First flush stdout stream to preserve log order as perror writes directly
 * to file
 */

p_error(str)
	char *str;
{
  (void) fflush(stdout);
  syslog(LOG_ERR, "%s %m", str);
}

/*
 * set dump flag so the database may be dumped.
 */
int setdumpflg()
{
	sched_a_dump = 1;
}

/*
 * toggle the trace on/off on a HUP.
 */
int chgtrace()
{
  if (logfile == NULL) {
    syslog(LOG_ERR, "Error: can not toggle tracing to console");
    return;
  }
  if (are_tracing) {
    traceoff(GEN_TRACE);
    are_tracing = 0;
  } else {
    traceon(logfile, GEN_TRACE);
    are_tracing = 1;
  }
}
