
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: rip.c,v 1.3.1.17 88/01/09 18:59:27 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: rip.c,v 1.3.1.17 88/01/09 18:59:27 jch Exp $";
#endif	not lint

#include "include.h"

/*
 * 	Check out a newly received RIP packet.
 */

ripin(from, size, pkt)
	struct sockaddr *from;
	int size;
	char *pkt;
{
  register struct rt_entry *rt, *exrt;
  register struct netinfo *n;
  register struct afswitch *afp;
  register struct interface *ifp, *ifpc;
  struct rip *inripmsg = (struct rip *)pkt;
  int change = FALSE;
  int newsize, rte_table, fromproto;
  struct sockaddr_in *sin_from = (struct sockaddr_in *)from;
  u_short src_port = sin_from->sin_port;

  if (from->sa_family != AF_INET)
    return;

  /* not our version? */
  if (inripmsg->rip_vers != RIPVERSION)
    return;

  /* ignore if not from a trusted RIPer */
  if (trustedripperlist != (struct advlist *)NULL) {
    register struct advlist *ad;   /* list of trusted RIPpers */
    register int OK = 0;

    for (ad = trustedripperlist; ad; ad = ad->next)
      if (sin_from->sin_addr.s_addr == ad->destnet.s_addr) {
        OK++;
	break;
      }
    if (OK == 0)
      return;
  }
  afp = &afswitch[from->sa_family];
  switch (inripmsg->rip_cmd) {
    case RIPCMD_REQUEST:
      if (from->sa_family != AF_INET)  /* a little paranoid here */
        return;
      (*afp->af_canon)(from);
      if ((ifp = if_withnet(from)) <= (struct interface *)0) {
        syslog(LOG_ERR, "RIP-ERROR: invalid RIP request from %s\n",
                 inet_ntoa(sin_from->sin_addr));
        return;
      }
      if (ifp->int_flags & IFF_NORIPIN)
        return;
      if (rip_supplier <= 0)
        return;
      /*
       * are we talking to ourselves?
       */
      ifpc = if_ifwithaddr(from);
      if (ifpc)
        return;

      TRACE_RIPINPUT(ifp, from, size, (char *)inripmsg);
      if (src_port != sp->s_port) {
        sin_from->sin_port = src_port;
      } else {
        (void) if_updateactivegw(ifp, sin_from->sin_addr.s_addr, RTPROTO_RIP);
      }
      newsize = 0;
      size -= 4 * sizeof(char);
      n = inripmsg->rip_nets;
      while (size > 0) {
        if (size < sizeof(struct netinfo))
          break;
        size -= sizeof(struct netinfo);
        n->rip_dst.sa_family = ntohs(n->rip_dst.sa_family);
        n->rip_metric = ntohl((u_long)n->rip_metric);
        if (n->rip_dst.sa_family == AF_UNSPEC &&
            n->rip_metric == RIPHOPCNT_INFINITY &&
            size == 0) {
          supply(from, 0, ifp);
          return;
        }
        rt = rt_lookup((int)INTERIOR, &n->rip_dst);
        n->rip_metric = (rt == 0) ? RIPHOPCNT_INFINITY :
                        min(mapmetric(HELLO_TO_RIP, rt->rt_metric) + 1 +
                            ifp->int_metric, RIPHOPCNT_INFINITY);
        n++;
        newsize += sizeof(struct netinfo);
      }
      if (newsize > 0) {
        inripmsg->rip_cmd = RIPCMD_RESPONSE;
        newsize += sizeof(int);
        if (from->sa_family != AF_INET)
          return;
        (*afp->af_output)(rip_socket, 0, from, newsize);
      }
      return;
    case RIPCMD_TRACEON:
    case RIPCMD_TRACEOFF:
      if (from->sa_family != AF_INET)  /* a little paranoid here */
        return;
      /* verify message came from a router */
      if ((*afp->af_portcheck)(from) == 0)
        return;
      if ((ifp = if_withnet(from)) <= (struct interface *)0) {
        syslog(LOG_ERR, "RIP-ERROR: invalid RIP trace request from %s\n",
                 inet_ntoa(sin_from->sin_addr));
        return;
      }
      if (ifp->int_flags & IFF_NORIPIN)
        return;
      *(pkt + size) = '\0';
      if (inripmsg->rip_cmd == RIPCMD_TRACEON)
        traceon(inripmsg->rip_tracefile, RIP_TRACE);
      else
        traceoff(RIP_TRACE);
      return;
    case RIPCMD_RESPONSE:
      if (from->sa_family != AF_INET)  /* a little paranoid here */
        return;
      /* verify message came from a router */
      if ((*afp->af_portmatch)(from) == 0)
        return;
      (*afp->af_canon)(from);
      if ((ifp = if_withnet(from)) <= (struct interface *)0) {
        syslog(LOG_ERR, "RIP-ERROR: invalid RIP response from %s\n",
                 inet_ntoa(sin_from->sin_addr));
        return;
      }
      if (ifp->int_flags & IFF_NORIPIN)
        return;

      /*
       *  Are we talking to ourselves???
       *
       *  if_ifwithaddr() handles PTP's also.  If from a
       *  dst of a PTP link, let it through for further processing.
       *  you shouldn't receive your own RIPs on a PTP.
       */
      ifpc = if_ifwithaddr(from);
      if (ifpc) {
        rt = rt_find(from);
        if (rt == 0 || (rt->rt_state & RTS_INTERFACE) == 0)
          addrouteforif(ifpc);
        else
          rt->rt_timer = 0;
        if ((ifpc->int_flags & IFF_POINTOPOINT) == 0)
          return;
      }
      /*
       * update interface timer on interface that packet came in on.
       */
      if ((rt = rt_find(from)) && (rt->rt_state & (RTS_INTERFACE|RTS_REMOTE)))
        rt->rt_timer = 0;
      else  /* re-install route for interface */
        addrouteforif(ifp);

      TRACE_RIPINPUT(ifp, from, size, (char *)inripmsg);
      fromproto = if_updateactivegw(ifp,sin_from->sin_addr.s_addr,RTPROTO_RIP);
      size -= 4 * sizeof (char);
      n = inripmsg->rip_nets;
      for (; size > 0; size -= sizeof (struct netinfo), n++) {
        if (size < sizeof (struct netinfo))
          break;
        n->rip_dst.sa_family = ntohs(n->rip_dst.sa_family);
        n->rip_metric = ntohl((u_long)n->rip_metric);
        /*
         * if valid on this interface, reconstitute the metric.
         * I don't like this!
         */
        if ((n->rip_metric < RIPHOPCNT_INFINITY) &&
            (ifp->int_flags & IFF_RECONST))
          n->rip_metric = ifp->int_reconstmetric;
        /*
         * Now map rip metric to Time delay in millisec.'s
         */
        n->rip_metric = mapmetric(RIP_TO_HELLO,(n->rip_metric+ifp->int_metric));

        if (n->rip_dst.sa_family != AF_INET)
          continue;
        afp = &afswitch[n->rip_dst.sa_family];
        if (((*afp->af_checkhost)(&n->rip_dst)) == 0)
          continue;
        if ((*afp->af_ishost)(&n->rip_dst))
          rte_table = HOSTTABLE;
        else
          rte_table = INTERIOR;
        rt = rt_lookup(rte_table, &n->rip_dst);
        if (rt == NULL) {  /* new route */
          struct sockaddr_in *tmp = (struct sockaddr_in *)&n->rip_dst;
          struct rt_entry rttmp;

          bzero((char *)&rttmp, sizeof(rttmp));
          rttmp.rt_listenlist = control_lookup(RT_NOLISTEN, &n->rip_dst);
          rttmp.rt_srclisten = control_lookup(RT_SRCLISTEN, &n->rip_dst);
          if (is_valid_in(&rttmp, RTPROTO_RIP, ifp, from) == 0)
            continue;
          if (n->rip_metric >= DELAY_INFINITY)
            continue;
          if ((rte_table == INTERIOR) &&
              (exrt = rt_lookup((int)EXTERIOR, &n->rip_dst))) {
            struct sockaddr_in *ntmp = (struct sockaddr_in *)&n->rip_dst;

            TRACE_EXT("GATED-RIP WARNING: route to %s in exterior table\n",
                            inet_ntoa(ntmp->sin_addr));
            do {
              rt_delete(exrt, KERNEL_INTR);
            } while (exrt = rt_lookup((int)EXTERIOR, &n->rip_dst));
            TRACE_EXT("GATED-RIP: Route in exterior table deleted\n");
          }
          TRACE_EXT("RIP_ADD %s metric %d\n", inet_ntoa(tmp->sin_addr),
                               mapmetric(HELLO_TO_RIP, n->rip_metric));
          (void) rt_add(rte_table, &n->rip_dst, from, n->rip_metric, 0,
                        RTPROTO_RIP, fromproto);
          change = TRUE;
        }
        else {
          if ((rt->rt_flags & RTF_GATEWAY) == 0)
            continue;
          if (rt->rt_state & RTS_INTERFACE)
            continue;
          if (rt->rt_state & RTS_PASSIVE)
            continue;
          if (is_valid_in(rt, RTPROTO_RIP, ifp, from) == 0)
            continue;
          if ((rte_table == INTERIOR) &&
              (exrt = rt_lookup((int)EXTERIOR, &n->rip_dst))) {
            struct sockaddr_in *tmp = (struct sockaddr_in *)&n->rip_dst;

            TRACE_EXT("GATED-RIP WARNING: route to %s in both tables\n",
                            inet_ntoa(tmp->sin_addr));
            do {
              rt_delete(exrt, KERNEL_INTR);
            } while (exrt = rt_lookup((int)EXTERIOR, &n->rip_dst));
            TRACE_EXT("GATED-RIP: Route in exterior table deleted\n");
          }
          if (equal(&rt->rt_router, from)) {
            if (n->rip_metric >= DELAY_INFINITY) {
              if (rt->rt_metric < DELAY_INFINITY) {
                TRACE_ACTION(UNREACHABLE, rt);
                change = TRUE;
                rt->rt_metric = DELAY_INFINITY;
                rt->rt_timer = EXPIRE_TIME;
              }
              continue;
            }
            if (n->rip_metric != rt->rt_metric)
              if (rt_change(rt, from, n->rip_metric, RTPROTO_RIP, fromproto))
                change = TRUE;
            rt->rt_timer = 0;
          }
          else {
            /* if a metric is INFINITY at this point
             * we don't care about the new router.
             * The only way it would accept this
             * route anyway would be if the metric
             * was already 16 and the route was old.
             * we will stick with old gateway and let
             * it time out in 2 minutes if it wants to.
             *
             * also, if the current metric is INFINITY,
             * we will only listen to our current
             * gateway.  Yes, a terrible hold down!
             * if our current gateway says nothing then
             * this route will expire in 120 seconds.
             */
            if ((n->rip_metric >= DELAY_INFINITY) ||
                (rt->rt_metric >= DELAY_INFINITY))
              continue;
            if ((n->rip_metric < rt->rt_metric) ||
                ((rt->rt_timer > (EXPIRE_TIME/2)) &&
                (rt->rt_metric == n->rip_metric))) {
              if ((rt->rt_proto & RTPROTO_HELLO) &&
                  (n->rip_metric >= rt->rt_hwindow.h_min))
                continue;
              if (rt_change(rt, from, n->rip_metric, RTPROTO_RIP, fromproto))
                change = TRUE;
              rt->rt_timer = 0;
            }
          }
        }
      }  /*  for each net */
      break;
    default:
      syslog(LOG_ERR, "ripin: invalid rip command %d from %s\n",
                   inripmsg->rip_cmd, inet_ntoa(sin_from->sin_addr));
      break;
  }
  if (change && tracing & TR_RT)
    printf("rip_update: above routes supplied from %s updates %s\n",
                inet_ntoa(sin_from->sin_addr), strtime);
}

/*
 * Apply the function "f" to all non-passive
 * interfaces.  If the interface supports the
 * use of broadcasting use it, otherwise address
 * the output to the known router.
 */

toall(f)
	int (*f)();
{
  register struct interface *ifp;
  register struct advlist *ad;
  register struct sockaddr *dst;
  register int flags;
  extern struct interface *ifnet;

  if (!(rip_pointopoint)) {
    for (ifp = ifnet; ifp; ifp = ifp->int_next) {
      if ((ifp->int_flags & IFF_PASSIVE) ||
          ((ifp->int_flags & IFF_UP) == 0) ||
          (ifp->int_flags & IFF_NORIPOUT))
        continue;
      dst = (ifp->int_flags & IFF_BROADCAST) ? &ifp->int_broadaddr :
               (ifp->int_flags & IFF_POINTOPOINT) ? &ifp->int_dstaddr :
                   &ifp->int_addr;
      dst->sa_family = AF_INET;   /*  what else???? */
      flags = ((ifp->int_flags & IFF_INTERFACE) &&
               !(ifp->int_flags & IFF_POINTOPOINT)) ? MSG_DONTROUTE : 0;
      (*f)(dst, flags, ifp);
    }
  }
  for (ad = srcriplist; ad; ad = ad->next) {
    struct sockaddr_in tmpdst;

    bzero((char *)&tmpdst, sizeof(tmpdst));
    tmpdst.sin_family = AF_INET;
    tmpdst.sin_addr = ad->destnet;
    if ((ifp = if_withnet(&tmpdst)) <= (struct interface *)0) {
      syslog(LOG_ERR, "Source RIP gateway %s not on same net\n",
                    inet_ntoa(tmpdst.sin_addr));
      continue;
    }
    if (((ifp->int_flags & IFF_UP) == 0) || (ifp->int_flags & IFF_NORIPOUT))
      continue;
    flags = ((ifp->int_flags & IFF_INTERFACE) &&
             !(ifp->int_flags & IFF_POINTOPOINT)) ? MSG_DONTROUTE : 0;
    (*f)(&tmpdst, flags, ifp);
  }
}

/*
 * Output a preformed RIP packet.
 */

/*ARGSUSED*/
sendripmsg(dst, flags, ifp)
	struct sockaddr *dst;
	int flags;
	struct interface *ifp;
{
  register u_long tmp = ntohl((u_long)ripmsg->rip_nets[0].rip_metric);
  struct rt_entry *rt;
  struct netinfo *n = ripmsg->rip_nets;

  if (dst->sa_family != AF_INET)
    return;
  /*
   * Check to see if we are sending the initial RIP request to other
   * gateways.  That request has no restrictions other than whether RIP
   * is allowed on that interface or not.  This restriction is handled
   * in toall().
   */
  if (!((ntohs(n->rip_dst.sa_family) == AF_UNSPEC) &&
       (ntohl((u_long)n->rip_metric) == RIPHOPCNT_INFINITY))) {
    n->rip_dst.sa_family = ntohs(n->rip_dst.sa_family);
    rt = rt_lookup((int)INTERIOR, &n->rip_dst);
    if (rt == NULL) {
      rt = rt_lookup((int)HOSTTABLE, &n->rip_dst);
      if (rt == NULL) {
        syslog(LOG_ERR, "sendripmsg: bad route %s\n",
               inet_ntoa(sock_inaddr(&n->rip_dst)));
        return;
      }
    }
    n->rip_dst.sa_family = htons(n->rip_dst.sa_family);
    /*
     * make sure this route can be announced via this interface/proto.
     */
    if (is_valid(rt, RTPROTO_RIP, ifp) == 0)
      return;
    /*
     * since we are only sending out this one packet, we can add the
     * interface metric here.  Don't forget Split Horizon.
     */
    if ((rt->rt_ifp == ifp) &&
        (rt->rt_fromproto & (RTPROTO_RIP|RTPROTO_DIRECT|RTPROTO_KERNEL)) &&
        ((rt->rt_state & RTS_STATIC) == 0)) {
      tmp = ntohl((u_long)n->rip_metric);
      n->rip_metric = htonl((u_long)RIPHOPCNT_INFINITY);
    }
    else if ((tmp = ntohl((u_long) n->rip_metric)) != RIPHOPCNT_INFINITY) {
      if ((tmp + ifp->int_metric) >= RIPHOPCNT_INFINITY)
        n->rip_metric = htonl((u_long)RIPHOPCNT_INFINITY);
      else
        n->rip_metric = htonl((u_long)(tmp + ifp->int_metric));
    }
  }
  (*afswitch[dst->sa_family].af_output)(rip_socket, flags,
                                           dst, sizeof (struct rip));
  TRACE_RIPOUTPUT(ifp, dst, sizeof (struct rip));
  n->rip_metric = htonl(tmp);
}

/*
 * Supply dst with the contents of the routing tables.
 * If this won't fit in one packet, chop it up into several.
 */
supply(dst, flags, ifp)
	struct sockaddr *dst;
	int flags;
	struct interface *ifp;
{
  register struct rt_entry *rt;
  struct netinfo *n = ripmsg->rip_nets;
  register struct rthash *rh;
  struct rthash *base = hosthash;
  int doinghost = 1, size;
  int (*output)() = afswitch[AF_INET].af_output;

  ripmsg->rip_cmd = RIPCMD_RESPONSE;
  ripmsg->rip_vers = RIPVERSION;
again:
  for (rh = base; rh < &base[ROUTEHASHSIZ]; rh++)
    for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
      if ((doinghost == 0) && ((rt->rt_state & RTS_INTERIOR) == 0))
        continue;
      if (is_valid(rt, RTPROTO_RIP, ifp) == 0)
        continue;
      if (rt->rt_proto & RTPROTO_REDIRECT)
        continue;
      /*
       * don't broadcast subnets where they shouldn't be announced.
       */
      if ((doinghost == 0) && (rt->rt_state & RTS_SUBNET)) {
        if ((ifp->int_flags & IFF_SUBNET) == 0)
          continue;
        if (inet_wholenetof(sock_inaddr(&rt->rt_dst)) !=
            inet_wholenetof(sock_inaddr(dst)))
          continue;
      }
      size = (char *)n - rip_packet;
      if (size > RIPPACKETSIZE - sizeof (struct netinfo)) {
        (*output)(rip_socket, flags, dst, size);
        TRACE_RIPOUTPUT(ifp, dst, size);
        n = ripmsg->rip_nets;
      }
      n->rip_dst = rt->rt_dst;
      n->rip_dst.sa_family = htons(n->rip_dst.sa_family);
      if (rip_gateway && (sock_inaddr(&rt->rt_dst).s_addr == DEFAULTNET) &&
          (rt->rt_state & RTS_NOTINSTALL))
        n->rip_metric = min((rip_default + ifp->int_metric),RIPHOPCNT_INFINITY);
      else
        if ((ifp == rt->rt_ifp) &&
            (rt->rt_fromproto & (RTPROTO_RIP|RTPROTO_DIRECT|RTPROTO_KERNEL)) &&
            ((rt->rt_state & RTS_STATIC) == 0))
          n->rip_metric = RIPHOPCNT_INFINITY;
      else {
        switch (rt->rt_proto) {
          case RTPROTO_RIP:
          case RTPROTO_HELLO:
            if (rt->rt_metric >= DELAY_INFINITY)
              n->rip_metric = RIPHOPCNT_INFINITY;
            else
              n->rip_metric = min(mapmetric(HELLO_TO_RIP, rt->rt_metric) +
                                    ifp->int_metric + 1, RIPHOPCNT_INFINITY);
            break;
          default:
            if (rt->rt_metric >= TDHOPCNT_INFINITY)
              n->rip_metric = RIPHOPCNT_INFINITY;
            else
              n->rip_metric = min(mapmetric(HELLO_TO_RIP, rt->rt_metric) +
                                    ifp->int_metric + 1, RIPHOPCNT_INFINITY);
            break;
        }
        if ( (rt->rt_ifp->int_flags & IFF_UP) == 0 ) {
          n->rip_metric = RIPHOPCNT_INFINITY;
        }
        /*
         * If fixed metrics are in use for RIP on this interface and the
         * current metric is less than infinity, use the fixed metric
         * instead. 
         */
        if ( (ifp->int_flags & IFF_RIPFIXEDMETRIC) && (n->rip_metric < RIPHOPCNT_INFINITY) ) {
          n->rip_metric = ifp->int_ripfixedmetric;
        }
      }
      n->rip_metric = htonl((u_long)n->rip_metric);
      n++;
    }
  if (doinghost) {
    doinghost = 0;
    base = nethash;
    goto again;
  }
  if (n != ripmsg->rip_nets) {
    size = (char *)n - rip_packet;
    (*output)(rip_socket, flags, dst, size);
    TRACE_RIPOUTPUT(ifp, dst, size);
  }
}
