
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  January 8, 1987 - Mark Fedor
 *
 * $Header: rt_control.c,v 1.3.1.17 88/01/09 19:02:41 jch Locked $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: rt_control.c,v 1.3.1.17 88/01/09 19:02:41 jch Locked $";
#endif	not lint

#include "include.h"

extern char *calloc();
extern char *strncpy();

/*
 * control_lookup() looks up a restricted network for an exact match.
 */

struct restrictlist *
control_lookup(table, dst)
	int table;
	struct sockaddr_in *dst;
{
  register struct restrictlist *rt;
  register struct restricthash *rh;
  register unsigned hash;
  struct afhash h;

  if (dst->sin_family != AF_INET)
    return ((struct restrictlist *)NULL);

  (*afswitch[dst->sin_family].af_hash)(dst, &h);
  hash = h.afh_nethash;
  rh = &rt_restrict[hash & ROUTEHASHMASK];

  for (rt = rh->rt_forw; rt != (struct restrictlist *)rh; rt = rt->rt_forw) {
    if (rt->rhash != hash)
      continue;
    if ((equal(&rt->rdst, dst)) && (rt->flags & table))
      return (rt);
  }
  return ((struct restrictlist *)NULL);
}

/*
 * control_add() adds a route to either the announce or forbid control tables.
 */

control_add(table, control_net)
	int table;			/* interior or exterior table */
	struct restrictlist *control_net;
{
  register struct restrictlist *rt;
  struct restricthash *rh;
  unsigned hash;
  struct afhash h;

  if (control_net->rdst.sin_family != AF_INET)
    return;

  (*afswitch[control_net->rdst.sin_family].af_hash)(&control_net->rdst, &h);
  hash = h.afh_nethash;
  rh = &rt_restrict[hash & ROUTEHASHMASK];

  rt = (struct restrictlist *)malloc(sizeof(*rt));
  if (rt == 0) {
    syslog(LOG_ERR, " control_add: malloc: out of memory\n");
    return;
  }
  rt->rhash = hash;
  rt->rdst = control_net->rdst;
  rt->regpmetric = control_net->regpmetric;
  rt->rproto = control_net->rproto;
  rt->flags = table;
  bcopy((char *)control_net->rintf, (char *)rt->rintf, sizeof(rt->rintf));
  insque(rt, rh);
}

rt_resinit(fp)
	FILE *fp;
{
  char *cap();
  char *keyword;
  char *savekeyword;
  char *option;
  char buf[BUFSIZ];
  struct 	sockaddr_in addr_in;
  struct  restrictlist res_net;
  int	fatalerror = FALSE, intfc, error = FALSE, line = 0, protocols;
  char nethost[MAXHOSTNAMELENGTH+1];
  struct interface *tifp;
  int global_flag_set = 0;

  rewind(fp);
  announcethesenets = 0;
  donotannounce = 0;
  glob_announcethesenets = 0;
  glob_donotannounce = 0;
  donotlisten = 0;
  islisten = 0;

  while (fgets(buf, sizeof(buf), fp) != NULL) {
    line++;
    bzero((char *)&addr_in, sizeof(addr_in));
    bzero((char *)&res_net, sizeof(res_net));
    if (error == TRUE && fatalerror == FALSE)
      fatalerror = TRUE;
    error = FALSE;
    if ((buf[0] == '#') || (buf[0] == '\n'))
      continue;
    if (((keyword = cap(buf)) != (char *)NULL) &&
        ((strcmp(keyword, "announce") == 0) ||
        (strcmp(keyword, "announcehost") == 0) ||
        (strcmp(keyword, "noannounce") == 0) ||
        (strcmp(keyword, "noannouncehost") == 0) ||
        (strcmp(keyword, "listen") == 0) ||
        (strcmp(keyword, "listenhost") == 0) ||
        (strcmp(keyword, "donotlisten") == 0) ||
        (strcmp(keyword, "donotlistenhost") == 0))) {
      savekeyword = calloc((unsigned)strlen(keyword), sizeof(char));
      (void) strcpy(savekeyword, keyword);
      if ((option = cap(buf)) == (char *)NULL) {
        printf("rt_resinit: unexpected EOLN at line %d\n",line);
        error = TRUE;
        continue;
      }
      if ((strcmp(savekeyword, "announcehost") == 0) ||
          (strcmp(savekeyword, "noannouncehost") == 0) ||
          (strcmp(savekeyword, "listenhost") == 0) ||
          (strcmp(savekeyword, "donotlistenhost") == 0))
        (void) strcpy(nethost, "host");
      else
        (void) strcpy(nethost, "net");
      if (getnetorhostname(nethost, option, &addr_in) <= 0) {
        printf("rt_resinit: invalid %s %s at line %d\n",
                    nethost, option, line);
        error = TRUE;
        continue;
      }
      res_net.rdst = addr_in;
      if ((option = cap(buf)) == (char *)NULL) {
        printf("rt_resinit: unexpected EOLN at line %d\n",line);
        error = TRUE;
        continue;
      }
      if (((strcmp(savekeyword, "listenhost") == 0) ||
          (strcmp(savekeyword, "listen") == 0)) &&
          (strcmp(option,"gateway") != 0)) {
        printf("rt_resinit: 'gateway' expected at line %d\n",line);
        error = TRUE;
        continue;
      }
      else {
        if ((strcmp(savekeyword, "listenhost") != 0) &&
            (strcmp(savekeyword, "listen") != 0)) {
          if (strcmp(option,"intf") != 0) {
            printf("rt_resinit: 'intf' expected at line %d\n",line);
            error = TRUE;
            continue;
          }
        }
      }
      intfc = 0;
      while ((option = cap(buf)) != (char *)NULL) {
        if (strcmp(option, "proto") == 0) {
          if (intfc == 0) {
	    printf("rt_resinit: No interface list at line %d\n",line);
            error = TRUE;
          }
          break;
        }
        if (strcmp(option, "all") == 0) {
          if (intfc != 0) {
            printf("rt_resinit: 'all' unexpected at line %d\n",line);
            error = TRUE;
          }
          else {
            if ((option = cap(buf)) == (char *)NULL) {
              printf("rt_resinit: unexpected EOLN at line %d\n",line);
              error = TRUE;
            }
            else
              if (strcmp(option, "proto") != 0) {
                printf("rt_resinit: 'proto' expected at line %d\n",line);
                error = TRUE;
              }
          }
          global_flag_set++;
          break;
        }
        if (!getnetorhostname("host",option,&addr_in)) {
          printf("rt_resinit: invalid host %s at line %d\n",
                          option, line);
          error = TRUE;
          break;
        }
        if ((strcmp(savekeyword, "listenhost") != 0) &&
            (strcmp(savekeyword, "listen") != 0)) {
          if ((tifp = if_ifwithaddr(&addr_in)) <= (struct interface *)0) {
            printf("rt_resinit: invalid interface %s at line %d\n",
                          option, line);
            error = TRUE;
            break;
          }
        }
        if (intfc >= MAXINTERFACE) {
          printf("rt_resinit: max number of interfaces at line %d\n",line);
          error = TRUE;
          break;
        }
        res_net.rintf[intfc] = addr_in.sin_addr.s_addr;
        intfc++;
        if ((strcmp(savekeyword, "noannounce") == 0) ||
            (strcmp(savekeyword, "noannouncehost") == 0)) {
          if (tifp->int_flags & IFF_ANNOUNCE) {
            printf("rt_resinit: announce and noannounce specified ");
            printf("for interface %s at line %d\n", option, line);
            error = TRUE;
	    fatalerror = TRUE;
            break;
          }
          else
            tifp->int_flags |= IFF_NOANNOUNCE;
        }
        else if ((strcmp(savekeyword, "announce") == 0) ||
            (strcmp(savekeyword, "announcehost") == 0)) {
          if (tifp->int_flags & IFF_NOANNOUNCE) {
            printf("rt_resinit: announce and noannounce specified ");
            printf("for interface %s at line %d\n", option, line);
            error = TRUE;
	    fatalerror = TRUE;
            break;
          }
          else
            tifp->int_flags |= IFF_ANNOUNCE;
        }
      }
      if (error == TRUE)
        continue;
      if (option == (char *)NULL) {
        printf("rt_resinit: unexpected EOLN at line %d\n",line);
        error = TRUE;
      }
      protocols = 0;
      while ((option = cap(buf)) != (char *)NULL) {
        if (strcmp(option, "egpmetric") == 0) {
          if (protocols == 0) {
            printf("rt_resinit: No protocol list at line %d\n",line);
            error = TRUE;
          }
          break;
        }
        if (strcmp(option, "all") == 0) {
          if (protocols != 0) {
            printf("rt_resinit: 'all' unexpected at line %d\n",line);
            error = TRUE;
          }
          else {
            if ((option = cap(buf)) == (char *)NULL) {
              printf("rt_resinit: unexpected EOLN at line %d\n",line);
              error = TRUE;
            }
            else
              if (strcmp(option, "egpmetric") != 0) {
                printf("rt_resinit: 'egpmetric' expected at line %d\n",line);
                error = TRUE;
              }
          }
          res_net.rproto = (RTPROTO_RIP|RTPROTO_HELLO|RTPROTO_EGP);
          break;
        }
        if (strcmp(option, "rip") == 0)
          res_net.rproto |= RTPROTO_RIP;
        else if (strcmp(option, "hello") == 0)
          res_net.rproto |= RTPROTO_HELLO;
        else if (strcmp(option, "egp") == 0)
          res_net.rproto |= RTPROTO_EGP;
        else if ((strcmp(option, "#") == 0) || (strcmp(option, "\n") == 0)) {
          if (res_net.rproto & RTPROTO_EGP) {
            printf("rt_resinit: 'egpmetric' expected at line %d\n",line);
            error = TRUE;
          }
          break;
        }
        else {
          printf("rt_resinit: unknown protocol %s at line %d\n",
                     option, line);
          error = TRUE;
          break;
        }
        protocols++;
      }
      if (error == TRUE)
        continue;
      if (((option == (char *)NULL) || (strcmp(option, "#") == 0)) &&
          ((protocols == 0) || (res_net.rproto & RTPROTO_EGP))) {
        printf("rt_resinit: unexpected EOLN at line %d\n",line);
        error = TRUE;
        continue;
      }
      if (res_net.rproto & RTPROTO_EGP) {
        if ((option = cap(buf)) == (char *)NULL) {
          printf("rt_resinit: unexpected EOLN at line %d\n",line);
          error = TRUE;
          continue;
        }
        res_net.regpmetric = atoi(option);
        if ((res_net.regpmetric >= 6) || (res_net.regpmetric < 0)) {
          printf("rt_resinit: invalid EGP metric %d at line %d\n",
                    res_net.regpmetric, line);
          error = TRUE;
          continue;
        }
      }
      if ((strcmp(savekeyword, "announce") == 0) ||
          (strcmp(savekeyword, "announcehost") == 0)) {
        if (res_net.rproto & RTPROTO_EGP)
          n_remote_nets++;
        if (global_flag_set != 0)
          glob_announcethesenets++;
        announcethesenets++;
        control_add(RT_ANNOUNCE, &res_net);
      }
      if ((strcmp(savekeyword, "noannounce") == 0) ||
          (strcmp(savekeyword, "noannouncehost") == 0)) {
        if (global_flag_set != 0)
          glob_donotannounce++;
        donotannounce++;
        control_add(RT_NOANNOUNCE, &res_net);
      }
      if ((strcmp(savekeyword, "donotlisten") == 0) ||
          (strcmp(savekeyword, "donotlistenhost") == 0)) {
        donotlisten++;
        control_add(RT_NOLISTEN, &res_net);
      }
      if ((strcmp(savekeyword, "listen") == 0) ||
          (strcmp(savekeyword, "listenhost") == 0)) {
        islisten++;
        control_add(RT_SRCLISTEN, &res_net);
      }
      TRACE_INT("RESTRICT %s %s proto %d egpmetric %d\n",
		    savekeyword, inet_ntoa(res_net.rdst.sin_addr),
		    res_net.rproto, res_net.regpmetric);
      global_flag_set = 0;
      free(savekeyword);
    }
  }
  if (((glob_announcethesenets != 0) && ((glob_donotannounce !=0) ||
       (donotannounce != 0))) || ((glob_donotannounce != 0) &&
       (announcethesenets != 0))) {
    printf("The \"Do not announce\" list and \"announce\" list both\n");
    printf("initialized globally.  You can only have one or the other\n");
    printf("applied globally.  You can have both on different interfaces.\n");
    fatalerror = TRUE;
  }
  if (fatalerror == TRUE) {
    printf("rt_resinit: %s: initialization error\n", EGPINITFILE);
    quit();
  }
}

/* 
 * Thanks to Doug Elias, Cornell Theory Center for these string
 * routines.
 */
/******************************************************************
*name:        cap  <cut_and_paste>
*
*function:    take off next white-space-separated "word" from 
*             "string", resetting "string" to 1 character past 
*             "word" in original "string"
*
*arguments:   (char *)string
*
*algorithm:   skip over any leading "white-space";
*             if (string is blank)
*               return (NULL);
*             else
*               send string to substr() to take off next "word" 
*                <contiguous string of non-white-space characters>;
*               set string to 1 character past "word";
*               return("word");
*
*calls:       (char *)next_white()
*               returns ptr to next "white-space" character
*             (char *)substr()
*               returns a substring of characters according to 
*                 arguments given on starting and ending positions 
*                 in parent string
*
*returns:     (char *)NULL if at end of string
*             (char *)word if not
*
*bugs:
*
*to-do:
*
*history:
*******************************************************************/
#include <stdio.h>

#define EOL ('\0')


char *cap(string)
  char *string;
{
  static char word[80];
  char *substr();
  char *next_white();
  char *str = string;
  char *w_ptr;
  int start;
  int stop;

#ifdef  DEBUG
  printf("cap: rcvd \"%s\"\n", string);
#endif

  while (*str != EOL && (*str == ' ' || *str == '\t'))
    ++str;

  if (*str == EOL)
    return(NULL);

  word[0] = EOL;
  start = str - string;
  stop = ((w_ptr = next_white(str)) == NULL) ? -1 :
						  w_ptr - string;
  (void) strcpy(word, substr(string, start, stop));
  if (stop == -1)
    string[0] = EOL;
  else
    (void) strcpy(string, string + stop + 1);

#ifdef DEBUG
  printf("cap: rtn \"%s\", string == \"%s\"\n", word, string);
#endif

  return(word);
}

/*******************************************************************
*name:        next_white
*
*function:    return ptr to next "white-space" character
*
*arguments:   string
*
*algorithm:   set ptr to beginning of string;
*             while (character under ptr not white-space or EOL)
*               skip to next character;
*             if (EOL)
*               return(NULL);
*             else
*               return(ptr);
*
*calls:       none
*
*returns:     (char *)NULL if no white space before EOL
*             (char *)ptr if white space exists
*
*bugs:
*
*to-do:
*
*history:
*******************************************************************/

char *next_white(string)
  char *string;
{
  char *ptr = string;

#ifdef DEBUG
  printf("next_white: rcvd \"%s\"\n", string);
#endif

  while (*ptr != ' ' && 
	 *ptr != '\t' &&
	 *ptr != '\n' && 
	 *ptr != EOL)
    ++ptr;

#ifdef DEBUG
  printf("next_white: rtn %d\n", (*ptr == '\0') ? 0 :
						  ptr - string);
#endif

  return((*ptr == EOL) ? NULL : ptr);
}

/******************************************************************
*name:        substr
*
*function:    return a string from the middle of another string
*
*arguments:   (char *) original string
*             (int) starting position
*             (int) ending position
*
*algorithm:   if (stop < 0)
*               set stop to length of string;
*             if (start >= stop)
*               return(NULL);
*             set substr to (string + start) thru (string + stop);
*             return(substr);
*
*calls:       none
*
*returns:     (char *)NULL if start >= stop
*             (char *)substr otherwise
*
*bugs:
*
*to-do:
*
*history:
******************************************************************/

char *substr(string, start, stop)
  char *string;
  int start;
  int stop;
{
  char asubstr[80];

#ifdef DEBUG
  printf("asubstr: rcvd \"%s\"(string) %d(start) %d(stop)\n",
   string, start, stop);
#endif

  if (stop < 0)
    stop = strlen(string);

  if (start >= stop)
    return(NULL);

  asubstr[0] = EOL;
  (void) strncpy(asubstr, string + start, stop - start);
  asubstr[stop - start] = EOL;

#ifdef DEBUG
  printf("asubstr: rtn \"%s\"\n", asubstr);
#endif

  return(asubstr);
}

/*
 * given a route entry, protocol, and interface, check and see
 * if the route is valid.  If it is, return a 1.  If not, return 0.
 */
is_valid(rt, protocol, intfp)
	struct rt_entry *rt;
	int protocol;
	struct interface *intfp;
{
  int intfc;
  u_long intfcaddr;

  if (intfp == (struct interface *)NULL)
    return(0);

  intfcaddr = (intfp->int_flags & IFF_POINTOPOINT) ?
		     sock_inaddr(&intfp->int_dstaddr).s_addr :
		     sock_inaddr(&intfp->int_addr).s_addr;

  if ((glob_announcethesenets != 0) || (intfp->int_flags & IFF_ANNOUNCE)) {
    if (rt->rt_announcelist == (struct restrictlist *)NULL)
      return(0);
    if ((rt->rt_announcelist->rproto & protocol) == 0)
      return(0);
    intfc = 0;
    while ((rt->rt_announcelist->rintf[intfc] != 0) &&
           (intfc < MAXINTERFACE)) {
      if (rt->rt_announcelist->rintf[intfc] == intfcaddr)
        break;
      intfc++;
    }
    if ((rt->rt_announcelist->rintf[intfc] == 0) &&
        (intfc != 0))
      return(0);
  }
  else {
    if ((glob_donotannounce != 0) || (intfp->int_flags & IFF_NOANNOUNCE)) {
      if (rt->rt_noannouncelist != (struct restrictlist *)NULL) {
        if (rt->rt_noannouncelist->rproto & protocol) {
          intfc = 0;
          while ((rt->rt_noannouncelist->rintf[intfc] != 0) &&
                 (intfc < MAXINTERFACE)) {
            if (rt->rt_noannouncelist->rintf[intfc] == intfcaddr)
              break;
            intfc++;
          }
          if ((rt->rt_noannouncelist->rintf[intfc] != 0) ||
              (intfc == 0))
            return(0);
        }
      }
    }
  }
  return(1);
}
		
/*
 *  given a route entry, protocol, interface pointer, and
 *  an address, check and see if the INPUT route is ok. Return
 *  0 if the input packet is NOT valid. Return 1 if it is valid.
 */
is_valid_in(rt, protocol, intfp, whofrom)
	struct rt_entry *rt;
	int protocol;
	struct interface *intfp;
	struct sockaddr_in *whofrom;
{
  int intfc;
  u_long intfcaddr;

  intfcaddr = (intfp->int_flags & IFF_POINTOPOINT) ?
		     sock_inaddr(&intfp->int_dstaddr).s_addr :
		     sock_inaddr(&intfp->int_addr).s_addr;

  if (intfp == (struct interface *)NULL)
    return(0);

  /*
   * Since the kernel doesn't know about these neat restrictions on
   * input, return VALID for a redirect.  This will keep the gated
   * tables more consistent with the kernel's.
   */
  if (protocol & RTPROTO_REDIRECT)
    return(1);

  if (donotlisten != 0) {
    if (rt->rt_listenlist != (struct restrictlist *)NULL) {
      if (rt->rt_listenlist->rproto & protocol) {
        intfc = 0;
        while ((rt->rt_listenlist->rintf[intfc] != 0) &&
               (intfc < MAXINTERFACE)) {
          if (rt->rt_listenlist->rintf[intfc] == intfcaddr)
            break;
          intfc++;
        }
        if ((rt->rt_listenlist->rintf[intfc] != 0) ||
            (intfc == 0))
          return(0);
      }
    }
  }
  if (islisten != 0) {
    if (rt->rt_srclisten != (struct restrictlist *)NULL) {
      if ((rt->rt_srclisten->rproto & protocol) == 0)
        return(0);
      intfc = 0;
      while ((rt->rt_srclisten->rintf[intfc] != 0) &&
             (intfc < MAXINTERFACE)) {
        if (rt->rt_srclisten->rintf[intfc] == whofrom->sin_addr.s_addr)
          break;
        intfc++;
      }
      if ((rt->rt_srclisten->rintf[intfc] == 0) && (intfc != 0))
        return(0);
    }
  }
  return(1);
}
