
/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: rt_init.c,v 1.3.1.17 88/01/09 19:00:59 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: rt_init.c,v 1.3.1.17 88/01/09 19:00:59 jch Exp $";
#endif	not lint

/* rt_init.c
 *
 * Initialization functions for routing tables - interface routes, reading 
 * kernel routes, non-routing gateways, and routes to be advised in EGP 
 * NR Update messages.
 *
 * Functions: rt_init, rt_readkernel, rt_ifinit, rt_dumbinit, rt_NRadvise_init
 */

#include <sys/types.h>
#include <sys/mbuf.h>			/* needed by <net/route.h> */
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <stdio.h>
#include <netdb.h>
#include <errno.h>
#include <nlist.h>
#include <syslog.h>

#include <net/if.h>
#define KERNEL 1		/* to ensure all <net/route.h> compiled */
#include <net/route.h>

#if ULTRIX1_X || BSD42
#include "routed.h"
#else
#include <protocols/routed.h>
#endif

#include "defs.h"
#include "egp.h"
#include "egp_param.h"
#include "if.h"
#include "rt_table.h"
#include "rt_control.h"
#include "rip.h"
#include "af.h"
#include "hello.h"
#include "trace.h"

/*
 * rt_init() initializes exterior and interior routing tables doubly linked
 * lists. 
 */

extern off_t lseek();

rt_init()
{
  register struct rthash *rh;
  register struct restricthash *ac;

#ifdef lint
  bzero((char *)&rtstat, sizeof(rtstat));   /* make lint happy */
#endif lint

  for (rh = nethash; rh < &nethash[ROUTEHASHSIZ]; rh++)	
    rh->rt_forw = rh->rt_back = (struct rt_entry *)rh;
  for (rh = hosthash; rh < &hosthash[ROUTEHASHSIZ]; rh++)	
    rh->rt_forw = rh->rt_back = (struct rt_entry *)rh;
  for (ac = rt_restrict; ac < &rt_restrict[ROUTEHASHSIZ]; ac++)	
    ac->rt_forw = ac->rt_back = (struct restrictlist *)ac;
}

/*
 * rt_readkernel() initializes the GATED routing tables from the current kernel
 * routing tables through /dev/kmem.
 * This is necessary to ensure consistency when the GATED process is terminated
 * and restarted while the supporting host continues to run. This may be done
 * if configuration information needs to be changed.
 */

struct nlist nl[] = {
#define	N_RTHOST		0
	{ "_rthost" },
#define N_RTNET			1
	{ "_rtnet"},
#define N_RTHASHSIZE		2
	{ "_rthashsize"},
	{ "" },
};


rt_readkernel()
{
  /*
   *  ..routed/table.h redefines some of the elements of 
   * struct rtentry defined in <net/route.h>. In order for the code from routed
   * to be able to be utilized part of the kernel structure has been redefined 
   * with a preceding k_.  
   */
  struct k_rtentry {
    u_long	k_rt_hash;		/* to speed lookups */
    struct	sockaddr k_rt_dst;	/* key */
    struct	sockaddr k_rt_gateway;	/* value */
    short	k_rt_flags;		/* up/down?, host/net */
  } *rt;

  int 	i, hashsize, kmem, doinghost;
  struct 	mbuf 	*next, m_buf, **base;

  (void) nlist("/vmunix", nl);
  if ((nl[N_RTHOST].n_value == 0) || (nl[N_RTNET].n_value == 0)) {
    syslog(LOG_ERR, "rt_readkernel: rthost and rtnet not in namelist\n");
    quit();
  }
  kmem = open("/dev/kmem", 0);
  if (kmem < 0) {
    p_error("rt_readkernel: /dev/kmem");
    quit();
  }
  if (nl[N_RTHASHSIZE].n_value != 0) {
    (void) lseek(kmem, (off_t) nl[N_RTHASHSIZE].n_value, 0);
    (void) read(kmem, (char *)&hashsize, sizeof(hashsize));
  }
  else {
#if (BSD42 || SUN3_3PLUS) && RTHASHSIZ
    hashsize = RTHASHSIZ;
#else
    syslog(LOG_ERR, "rt_readkernel: rthashsize not in namelist\n");
    quit();
#endif
  }
	
  /* set up to read table of net hash chains */

  if ((lseek(kmem, (off_t)nl[N_RTNET].n_value, 0) == -1) ||
       (read(kmem, (char *)rtnet, sizeof (rtnet)) != sizeof (rtnet))) {
    syslog(LOG_ERR, "rt_readkernel: error reading kmem\n");
    quit();
  }
  TRACE_RT("rt_readkernel: Initial routes read from kernel (if any):\n");
  base = rtnet;
  doinghost = 0;

doitagain:
  for (i = 0; i < hashsize; i++) {
    for (next = base[i]; next != NULL; next = m_buf.m_next) {
      if ((lseek(kmem, (off_t)next, 0) == -1) ||
          (read(kmem, (char *)&m_buf, MMINOFF + sizeof(struct k_rtentry)) !=
          (MMINOFF + sizeof (struct k_rtentry)))) {
        p_error("rt_readkernel: read");
        quit();
      }
      rt = mtod(&m_buf, struct k_rtentry *);
      if (!(rt->k_rt_flags & RTF_GATEWAY))
        continue;
      if (rt->k_rt_gateway.sa_family != AF_INET)
        continue;
      install = FALSE;  /* don't install routes in kernel */
      {
        register struct interface *ifp;
        int interior =  0, saveinstall, addgood = 0;
        struct rt_entry dup_rt;
        struct rt_entry *foundit;
			
        if (doinghost != 0) {
          if (!(foundit = rt_lookup((int)HOSTTABLE, &rt->k_rt_dst))) {
            addgood = rt_add((int)HOSTTABLE, &rt->k_rt_dst, &rt->k_rt_gateway,
                      (HOPCNT_INFINITY - 1) * ETOTD_CONV, 0, RTPROTO_KERNEL,
                      RTPROTO_KERNEL);
          }
          if (foundit || !(addgood)) {
            bzero((char *)&dup_rt, sizeof(struct rt_entry));
            dup_rt.rt_dst = rt->k_rt_dst;
            dup_rt.rt_router = rt->k_rt_gateway;
            dup_rt.rt_flags = rt->k_rt_flags;
            saveinstall = install;
            install = TRUE;
            rt_delete(&dup_rt, KERNEL_ONLY);
            install = saveinstall;
          }
        }
        else {
          for (ifp = ifnet; ifp; ifp = ifp->int_next)
            if (inet_wholenetof(((struct sockaddr_in *)&rt->k_rt_dst)->sin_addr)
                == inet_wholenetof(in_addr_ofs(&ifp->int_addr))) {
              interior++;
              break;
            }
          if (interior) {
            if (!(foundit = rt_lookup((int)INTERIOR, &rt->k_rt_dst))) {
              addgood = rt_add((int)INTERIOR, &rt->k_rt_dst, &rt->k_rt_gateway,
                         (HOPCNT_INFINITY - 1) * ETOTD_CONV, 0, RTPROTO_KERNEL,
                         RTPROTO_KERNEL);
            }
            if (foundit || !(addgood)) {
              bzero((char *)&dup_rt, sizeof(struct rt_entry));
              dup_rt.rt_dst = rt->k_rt_dst;
	      dup_rt.rt_router = rt->k_rt_gateway;
              dup_rt.rt_flags = rt->k_rt_flags;
              saveinstall = install;
              install = TRUE;
              rt_delete(&dup_rt, KERNEL_ONLY);
              install = saveinstall;
            }
          }
          else {
            if (!(foundit = rt_lookup((int)EXTERIOR, &rt->k_rt_dst))) {
              addgood = rt_add((int)EXTERIOR, &rt->k_rt_dst, &rt->k_rt_gateway,
                         (HOPCNT_INFINITY - 1) * ETOTD_CONV, 0, RTPROTO_KERNEL,
                         RTPROTO_KERNEL);
            }
            if (foundit || !(addgood)) {
              bzero((char *)&dup_rt, sizeof(struct rt_entry));
              dup_rt.rt_dst = rt->k_rt_dst;
              dup_rt.rt_router = rt->k_rt_gateway;
              dup_rt.rt_flags = rt->k_rt_flags;
              saveinstall = install;
              install = TRUE;
              rt_delete(&dup_rt, KERNEL_ONLY);
              install = saveinstall;
            }
          }
        }
      }
    }
  }
  /* set up to read the host hash chain. */
  if (doinghost == 0) {
    (void) lseek(kmem, (off_t)0 ,0);
    if ((lseek(kmem, (off_t)nl[N_RTHOST].n_value, 0) == -1) ||
        (read(kmem, (char *)rthost, sizeof (rthost)) != sizeof (rthost))) {
      syslog(LOG_ERR, "rt_readkernel: error reading kmem\n");
      quit();
    }
    base = rthost;
    doinghost++;
    goto doitagain;
  }
  (void) close(kmem);
  return;
}

/*
 * rt_ifinit() initializes the interior routing table with direct nets as
 * per the interface table. Such routes read from the kernel routing tables
 * are deleted from the exterior routing table.
 */

rt_ifinit()
{
  register  struct 	interface *ifp;
  register  struct	rt_entry *rt;
  struct sockaddr_in	dst;
  int iflags = RTS_INTERFACE;

  install = FALSE;	/* routes for interfaces already installed by kernel */
  TRACE_RT("rt_ifinit: interior routes for direct interfaces:\n");

  for (ifp = ifnet; ifp != NULL; ifp = ifp->int_next) {
    iflags = RTS_INTERFACE;
    if ((ifp->int_flags & IFF_UP) == 0)
      continue;
    if (((ifp->int_flags & (IFF_NORIPIN|IFF_NOHELLOIN)) ==
        (IFF_NORIPIN|IFF_NOHELLOIN)) || (!rip_supplier && !hello_supplier) ||
	(ifp->int_flags & IFF_NOAGE))
      iflags |= RTS_PASSIVE;
    if (ifp->int_flags & IFF_POINTOPOINT) {
      struct sockaddr_in *tdst = (struct sockaddr_in *)&ifp->int_dstaddr;
     
      iflags |= RTS_POINTOPOINT;
      install = TRUE;
      if ((rt = rt_lookup((int)HOSTTABLE, tdst)) != (struct rt_entry *) NULL)
        rt_delete(rt, KERNEL_INTR);
      install = FALSE;
      (void) rt_add((int)HOSTTABLE, tdst, &ifp->int_addr, 0, iflags,
                    RTPROTO_DIRECT, RTPROTO_DIRECT);
      continue;
    }
    install = TRUE;
    bzero((char *)&dst, sizeof(dst));
    dst.sin_family = AF_INET;
    dst.sin_addr = gd_inet_makeaddr((int)ifp->int_subnet, 0, TRUE);
    if ((rt = rt_lookup((int)EXTERIOR, &dst)) != (struct rt_entry *) NULL)
      rt_delete(rt, KERNEL_INTR);
    if ((rt = rt_lookup((int)INTERIOR, &dst)) != (struct rt_entry *) NULL)
      rt_delete(rt, KERNEL_INTR);
    install = FALSE;
    (void) rt_add((int)INTERIOR, &dst, &ifp->int_addr, 0, iflags,
                  RTPROTO_DIRECT, RTPROTO_DIRECT);
			
    if (ifp->int_net == ifp->int_subnet)
      continue;
    /*
     * Deal with subnets.
     * Since we're a gateway, we assume that if we can
     * reach one subnet of a given net, 
     * the entire net is reachable.
     */
    install = TRUE;
    bzero((char *)&dst, sizeof(dst));
    dst.sin_family = AF_INET;
    dst.sin_addr = gd_inet_makeaddr((int)ifp->int_net, 0, FALSE);
    if ((rt = rt_lookup((int)EXTERIOR, &dst)) != (struct rt_entry *)NULL)
      rt_delete(rt, KERNEL_INTR);
    if ((rt = rt_lookup((int)INTERIOR, &dst)) != (struct rt_entry *)NULL)
      rt_delete(rt, KERNEL_INTR);
    install = FALSE;
    /* add fake route to entire net */
    iflags |= RTS_PASSIVE;
    (void) rt_add((int)INTERIOR, &dst,  &ifp->int_addr, 0, iflags,
                  RTPROTO_DIRECT,0);
  }
}

/*
 * rt_dumbinit() reads the initialization file EGPINITFILE to
 * initialize: 
 * - the interior route table with routes to known non-routing gateways on a
 * shared net, these routes are static and not updated;
 * - a default gateway prior to another default being heard (if active default);
 * It installs them in the kernel if they were not previously read from the
 * kernel.
 */

rt_dumbinit(fp)
	FILE *fp;
{
  char keyword[MAXHOSTNAMELENGTH+1];
  char netname[MAXHOSTNAMELENGTH+1];
  char gname[MAXHOSTNAMELENGTH+1];
  char proto_type[MAXHOSTNAMELENGTH+1];
  char deftype[MAXHOSTNAMELENGTH+1];
  char buf[BUFSIZ];
  struct 	sockaddr_in	netaddr, gateway, defaultdst;
  struct	rt_entry *rt;
  int	metric, error = FALSE, old_install, line, ptype = 0;
  int   tableadd;
  struct interface *ifptr;

  bzero((char *)&netaddr, sizeof(netaddr));
  bzero((char *)&gateway, sizeof(gateway));
  bzero((char *)&defaultdst, sizeof(defaultdst));

  TRACE_RT("rt_dumbinit: non-routing gateway routes (if any):\n");

  rewind(fp);
  line = 0;

  while (fgets(buf, sizeof(buf), fp) != NULL) {
    line++;
    if ((buf[0] == '#') || (buf[0] == '\n'))
      continue;
    if (sscanf(buf, "%s", keyword) != 1)
      continue;
    if ((strcmp(keyword, "net") == 0) ||
        (strcmp(keyword, "host") == 0)) {
      if (sscanf(buf, "%*s %s gateway %s metric %d %s",
                     netname, gname, &metric, proto_type) != 4) {
        syslog(LOG_WARNING, "rt_dumbinit: syntax error line %d\n", line);
        error = TRUE;
      }
      else if (!getnetorhostname(keyword, netname, &netaddr)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid %s name or address %s\n",
                  keyword, netname);
        error = TRUE;
      }
      else if (!getnetorhostname("host", gname, &gateway)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid gateway name or address %s\n",
                  gname);
        error = TRUE;
      }
      else {    /* Initialize routings tables with non-routing gateway */
        /*
         * first delete any old route in exterior 
         * route table read from kernel if not a host route.
         */
        old_install = install;
        if (strcmp(keyword, "host") == 0)
          tableadd = HOSTTABLE;
        else {
          tableadd = INTERIOR;
          rt = rt_lookup((int)EXTERIOR, &netaddr);
          if (rt != NULL) {
            if (equal(&rt->rt_router, &gateway))
              install = FALSE;	/* already in kernel */
            rt_delete(rt, KERNEL_INTR);
          }
        }
        rt = rt_lookup(tableadd, &netaddr);
        if (rt != NULL) {
          if (equal(&rt->rt_router, &gateway))
            install = FALSE;	/* already in kernel */
          rt_delete(rt, KERNEL_INTR);
        }
        if (strcmp("rip", proto_type) == 0) {
          ptype = RTPROTO_RIP;
          metric = mapmetric(RIP_TO_HELLO, metric);
        }
        else if (strcmp("hello", proto_type) == 0)
          ptype = RTPROTO_HELLO;
        else if (strcmp("egp", proto_type) == 0) {
          ptype = RTPROTO_EGP;
          metric *= ETOTD_CONV;
        }
        else {
          syslog(LOG_WARNING, "unsupported protocol %s line %d\n",
                    proto_type, line);
          error = TRUE;
          continue;
        }
        (void) rt_add(tableadd, &netaddr, &gateway, metric,
				 RTS_STATIC|RTS_PASSIVE, ptype, ptype);
        install = old_install;
      }
    } /* end "net" || "host" entry */
    else if (strcmp(keyword, "defaultgateway") == 0) {
      if (sscanf(buf, "%*s %s %s %s", gname, proto_type, deftype) != 3) {
        syslog(LOG_WARNING, "rt_dumbinit: syntax error, line %d\n", line);
        error = TRUE;
      }
      else if (!getnetorhostname("host", gname, &gateway)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid gateway name or address %s\n",
                  gname);
        error = TRUE;
      }
      else {		/* Initialize default gateway */
        if (rip_gateway)
          continue;
        /*
         * check if a kernel entry exists for default
         * route, if not add new default else change 
         * to new default
         */
        defaultdst.sin_family = AF_INET;
        defaultdst.sin_addr.s_addr = DEFAULTNET;
        metric = (HOPCNT_INFINITY - 1) * ETOTD_CONV;
        if (strcmp("rip", proto_type) == 0)
          ptype = RTPROTO_RIP;
        else if (strcmp("hello", proto_type) == 0)
          ptype = RTPROTO_HELLO;
        else if (strcmp("egp", proto_type) == 0)
          ptype = RTPROTO_EGP;
        else {
          syslog(LOG_WARNING, "unsupported protocol %s line %d\n",
                    proto_type, line);
          error = TRUE;
          continue;
        }
        if (strcmp("passive", deftype) == 0) {
          /*
           * get rid of all left over defaults from initialization.
           */
          while (rt = rt_lookup((int)EXTERIOR, &defaultdst))
            rt_delete(rt, KERNEL_INTR);
          rt = rt_lookup((int)INTERIOR, &defaultdst);
          if (rt == NULL)
            (void) rt_add((int)INTERIOR, &defaultdst, &gateway, metric,
                          RTS_PASSIVE, ptype, ptype);
          else {
            (void) rt_change(rt, &gateway, metric, ptype, ptype);
            rt->rt_state |= RTS_PASSIVE;
          }
          rt_default_status = INSTALLED;
        }
        else if (strcmp("active", deftype) == 0) {
          /*
           * get rid of all left over defaults from initialization.
           */
          while (rt = rt_lookup((int)INTERIOR, &defaultdst))
            rt_delete(rt, KERNEL_INTR);
          rt = rt_lookup((int)EXTERIOR, &defaultdst);
          if (rt == NULL)
            (void) rt_add((int)EXTERIOR, &defaultdst, &gateway, metric,
                          RTS_PASSIVE, ptype, ptype);
          else {
            (void) rt_change(rt, &gateway, metric, ptype, ptype);
            rt->rt_state |= RTS_PASSIVE;
          }
          rt_default_status = INSTALLED;
        }
        else {
          syslog(LOG_WARNING, "unsupported default type %s line %d\n",
                    deftype, line);
          error = TRUE;
          continue;
        }
      }
    }	/* end "defaultgateway" entry */
    else if (strcmp(keyword, "interfacemetric") == 0) {
      if (sscanf(buf, "%*s %s %s", gname, proto_type) != 2) {
        syslog(LOG_WARNING, "rt_dumbinit: syntax error, line %d\n", line);
        error = TRUE;
      }
      else if (!getnetorhostname("host", gname, &gateway)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  gname);
        error = TRUE;
      }
      else if ((ifptr = if_ifwithaddr(&gateway)) <= (struct interface *)0) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  inet_ntoa(gateway.sin_addr));
        error = TRUE;
      }
      else
        ifptr->int_metric = atoi(proto_type);
    } /* end interfacemetric */
    else if (strcmp(keyword, "fixedmetric") == 0) {
      if (sscanf(buf, "%*s %s proto %s %s", gname, proto_type, deftype) != 3) {
        syslog(LOG_WARNING, "rt_dumbinit: syntax error, line %d\n", line);
        error = TRUE;
      } else if (!getnetorhostname("host", gname, &gateway)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  gname);
        error = TRUE;
      } else if ((ifptr = if_ifwithaddr(&gateway)) <= (struct interface *)0) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  inet_ntoa(gateway.sin_addr));
        error = TRUE;
      } else if (strcmp("rip", proto_type) == 0) {
        metric = atoi(deftype);
        if ((metric < 0) || (metric > RIPHOPCNT_INFINITY)) {
          syslog(LOG_WARNING, "rt_dumbinit: metric %d invalid in line %d\n",
                   metric, line);
          error = TRUE;
        } else {
          ifptr->int_ripfixedmetric = metric;
          ifptr->int_flags |= IFF_RIPFIXEDMETRIC;
        }
      } else if (strcmp("hello", proto_type) == 0) {
        metric = atoi(deftype);
        if ((metric < 0) || (metric > DELAY_INFINITY)) {
          syslog(LOG_WARNING, "rt_dumbinit: metric %d invalid in line %d\n",
                   metric, line);
          error = TRUE;
        } else {
          ifptr->int_hellofixedmetric = metric;
          ifptr->int_flags |= IFF_HELLOFIXEDMETRIC;
        }
      } else {
        syslog(LOG_WARNING, "rt_dumbinit: unsupported protocol %s in line %d\n",
                 proto_type, line);
        error = TRUE;
      }
      if (!error) {
        syslog(LOG_WARNING, "fixed metric interface %s protocol %s metric %d",
                 inet_ntoa(gateway.sin_addr), proto_type, metric);
        TRACE_RT("fixedmetric %s proto %s %d\n",
                   inet_ntoa(gateway.sin_addr), proto_type, metric);
      }
    } /* end fixedmetric */
    else if (strcmp(keyword, "reconstmetric") == 0) {
      if (sscanf(buf, "%*s %s %s", gname, proto_type) != 2) {
        syslog(LOG_WARNING, "rt_dumbinit: syntax error, line %d\n", line);
        error = TRUE;
      }
      else if (!getnetorhostname("host", gname, &gateway)) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  gname);
        error = TRUE;
      }
      else if ((ifptr = if_ifwithaddr(&gateway)) <= (struct interface *)0) {
        syslog(LOG_WARNING, "rt_dumbinit: invalid interface address %s\n",
                  inet_ntoa(gateway.sin_addr));
        error = TRUE;
      }
      else {
        if ((atoi(proto_type) > RIPHOPCNT_INFINITY) ||
            (atoi(proto_type) < 0)) {
           syslog(LOG_WARNING, "rt_dumbinit: invalid reconst metric %d\n",
                  atoi(proto_type));
           error = TRUE;
        }
        else {
          ifptr->int_reconstmetric = atoi(proto_type);
          ifptr->int_flags |= IFF_RECONST;
        }
      }
    } /* end reconst */
  } /* end while */
  if (error) {
    syslog(LOG_EMERG, "rt_dumbinit: %s: initialization error\n", EGPINITFILE);
    quit();
  }
}

/*
 * rt_NRadvise_init() reads the initialization file EGPINITFILE to
 * determine user specification of nets allowed to be advised in Network 
 * Reachability messages. If any such nets are specified, only those specified
 * are allowed to be advised.
 * These nets must still meet the normal rules. i.e. they should be either
 * direct nets or nets reached via non-routing gateways of the same autonomous
 * sytem reported in EGPINITFILE. If the net is not one of these it is
 * ignored.
 *
 * EGPINITFILE relevant format is:
 * egpnetsreachable name name .... name
 */

rt_NRadvise_init(fp)
	FILE *fp;
{
  char keyword[MAXHOSTNAMELENGTH+1];
  char netname[MAXHOSTNAMELENGTH+1];
  struct sockaddr_in netaddr;
  struct advlist *templist;
  struct advlist *tailof;
  int	c, error = FALSE, morenets;

  adlist = (struct advlist *) NULL;
  srcriplist = (struct advlist *) NULL;
  srchellolist = (struct advlist *) NULL;
  trustedhelloerlist = (struct advlist *) NULL;
  trustedripperlist = (struct advlist *) NULL;
  templist = (struct advlist *) NULL;
  tailof = (struct advlist *) NULL;
  bzero((char *)&netaddr, sizeof(netaddr));
  netaddr.sin_family = AF_INET;

  rewind(fp);
  /*
   * read first word of line and compare to key words
   */
  while(fscanf(fp, "%s", keyword) != EOF) {
    if ((strcmp(keyword, "egpnetsreachable") == 0) ||
        (strcmp(keyword, "noripoutinterface") == 0) ||
        (strcmp(keyword, "noripfrominterface") == 0) ||
        (strcmp(keyword, "nohellooutinterface") == 0) ||
        (strcmp(keyword, "nohellofrominterface") == 0) ||
        (strcmp(keyword, "trustedripgateways") == 0) ||
        (strcmp(keyword, "trustedhellogateways") == 0) ||
        (strcmp(keyword, "passiveinterfaces") == 0) ||
        (strcmp(keyword, "sourcehellogateways") == 0) ||
        (strcmp(keyword, "sourceripgateways") == 0)) {

      TRACE_INT("%s ", keyword);
      /*
       *  Don't allow egpnetsreachable if we have restriction
       *  lists.  If we have no restriction lists we will
       *  allow it as a "soft" level of restriction.
       */
      if ((strcmp(keyword, "egpnetsreachable") == 0) &&
          ((announcethesenets != 0) || (donotannounce != 0))) {
        printf("ignored with announce or noannounce restrictions.\n");
        goto nextone;
      }
      /*
       * read successive net strings into destination until end of line
       */
      morenets = TRUE;
      while ((c = fgetc(fp)) != EOF && morenets) {
        switch (c) {
          case '\n':
            (void) ungetc(c, fp);      /* end line read below */
            (void) ungetc('\015', fp); /* push back CR after LF */
          case '#':		/* start comment */
            morenets = FALSE;
            TRACE_INT("\n");
            break;
          case ' ':
          case '\t':
            break;		/* next character */
          default:
            (void) ungetc(c, fp);
            (void) fscanf(fp, "%s", netname);
            TRACE_INT("%s ", netname);
            if (!getnetorhostname((strcmp(keyword,"egpnetsreachable") == 0) ?
                                     "net" : "host", netname, &netaddr)) {
              syslog(LOG_WARNING, "NRadvise_init: invalid name or address %s",
                        netname);
              error = TRUE;
            }
            else {
              struct advlist *al;
              int rprotoflag = 0;
              struct interface *ifp;

              if (strcmp(keyword, "noripoutinterface") == 0)
                rprotoflag = IFF_NORIPOUT;
              if (strcmp(keyword, "noripfrominterface") == 0)
                rprotoflag = IFF_NORIPIN;
              if (strcmp(keyword, "nohellooutinterface") == 0)
                rprotoflag = IFF_NOHELLOOUT;
              if (strcmp(keyword,"nohellofrominterface") == 0)
                rprotoflag = IFF_NOHELLOIN;
              if (strcmp(keyword,"passiveinterfaces") == 0)
                rprotoflag = IFF_NOAGE;
              if (rprotoflag != 0) {
                if ((ifp = if_ifwithaddr(&netaddr)) <= (struct interface *)0) {
                  syslog(LOG_WARNING, "bad interface in control list.\n");
                  error = TRUE;
                }
                else
                  ifp->int_flags |= rprotoflag;
              }
              else {
                if ((al = (struct advlist *)malloc(sizeof
                              (struct advlist))) == NULL) {
                  syslog(LOG_EMERG, "out of memory\n");
                  exit(1);
                }
                al->next = templist;
                if (templist == NULL)
                  tailof = al;
                templist = al;
                al->destnet = netaddr.sin_addr;
              }
            }
        }			/* end switch c */
      }			/* end while morenets */
      if (strcmp(keyword, "egpnetsreachable") == 0) {
        n_remote_nets++;
        if (adlist)
          tailof->next = adlist;
        adlist = templist;
      }
      else if (strcmp(keyword, "sourceripgateways") == 0) {
        if (srcriplist)
          tailof->next = srcriplist;
        srcriplist = templist;
      }
      else if (strcmp(keyword, "sourcehellogateways") == 0) {
        if (srchellolist)
          tailof->next = srchellolist;
        srchellolist = templist;
      }
      else if (strcmp(keyword, "trustedhellogateways") == 0) {
        if (trustedhelloerlist)
          tailof->next = trustedhelloerlist;
        trustedhelloerlist = templist;
      }
      else if (strcmp(keyword, "trustedripgateways") == 0) {
        if (trustedripperlist)
          tailof->next = trustedripperlist;
        trustedripperlist = templist;
      }
      templist = (struct advlist *) NULL;
    }				/* end if "THE WORLD" */
nextone:
    do c = fgetc(fp); while (c != '\n' && c != EOF);  /* next line */
  } 				/*end while*/
  if (error) {
    syslog(LOG_EMERG, "rt_NRadvise_init: %s: initialization error\n",
              EGPINITFILE);
    quit();
  }
}

init_options(fp)
	FILE *fp;
{
  char keyword[MAXHOSTNAMELENGTH+1];
  char option[MAXHOSTNAMELENGTH+1];
  int	route_proto = FALSE,
	c,
	error = FALSE,
	moreopts,
	egp_conv = 0;

  /*
   *  standard initializations - might as well be done here
   */
  doing_rip = doing_hello = FALSE;
  doing_egp = TRUE;
  rip_supplier = hello_supplier = -1;
  rip_gateway = hello_gateway = FALSE;
  rip_default = hello_default = 0;
  rip_pointopoint = FALSE;
  hello_pointopoint = FALSE;
  conv_factor = HOPCNT_INFINITY;
  rt_maxage = RT_MINAGE;
  egpsleep = MINHELLOINT + HELLOMARGIN;
  n_remote_nets = 0;
  /*
   * initialize the hello_default net.
   */
  bzero((char *)&hello_dfltnet, sizeof(hello_dfltnet));
  hello_dfltnet.sin_family = AF_INET;
  hello_dfltnet.sin_addr.s_addr = htonl((u_long)HELLO_DEFAULT);

  rewind(fp);

  TRACE_INT("init_options: Reading configuration protocol options:\n");

  while(fscanf(fp, "%s", keyword) != EOF) {
    route_proto = 0;
    if (strcmp(keyword, "RIP") == 0)
      route_proto = IPPROTO_RIP;
    else if (strcmp(keyword, "HELLO") == 0)
      route_proto = IPPROTO_HELLO;
    else if (strcmp(keyword, "EGP") == 0)
      route_proto = IPPROTO_EGP;
    else if (strcmp(keyword, "defaultegpmetric") == 0) {
      route_proto++;
      egp_conv++;
    }

    if (route_proto != 0) {
      TRACE_INT("%s options: ", keyword);
      moreopts = TRUE;
      while (moreopts && (c = fgetc(fp)) != EOF) {
        switch (c) {
          case '\n':
            (void) ungetc(c, fp);      /* end line read below */
            (void) ungetc('\015', fp); /* push back CR after LF */
          case '#':		       /* start comment */
            moreopts = FALSE;
            TRACE_INT("\n");
            break;
          case ' ':
          case '\t':
            break;			/* next character */
          default:
            (void) ungetc(c, fp);
            (void) fscanf(fp, "%s", option);
            TRACE_INT("%s ", option);
            if (egp_conv) {
              if ((conv_factor = atoi(option)) < 0 ||
                  conv_factor > HOPCNT_INFINITY - 1) {
                error = TRUE;
                syslog(LOG_WARNING, "EGP conversion factor error - %d\n",
                          conv_factor);
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "yes") == 0) {
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = TRUE;
                  break;
                case IPPROTO_HELLO:
                  doing_hello = TRUE;
                  break;
                case IPPROTO_EGP:
                  doing_egp = TRUE;
                  break;
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "no") == 0) {
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = FALSE;
                  break;
                case IPPROTO_HELLO:
                  doing_hello = FALSE;
                  break;
                case IPPROTO_EGP:
                  doing_egp = FALSE;
                  break;
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "quiet") == 0) {
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = TRUE;
                  rip_supplier = FALSE;
                  break;
                case IPPROTO_HELLO:
                  doing_hello = TRUE;
                  hello_supplier = FALSE;
                  break;
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "supplier") == 0) {
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = TRUE;
                  rip_supplier = TRUE;
                  break;
                case IPPROTO_HELLO:
                  doing_hello = TRUE;
                  hello_supplier = TRUE;
                  break;
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "pointopoint") == 0) {
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = TRUE;
                  rip_supplier = TRUE;
                  rip_pointopoint = TRUE;
                  break;
                case IPPROTO_HELLO:
                  doing_hello = TRUE;
                  hello_supplier = TRUE;
                  hello_pointopoint = TRUE;
                  break;
              }
              moreopts = FALSE;
              break;
            }
            if (strcmp(option, "gateway") == 0) {
              (void) fscanf(fp, "%s", option);
              switch (route_proto) {
                case IPPROTO_RIP:
                  doing_rip = TRUE;
                  rip_supplier = TRUE;
                  rip_gateway = TRUE;
                  rip_default = atoi(option);
                  if (rip_default < 0 || rip_default >= RIPHOPCNT_INFINITY) {
                    error = TRUE;
                    syslog(LOG_WARNING, "RIP default metric error - %d\n",
                              rip_default);
                  }
                  if (!(error))
                    TRACE_INT("%d ", rip_default);
                  break;
                case IPPROTO_HELLO:
                  doing_hello = TRUE;
                  hello_supplier = TRUE;
                  hello_gateway = TRUE;
                  hello_default = atoi(option);
                  if (hello_default < 0 || hello_default >= DELAY_INFINITY) {
                    error = TRUE;
                    syslog(LOG_WARNING, "HELLO default metric error - %d\n",
                              hello_default);
                  }
                  if (!(error))
                    TRACE_INT("%d ",hello_default);
                  break;
                }
                moreopts = FALSE;
                break;
            }
        }			/* end switch c */
      }			/* end while moreopts */
      TRACE_INT("\n");
    }				/* end if "route_proto" */
    do c = fgetc(fp); while (c != '\n' && c != EOF);  /* next line */
    
  } 				/*end while*/
  if (error) {
    syslog(LOG_EMERG, "init_options: %s: initialization error\n", EGPINITFILE);
    quit();
  }
}
