
/*
*   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
*			CORNELL UNIVERSITY
*
*      Portions of this software may fall under the following
*      copyrights: 
*
*	Copyright (c) 1983 Regents of the University of California.
*	All rights reserved.  The Berkeley software License Agreement
*	specifies the terms and conditions for redistribution.
*
*	The copyright (c) of the EGPUP User Process, Paul Kirton and
*	ISI, August 23, 1984
*
*/

/*
*	The following acknowledgments and thanks apply:
*
*	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
*	development of the HELLO routines used in this program.
*
*	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
*	thorough linting, profiling, and performance enhancements to
*	this program.
*
*	Charles Hedrick (hedrick@topaz.rutgers.edu),
*	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
*	and any other BETA testers who have helped in finding bugs.
*
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
*
*
*  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
*	    and DCN's HELLO routing Protocol.  HELLO output routines
*	    were developed and originally written at the University of
*	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: rt_table.c,v 1.3.1.17 88/01/09 19:01:56 jch Exp $
 *
 */

#ifndef	lint
static char *rcsid = "$Header: rt_table.c,v 1.3.1.17 88/01/09 19:01:56 jch Exp $";
#endif	not lint

 /*
 * Some modified from Routing Table Management Daemon routed/tables.c.
 *
 * Functions: rt_lookup, rt_add, rt_change, rt_delete,
 *	      rt_default, rt_time, rt_unreach, rt_redirect,
 *	      rt_check_default, rt_find, addrouteforif,
 *            init_hwin, add_win
 */

#include "include.h"

/*
 * rt_lookup() looks up a destination network for an exact match.
 */

struct rt_entry *
rt_lookup(table, dst)
	int table;
	struct sockaddr_in *dst;
{
  register struct rt_entry *rt;
  register struct rthash *rh;
  register unsigned hash;
  struct afhash h;

  if (dst->sin_family != AF_INET)
    return(0);
		
  (*afswitch[dst->sin_family].af_hash)(dst, &h);
  if (table == HOSTTABLE) {
    hash = h.afh_hosthash;
    rh = &hosthash[hash & ROUTEHASHMASK];
  }
  else {
    hash = h.afh_nethash;
    rh = &nethash[hash & ROUTEHASHMASK];
  }
  for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
    if (rt->rt_hash != hash)
      continue;
    if (equal(&rt->rt_dst, dst) && (rt->rt_state & table))
      return (rt);
  }
  return (0);
}

/*
 * rt_add() adds a route to either the interior or exterior routing tables.
 */

rt_add(table, dst, gate, metric, state, proto, fromproto)
	int table;
	struct sockaddr *dst, *gate;
	int metric, state, proto, fromproto;
{
  register struct rt_entry *rt;
  struct rthash *rh;
  unsigned hash;
  int af = AF_INET;
  struct afhash h;

  /*
   * take care of routes pointing back to the localhost interface.
   * We don't care about them and will ignore them in the kernel forever.
   */
  if ((unsigned)gd_inet_netof(sock_inaddr(gate)) == (unsigned)LOOPBACKNET)
    return(1);

  (*afswitch[af].af_hash)((struct sockaddr_in *)dst, &h);
  if (table == HOSTTABLE) {
    hash = h.afh_hosthash;
    rh = &hosthash[hash & ROUTEHASHMASK];
  }
  else {
    hash = h.afh_nethash;
    rh = &nethash[hash & ROUTEHASHMASK];
  }
  rt = (struct rt_entry *)malloc(sizeof(*rt));
  if (rt == 0) {
    syslog(LOG_ERR, "rt_add: malloc: out of memory\n");
    return(0);
  }
  rt->rt_hash = hash;
  rt->rt_dst = *dst;
  /*
   * set local part of dest addr zero for nets
   */
  if ((gd_inet_lnaof(in_addr_ofs(&rt->rt_dst)) != 0) && (table != HOSTTABLE)) {
    in_addr_ofs(&rt->rt_dst)
	    = gd_inet_makeaddr(gd_inet_netof(in_addr_ofs(&rt->rt_dst)),0,TRUE);
  }
  rt->rt_router = *gate;
  rt->rt_metric = metric;
  rt->rt_timer = 0;
  rt->rt_flags = RTF_UP;
  if (table == HOSTTABLE)
    rt->rt_flags |= RTF_HOST;
  rt->rt_state = state | RTS_CHANGED | table;
  if ((htonl(inet_wholenetof(sock_inaddr(&rt->rt_dst))) !=
      sock_inaddr(&rt->rt_dst).s_addr) && ((rt->rt_flags & RTF_HOST) == 0))
    rt->rt_state |= RTS_SUBNET;
  rt->rt_proto = proto;
  rt->rt_fromproto = fromproto;
  rt->rt_ifp = if_withnet(&rt->rt_router);
  rt->rt_announcelist = (struct restrictlist *)NULL;
  rt->rt_noannouncelist = (struct restrictlist *)NULL;
  rt->rt_listenlist = (struct restrictlist *)NULL;
  rt->rt_srclisten = (struct restrictlist *)NULL;
  if (rt->rt_proto == RTPROTO_HELLO)	/* initialize hello history window. */
    init_hwindow(&rt->rt_hwindow, metric);
  if (donotlisten != 0) {
    if ((rt->rt_listenlist = control_lookup(RT_NOLISTEN, dst)) <= (struct restrictlist *)0)
      rt->rt_listenlist = (struct restrictlist *)NULL;
    else /* see if there is a restriction before adding */
      if (rt->rt_proto != RTPROTO_EGP) { /* no restrictions in EGP */
        if (is_valid_in(rt, rt->rt_proto, rt->rt_ifp, gate) == 0) {
          free((char *) rt);
          return(0);
        }
      }
  }
  if (islisten != 0) {
    if ((rt->rt_srclisten = control_lookup(RT_SRCLISTEN, dst)) <= (struct restrictlist *)0)
      rt->rt_srclisten = (struct restrictlist *)NULL;
    else /* see if there is a restriction before adding */
      /*
       * No restrictions in EGP.  Also, don't prevent
       * rt_default(GATEWAY) from adding default route.
       */
      if ((rt->rt_proto != RTPROTO_EGP) && ((rt->rt_state & RTS_NOTINSTALL) == 0)) {
        if (is_valid_in(rt, rt->rt_proto, rt->rt_ifp, gate) == 0) {
          free((char *) rt);
          return(0);
        }
      }
  }
  if ((glob_announcethesenets != 0) || (announcethesenets != 0)) {
    if ((rt->rt_announcelist = control_lookup(RT_ANNOUNCE, dst)) <= (struct restrictlist *)0) {
      rt->rt_announcelist = (struct restrictlist *)NULL;
      if ((table == INTERIOR) || (table == HOSTTABLE)) {
        if (glob_announcethesenets != 0)
          syslog(LOG_WARNING,"invalid %s route %s not announced",
                               (table == INTERIOR) ? "net" : "host",
                               inet_ntoa(sock_inaddr(&rt->rt_dst)));
        else
          syslog(LOG_WARNING,
                    "no announcelist for %s route %s, may be restricted ",
                               (table == INTERIOR) ? "net" : "host",
                               inet_ntoa(sock_inaddr(&rt->rt_dst)));
      }
    }
  }
  if ((glob_donotannounce != 0) || (donotannounce != 0)) {
    if ((rt->rt_noannouncelist = control_lookup(RT_NOANNOUNCE, dst)) <= (struct restrictlist *)0)
      rt->rt_noannouncelist = (struct restrictlist *)NULL;
    else
      if ((table == INTERIOR) || (table == HOSTTABLE)) {
        if (glob_announcethesenets != 0)
          syslog(LOG_WARNING,"invalid %s route %s not announced",
                              (table == INTERIOR) ? "net" : "host",
                              inet_ntoa(sock_inaddr(&rt->rt_dst)));
        else
          syslog(LOG_WARNING,"noannouncelist for %s route %s",
                               (table == INTERIOR) ? "net" : "host",
                               inet_ntoa(sock_inaddr(&rt->rt_dst)));
      }
  }
  if (table == EXTERIOR) {
    rt->rt_flags |= RTF_GATEWAY;
    TRACE_ACTION(ADD EXT, rt)
  }
  else {
    if (metric)
      rt->rt_flags |= RTF_GATEWAY;
    if (table == INTERIOR) {
      TRACE_ACTION(ADD INT, rt);
    } else {
      TRACE_ACTION(ADD HOST, rt);
    }
  }
  insque(rt, rh);
  /*
   * If the ioctl fails because the gateway is unreachable
   * from this host, discard the entry.  This should only
   * occur because of an incorrect entry in /etc/gated.conf.
   */
  if (install && !(rt->rt_state & RTS_NOTINSTALL)) {
    if (ioctl(s, SIOCADDRT, (char *)&rt->rt_rt) < 0) {
      char msg[120];
			
      (void) sprintf(msg, "rt_add: SIOCADDRT dst %s gw %s flags %x",
                    inet_ntoa(((struct sockaddr_in *)&rt->rt_dst)->sin_addr),
                    inet_ntoa(((struct sockaddr_in *)&rt->rt_router)->sin_addr),
                    rt->rt_flags);
      p_error(msg);
      if (errno == ENETUNREACH) {
        TRACE_ACTION(DELETE, rt);
        remque(rt);
        free((char *)rt);
      }
    }
  }
  return(1);
}

/*
 * rt_change() changes a route &/or notes that an update was received.
 * returns 1 if change made
 */

rt_change(rt, gate, metric, proto, fromproto)
	struct rt_entry *rt;
	struct sockaddr *gate;
	short metric;
	int proto, fromproto;
{
  int doioctl = 0, metricchanged = 0, protochanged = 0;
  int ifchanged = 0, fromprotochanged = 0;
  struct interface *t_ifp;
  struct rtentry oldroute;

  rt->rt_state |= RTS_CHANGED;		/* ensures route age reset */
  if (!equal(&rt->rt_router, gate))
    doioctl++;
  if (metric != rt->rt_metric) {
    metricchanged++;
    rt->rt_metric = metric;
  }
  if (fromproto != rt->rt_fromproto) {
    fromprotochanged++;
    rt->rt_fromproto = fromproto;
  }
  if (proto != rt->rt_proto) {
    protochanged++;
    rt->rt_proto = proto;
    if (proto & RTPROTO_HELLO)
      init_hwindow(&rt->rt_hwindow, rt->rt_metric);
  }
  if (rt->rt_ifp != (t_ifp = if_withnet(gate))) {
    rt->rt_ifp = t_ifp;
    ifchanged++;
  }
  if (doioctl) {
    oldroute = rt->rt_rt;
    rt->rt_router = *gate;
    if (install && !(rt->rt_state & RTS_NOTINSTALL)) {
      if (ioctl(s, SIOCADDRT, (char *)&rt->rt_rt) < 0) {
        char msg[120];

        (void) sprintf(msg, "rt_change: SIOCADDRT dst %s gw %s flags %x",
                      inet_ntoa(((struct sockaddr_in *)&rt->rt_dst)->sin_addr),
                      inet_ntoa(((struct sockaddr_in *)
                                                  &rt->rt_router)->sin_addr),
                      rt->rt_flags);
        p_error(msg);
      }
      if (ioctl(s, SIOCDELRT, (char *)&oldroute) < 0) {
        char msg[120];

        (void) sprintf(msg, "rt_change: SIOCDELRT dst %s gw %s flags %x",
                      inet_ntoa(((struct sockaddr_in *)&rt->rt_dst)->sin_addr),
                      inet_ntoa(((struct sockaddr_in *)
                                             &oldroute.rt_gateway)->sin_addr),
                      rt->rt_flags);
        p_error(msg);
      }
    }
  }
  if (doioctl || metricchanged || protochanged || ifchanged ||
      fromprotochanged) {
    TRACE_ACTION(CHANGE, rt);
    return(1);
  }
  return(0);
}

/*
 * rt_delete deletes a route from the routing table.
 */

rt_delete(rt, just_kernel)
	struct rt_entry *rt;
	int just_kernel;
{
  TRACE_ACTION(DELETE, rt);
  if (install && !(rt->rt_state & RTS_NOTINSTALL) &&
      ioctl(s, SIOCDELRT, (char *)&rt->rt_rt) < 0) {
    char msg[120];
		
    (void) sprintf(msg,"rt_delete SIOCDELRT dst %s gw %s flags %x",
                  inet_ntoa(((struct sockaddr_in *)&rt->rt_dst)->sin_addr),
                  inet_ntoa(((struct sockaddr_in *)&rt->rt_router)->sin_addr),
                  rt->rt_flags);
    p_error(msg);
  }
  if (!(just_kernel)) {
    remque(rt);
    free((char *)rt);
  }
}

/*
 * rt_default() adds or deletes default route in kernel.
 * also adds a default in it's own tables only for the RIP/HELLO
 * gateway.
 */

rt_default(cmd)
	char *cmd;
{
  struct rt_entry *rt;
  struct sockaddr_in	defaultdst;
  int keepinstall;

  bzero((char *)&defaultdst, sizeof(defaultdst));
  defaultdst.sin_family = AF_INET;
  defaultdst.sin_addr.s_addr = DEFAULTNET;

  if (strcmp( cmd, "GATEWAY") == 0) {
    keepinstall = install;
    install = TRUE;
    rt = rt_lookup((int)EXTERIOR, &defaultdst);
    if (rt)
      rt_delete(rt, KERNEL_INTR);	/* delete old default route */
    rt = rt_lookup((int)INTERIOR, &defaultdst);
    if (rt)
      rt_delete(rt, KERNEL_INTR);	/* delete old default route */
    /*
     * this is only a false route, so just pick the first interface address
     * in the ifnet list to satisfy the gateway.
     */
    install = FALSE;
    (void) rt_add((int)INTERIOR, (struct sockaddr *)&defaultdst,
            &ifnet->int_addr, 0, RTS_PASSIVE|RTS_NOTINSTALL, 0, 0);
    install = keepinstall;
    rt_default_status = NOTINSTALLED;  /* default is NOT in kernel */
    TRACE_ACTION(GATEWAY DEFAULT, rt_lookup((int)INTERIOR, &defaultdst));
  }
  else {
    rt = rt_lookup((int)EXTERIOR, &defaultdst);
    if (rt == NULL)
      syslog(LOG_NOTICE, "rt_default: no active default route\n");
    else
      if (strcmp( cmd, "DELETE") == 0) {
        TRACE_ACTION(DELETE DEFAULT, rt);
        rt_default_status = NOTINSTALLED;
        if (install)
          if (ioctl(s, SIOCDELRT, (char *)&rt->rt_rt) < 0)
            p_error("rt_default: SIOCDELRT");
      }
    else
      if (strcmp(cmd, "ADD") == 0) {
        TRACE_ACTION(ADD DEFAULT, rt);
        rt_default_status = INSTALLED;
        if (install)
          if (ioctl(s, SIOCADDRT, (char *)&rt->rt_rt) < 0)
            p_error("rt_default: SIOCADDRT");
      }
  }
  return;
}

/*
 * rt_time() increments the age of all routes in the routing table
 * If any EGP routes are older than rt_maxage (set in egpstime()) the routes are
 * deleted. If RIP/HELLO routes are greater that GARBAGE_TIME, delete them.
 */

rt_time()
{
  struct rthash	*rh;
  struct rthash	*base = hosthash;
  struct rt_entry *rt, *old_rt;
  static int last_time;
  int old_routes = FALSE;
  int timetobroadcast, hellotimetobroad;
  int doinghost = 1;
  int do_flash_update = FALSE;
  int do_helloflash_update = FALSE;

  timetobroadcast = rip_supplier && ((gatedtime % RIP_INTERVAL) == 0);
  hellotimetobroad = hello_supplier && ((gatedtime % HELLO_TIMERRATE) == 0);

onemoretime:
  for (rh = base; rh < &base[ROUTEHASHSIZ]; rh++) {
    for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
      if (rt->rt_state & RTS_CHANGED) {    /* recently updated */
        rt->rt_state &= ~RTS_CHANGED;
        rt->rt_timer = 0;
        if (((rt->rt_state & RTS_PASSIVE) == 0) &&
            (rt->rt_state & RTS_INTERIOR)) {
	  if (doing_rip) {
            if (rip_supplier && !timetobroadcast)
              do_flash_update = TRUE;
          }
	  if (doing_hello) {
            if (hello_supplier && !hellotimetobroad)
              do_helloflash_update = TRUE;
          }
        }
      }
      else {
        if (!(rt->rt_state & RTS_PASSIVE))
          rt->rt_timer += gatedtime - last_time;
      }
      /*
       *  is route too old?
       */
      if ((rt->rt_proto & RTPROTO_EGP) || (rt->rt_state & RTS_EXTERIOR)) {
        if (rt->rt_timer >= rt_maxage - (FINAL_DELETE-EXPIRE_TIME))
          rt->rt_metric = TDHOPCNT_INFINITY;
        if (rt->rt_timer >= rt_maxage) {
          old_routes = TRUE;
          old_rt = rt;
          rt = rt->rt_back;
          if (rt->rt_state & RTS_EXTERIOR) {
            TRACE_RT("OLD EXT: ");
          } else {
            TRACE_RT("OLD EGP INT: ");
          }
          rt_delete(old_rt, KERNEL_INTR);
        }
      }
      else {   /* Any other IGP route.... */
        if (rt->rt_timer >= EXPIRE_TIME) {
          switch (rt->rt_proto) {
            case RTPROTO_RIP:
              rt->rt_metric = DELAY_INFINITY;
              break;
            case RTPROTO_HELLO:
              rt->rt_metric = DELAY_INFINITY;
              add_win(&rt->rt_hwindow, rt->rt_metric);
              break;
            default:
              rt->rt_metric = TDHOPCNT_INFINITY;
          }
        }
        if (rt->rt_timer >= FINAL_DELETE) {
          old_routes = TRUE;
          old_rt = rt;
          rt = rt->rt_back;
          if (doinghost) {
            TRACE_RT("OLD HOST: ");
          } else {
            TRACE_RT("OLD INT: ");
          }
          rt_delete(old_rt, KERNEL_INTR);
        }
      }
    }
  }
  if (doinghost) {
    base = nethash;
    doinghost = 0;
    goto onemoretime;
  }
  if (do_flash_update)  /* there was one or more changes, update RIP */
    toall(supply);
  if (do_helloflash_update)  /* there was one or more changes, update HELLO */
    hellojob();
  if (old_routes) {
    rt_check_default();
    TRACE_RT("rt_time: above old routes deleted %s\n", strtime);
  }
  last_time = gatedtime;
  return;
}

/*
 * rt_check_default() checks whether there are currently any EGP neighbors
 * that are both acquired and reachable and if not, stops the announcement
 * of default, if the gateway is a rip_gateway or hello_gateway.
 */

rt_check_default()
{
  int neigh_up = FALSE;
  struct egpngh	*ngp;
  struct sockaddr_in defaultdst;
  struct rt_entry *rt;
  int saveinstall;

  if (rt_default_status != NOTINSTALLED)
    return;
  if ( (!doing_egp) || (!(rip_gateway) && !(hello_gateway)) )
    return;
  for (ngp = egpngh; ngp != NULL; ngp = ngp->ng_next) {
    if (ngp->ng_state == NEIGHBOR && !(ngp->ng_reach & NG_DOWN)) {
      neigh_up = TRUE;
      break;
    }
  }
  bzero((char *)&defaultdst, sizeof(defaultdst));
  defaultdst.sin_family = AF_INET;
  defaultdst.sin_addr.s_addr = DEFAULTNET;

  if ((neigh_up == FALSE) && (nopeers == 0)) {
    rt = rt_lookup((int)INTERIOR, &defaultdst);
    if (rt == NULL)
      syslog(LOG_NOTICE, "rt_check_default: no active default route\n");
    else {
      saveinstall = install;
      install = FALSE;
      rt_delete(rt, KERNEL_INTR);
      install = saveinstall;
      nopeers = 1;
    }
  }
  else
    if ((neigh_up != FALSE) && (nopeers == 1)) {
      rt_default("GATEWAY");
      nopeers = 0;
    }
}

/*
 * rt_unreach() deletes all exterior routes from the routing table for a
 * specified gateway
 */

rt_unreach(gateway)
	struct in_addr	gateway;	/* internet address of gateway */
{
  struct rthash	*rh;
  struct rt_entry *rt, *unreach_rt;

  for (rh = nethash; rh < &nethash[ROUTEHASHSIZ]; rh++) {
    for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
      if ((sock_inaddr(&rt->rt_router).s_addr == gateway.s_addr) &&
          !(rt->rt_state & RTS_PASSIVE) && (rt->rt_state & RTS_EXTERIOR)) {
        unreach_rt = rt;
        rt = rt->rt_back;
        TRACE_RT("GW UNREACH: ");
        rt_delete(unreach_rt, KERNEL_INTR);
      }
    }
  }
  return;
}

/*
 * rt_redirect() changes the routing tables in response to an ICMP redirect 
 * message
 */

rt_redirect(dst, gateway, src, icmpcode)
	struct sockaddr	*dst, *gateway, *src;
	u_char icmpcode;
{
  int	saveinstall;
  struct	rt_entry  *rt;
  int	interior = 0;
  register struct interface *ifp;

  /* check gateway directly reachable */
  if (if_withnet(gateway) == NULL)
    return;
  if (src != NULL && if_ifwithaddr(src))
    return;
  if (if_ifwithaddr(gateway) || if_ifwithaddr(dst)) { /* a routing loop? */
    if (((rt = rt_lookup((int)INTERIOR, (struct sockaddr_in *)dst)) ||
        (rt = rt_lookup((int)EXTERIOR, (struct sockaddr_in *)dst)) ||
        (rt = rt_lookup((int)HOSTTABLE, (struct sockaddr_in *)dst))) &&
        (rt->rt_state & RTS_INTERFACE == 0))
      rt_delete(rt, KERNEL_INTR);
    return;			/* talking to ourselves */
  }
  saveinstall = install;
  install = FALSE;       /* route already in kernel */

  TRACE_RT("%s REDIRECT", icmpcode == ICMP_REDIRECT_NET ? "NET" : "HOST");
  if (src != NULL)
    TRACE_RT(" from %s", inet_ntoa(sock_inaddr(src)));
  TRACE_RT(": %s", inet_ntoa(sock_inaddr(dst)));
  TRACE_RT(" via %s:\n", inet_ntoa(sock_inaddr(gateway)));
	
  if (icmpcode == ICMP_REDIRECT_HOST) {
    rt = rt_lookup((int)HOSTTABLE, (struct sockaddr_in *)dst);
    if (src && rt && !equal(src, &rt->rt_router))
      goto invalid;
    if (rt) {
      if (!(rt->rt_state & IFF_INTERFACE) &&
          (rt_change(rt,gateway,rt->rt_metric,RTPROTO_REDIRECT,
           RTPROTO_REDIRECT) == 0))
        TRACE_RT(" tables not changed by HOST redirect\n");
    }
    else
      (void) rt_add((int)HOSTTABLE, dst, gateway, (TDHOPCNT_INFINITY - 100), 0,
                        RTPROTO_REDIRECT, RTPROTO_REDIRECT);
  }
  else {  /* is this an interior route? */
    for (ifp = ifnet; ifp; ifp = ifp->int_next)
      if (inet_wholenetof(sock_inaddr(dst)) ==
                   inet_wholenetof(in_addr_ofs(&ifp->int_addr))) {
        interior++;
        break;
      }
    if (rt = rt_lookup((int)INTERIOR, (struct sockaddr_in *)dst)) {
      if (src && !equal(src, &rt->rt_router))
        goto invalid;
      if (!(rt->rt_state & IFF_INTERFACE) &&
          (rt_change(rt,gateway,rt->rt_metric,RTPROTO_REDIRECT,RTPROTO_REDIRECT) == 0))
        TRACE_RT(" tables not changed\n");
    }
    else
      if (rt = rt_lookup((int)EXTERIOR, (struct sockaddr_in *)dst)) {
        if (src && !equal(src, &rt->rt_router))
          goto invalid;
        if (!(rt->rt_state & IFF_INTERFACE) &&
            (rt_change(rt,gateway,rt->rt_metric,RTPROTO_REDIRECT,
             RTPROTO_REDIRECT) == 0))
          TRACE_RT(" tables not changed\n");
      } else
          if (interior)
            (void) rt_add((int)INTERIOR, dst, gateway, TDHOPCNT_INFINITY - 100,
                          0, RTPROTO_REDIRECT, RTPROTO_REDIRECT);
          else
            (void) rt_add((int)EXTERIOR, dst, gateway, TDHOPCNT_INFINITY - 100,
                          0, RTPROTO_REDIRECT, RTPROTO_REDIRECT);
  }
invalid:
  install = saveinstall;
  return;
}

/*
 * rt_find() looks up a destination as the kernel would
 */

struct rt_entry *
rt_find(dst)
	struct sockaddr *dst;
{
  register struct rt_entry *rt;
  register struct rthash *rh;
  register unsigned hash;
  struct afhash h;
  int af = AF_INET;
  int doinghost = 1;
  int (*match)();

  (*afswitch[af].af_hash)((struct sockaddr_in *)dst, &h);
  hash = h.afh_hosthash;
  rh = &hosthash[hash & ROUTEHASHMASK];
  match = afswitch[af].af_netmatch;

nexttable:
  for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
    if (rt->rt_hash != hash)
      continue;
    if (doinghost) {
      if (equal(&rt->rt_dst, dst))
        return (rt);
    }
    else {
      if ((*match)(&rt->rt_dst, dst))
        return(rt);
    }
  }
  if (doinghost) {
    doinghost = 0;
    hash = h.afh_nethash;
    rh = &nethash[hash & ROUTEHASHMASK];
    goto nexttable;
  }
  return (0);
}

addrouteforif(ifp)
	struct interface *ifp;
{
  struct sockaddr_in net;
  struct sockaddr *dst;
  int table = INTERIOR;
  struct rt_entry *rt;

  if (ifp->int_flags & IFF_POINTOPOINT) {
    dst = &ifp->int_dstaddr;
    table = HOSTTABLE;
  }
  else {
    bzero((char *)&net, sizeof (net));
    net.sin_family = AF_INET;
    net.sin_addr = gd_inet_makeaddr((int)ifp->int_subnet, 0, 1);
    dst = (struct sockaddr *)&net;
  }
  rt = rt_find(dst);
  if (rt && (rt->rt_state & RTS_INTERFACE))
    return;
  if (rt)
    rt_delete(rt, KERNEL_INTR);
  ifp->int_transitions++;
  TRACE_INT("reinstalling route for interface %s \n", ifp->int_name);
  syslog(LOG_NOTICE, "reinstalling route for interface %s \n", ifp->int_name);
  (void) rt_add(table, dst, &ifp->int_addr, 0, RTS_INTERFACE, RTPROTO_DIRECT,
                RTPROTO_DIRECT);
}

/*
 * initialize the sliding HELLO history window.
 */

init_hwindow(hwp, tdelay)
	struct hello_win *hwp;
	int tdelay;
{
  int msf = 0;

  while (msf < HWINSIZE)
    hwp->h_win[msf++] = DELAY_INFINITY;
  hwp->h_index = 0;
  hwp->h_min = tdelay;
  hwp->h_min_ttl = 0;
  hwp->h_win[0] = tdelay;
}

/*
 * add a HELLO derived time delay to the route entries HELLO window.
 */

add_win(hwp, tdelay)
	struct hello_win *hwp;
	int tdelay;
{
  int msf, t_index = 0;

  hwp->h_index++;
  if (hwp->h_index >= HWINSIZE)
    hwp->h_index = 0;
  hwp->h_win[hwp->h_index] = tdelay;
  if (tdelay > hwp->h_min)
    hwp->h_min_ttl++;
  else {
    hwp->h_min = tdelay;
    hwp->h_min_ttl = 0;
  }
  if (hwp->h_min_ttl >= HWINSIZE) {
    hwp->h_min = DELAY_INFINITY;
    for (msf = 0; msf < HWINSIZE; msf++)
      if (hwp->h_win[msf] <= hwp->h_min) {
        hwp->h_min = hwp->h_win[msf];
        t_index = msf;
      }
    hwp->h_min_ttl = 0;
    if (t_index < hwp->h_index)
      hwp->h_min_ttl = hwp->h_index - t_index;
    else
      if (t_index > hwp->h_index)
        hwp->h_min_ttl = HWINSIZE - (t_index - hwp->h_index);
  }
}

/*
 * map HELLO time delay to RIP metric  OR
 * RIP metric to HELLO time delay (in miliseconds)
 */

mapmetric(proto, cost)
	int proto;
	int cost;
{
  if (proto == HELLO_TO_RIP) {
    if (cost == 0)
      return(0);
    if ((cost <= 100) && (cost > 0))
      return(1);
    if ((cost <= 148) && (cost > 0))
      return(2);
    if ((cost <= 219) && (cost > 0))
      return(3);
    if ((cost <= 325) && (cost > 0))
      return(4);
    if ((cost <= 481) && (cost > 0))
      return(5);
    if ((cost <= 713) && (cost > 0))
      return(6);
    if ((cost <= 1057) && (cost > 0))
      return(7);
    if ((cost <= 1567) && (cost > 0))
      return(8);
    if ((cost <= 2322) && (cost > 0))
      return(9);
    if ((cost <= 3440) && (cost > 0))
      return(10);
    if ((cost <= 5097) && (cost > 0))
      return(11);
    if ((cost <= 7552) && (cost > 0))
      return(12);
    if ((cost <= 11190) && (cost > 0))
      return(13);
    if ((cost <= 16579) && (cost > 0))
      return(14);
    if ((cost <= 24564) && (cost > 0))
      return(15);
    if ((cost <= 30000) && (cost > 0))
      return(16);
    /* > 30000 or negative! */
    return(16);
  }
  if (proto == RIP_TO_HELLO) {
    switch (cost) {
      case 0:  return(0);
      case 1:  return(100);
      case 2:  return(148);
      case 3:  return(219);
      case 4:  return(325);
      case 5:  return(481);
      case 6:  return(713);
      case 7:  return(1057);
      case 8:  return(1567);
      case 9:  return(2322);
      case 10: return(3440);
      case 11: return(5097);
      case 12: return(7552);
      case 13: return(11190);
      case 14: return(16579);
      case 15: return(24564);
      case 16: return(30000);
      default: return(30000);
    }
  }
  return(30000);   /* just in case some screwup, better to return INFINITY */
}
