/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY
 *
 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 */

#ifndef	lint
static char *rcsid = "$Header: trace.c,v 1.3.1.21 88/02/11 21:53:32 jch Locked $";
#endif	not lint

/* trace_egp.c
 *
 * Tracing functions for route changes, received packets and EGP messages.
 *
 * Functions: traceaction (modified from routed/trace.c), tracercv, traceegp
 */


#define RIPCMDS

#include "include.h"
#include <sys/stat.h>

/*
 * traceaction() traces changes to the routing tables
 */
traceaction(fd, action, rt)
	FILE *fd;			/* trace logfile */
	char *action;
	struct rt_entry *rt;
{
  struct sockaddr_in *dst, *gate;
  static struct bits {
		int	t_bits;
		char	*t_name;
  } flagbits[] = {
		{ RTF_UP,	"UP" },
		{ RTF_GATEWAY,	"GATEWAY" },
		{ RTF_HOST,	"HOST" },
		{ 0 }
  }, statebits[] = {
		{ RTS_PASSIVE,	   "PASSIVE" },
		{ RTS_REMOTE,	   "REMOTE" },
		{ RTS_INTERFACE,   "INTERFACE" },
		{ RTS_CHANGED,	   "CHANGED" },
		{ RTS_NOTINSTALL,  "NOTINSTALL" },
		{ RTS_NOTADVISENR, "NOTADVISENR" },
		{ RTS_SUBNET,      "SUBNET" },
		{ RTS_POINTOPOINT, "POINTOPOINT" },
		{ RTS_STATIC,      "STATIC" },
		{ RTS_HOSTROUTE,   "HOSTROUTE" },
		{ RTS_INTERIOR,    "INTERIOR" },
		{ RTS_EXTERIOR,    "EXTERIOR" },
		{ 0 }
  }, protobits[] = {
		{ RTPROTO_EGP,	    "EGP" },
		{ RTPROTO_RIP,	    "RIP" },
		{ RTPROTO_HELLO,    "HELLO" },
		{ RTPROTO_KERNEL,   "KERNEL" },
		{ RTPROTO_REDIRECT, "REDIRECT" },
		{ RTPROTO_DIRECT,   "DIRECT" },
		{ 0 }
  };
  register struct bits *p;
  register int first;
  char *cp;

  if (fd == NULL)
    return;
  fprintf(fd, "%s ", action);
  dst = (struct sockaddr_in *)&rt->rt_dst;
  gate = (struct sockaddr_in *)&rt->rt_router;
  fprintf(fd, "dst %s, ", inet_ntoa(dst->sin_addr));
  fprintf(fd, "router %s, metric %d, proto ", inet_ntoa(gate->sin_addr),
          	 rt->rt_metric);
  cp = " %s";
  for (first = 1, p = protobits; p->t_bits > 0; p++) {
    if ((rt->rt_proto & p->t_bits) == 0)
      continue;
    fprintf(fd, cp, p->t_name);
    if (first) {
      cp = "|%s";
      first = 0;
    }
  }
  fprintf(fd, ", flags");
  cp = " %s";
  for (first = 1, p = flagbits; p->t_bits > 0; p++) {
    if ((rt->rt_flags & p->t_bits) == 0)
      continue;
    fprintf(fd, cp, p->t_name);
    if (first) {
      cp = "|%s";
      first = 0;
    }
  }
  fprintf(fd, ", state");
  cp = " %s";
  for (first = 1, p = statebits; p->t_bits > 0; p++) {
    if ((rt->rt_state & p->t_bits) == 0)
      continue;
    fprintf(fd, cp, p->t_name);
    if (first) {
      cp = "|%s";
      first = 0;
    }
  }
  if (rt->rt_proto & RTPROTO_HELLO)
    printf(" hwin_min %d ", rt->rt_hwindow.h_min);
  printf(" fromproto ");
  cp = " %s";
  for (first = 1, p = protobits; p->t_bits > 0; p++) {
    if ((rt->rt_fromproto & p->t_bits) == 0)
      continue;
    fprintf(fd, cp, p->t_name);
    if (first) {
      cp = "|%s";
      first = 0;
    }
  }
  (void) putc('\n', fd);

  if ((rt->rt_announcelist != NULL) && (tracing & TR_INT)) {
    dst = (struct sockaddr_in *)&rt->rt_announcelist->rdst;
    fprintf(fd, "ANNOUNCE RESTRICT dst %s, ", inet_ntoa(dst->sin_addr));
    fprintf(fd, "metric %d, proto %d",rt->rt_announcelist->regpmetric,
                     rt->rt_announcelist->rproto);
    (void) putc('\n', fd);
  }
  if ((rt->rt_noannouncelist != NULL) && (tracing & TR_INT)) {
    dst = (struct sockaddr_in *)&rt->rt_noannouncelist->rdst;
    fprintf(fd, "NOANNOUNCE RESTRICT dst %s, ", inet_ntoa(dst->sin_addr));
    fprintf(fd, "metric %d, proto %d",rt->rt_noannouncelist->regpmetric,
                     rt->rt_noannouncelist->rproto);
    (void) putc('\n', fd);
  }
  (void) fflush(fd);
}


/*
 * Turn off tracing.
 */
traceoff(tracetype)
	int tracetype;
{
  int tracedel = 0;

  if (tracetype & RIP_TRACE) {
    tracedel |= (TR_UPDATE|TR_RIP);
  }
  if (tracetype & HELLO_TRACE) {
    tracedel |= (TR_UPDATE|TR_HELLO);
  }
  if (!tracing) {
    return;
  }
  tracing &= ~tracedel;  /* unflag added tracing options */
  if ((ftrace != NULL) && (tracetype & GEN_TRACE)) {
    TRACE_TRC("Tracing to %s suspended at %s\n", logfile, strtime);
    tracing = 0;
    (void) fflush(ftrace);
    (void) fflush(stdout);
    (void) fflush(stderr);
    (void) fclose(ftrace);
    (void) fclose(stdout);
    (void) fclose(stderr);
    ftrace = NULL;
    syslog(LOG_ERR, "tracing to %s suspended", logfile);
  }
  return;
}

/*
 * Turn on tracing.
 */
traceon(file, tracetype)
	char *file;
	int tracetype;
{
  struct stat stbuf;
  int traceadd = 0;

  switch (tracetype) {
    case RIP_TRACE:
      traceadd |= (TR_UPDATE|TR_RIP);
      break;
    case HELLO_TRACE:
      traceadd |= (TR_UPDATE|TR_HELLO);
      break;
    case GEN_TRACE:
      traceadd |= savetrace;
      break;
  }

  if ((ftrace == NULL) && (file != NULL)) {
    if ((stat(file, &stbuf) >= 0) && ((stbuf.st_mode & S_IFMT) != S_IFREG)) {
      return;
    }
    if ((ftrace = fopen(file, "a")) == NULL) {
      syslog(LOG_ERR, "error opening %s: %m", file);
      return;
    }
    if ( freopen(file, "a", stdout) == NULL ) {
      syslog(LOG_ERR, "error opening %s: %m", file);
      return;
    }
    if ( freopen(file, "a", stderr) == NULL ) {
      syslog(LOG_ERR, "error opening %s: %m", file);
      return;
    }
    syslog(LOG_ERR, "tracing to %s started", file);
    printf("\fTracing to %s started at %s\n", file, strtime);
    fflush(stdout);
  }
  tracing |= traceadd;
}

/*
 * trace packets received
 *
 * Only trace data part - EGP packets include IP header but ICMP packets do
 * not (but on 4.3BSD they DO, well it is passed to the RAW ICMP socket.
 */
tracercv(sock, protocol, packet, length, frompt)
	int sock, protocol, length;
	struct sockaddr_in *frompt;
	char *packet;
{
  struct sockaddr_in  sockname;
  int  socklen = sizeof (sockname), i;
  struct ip *pkt;
  int hlen;

  pkt = (struct ip *)packet;
  switch (protocol) {
    case IPPROTO_EGP: 
      hlen = pkt->ip_hl;
      hlen <<= 2;			/* IP header length in octets */
      TRACE_EGPPKT(EGP RECV, pkt->ip_src, pkt->ip_dst,
			(struct egppkt *)(packet+hlen), pkt->ip_len);
      break;
    case IMPLINK_IP: 	/* not implemented */
    case IPPROTO_ICMP: 	/* IP header not passed with ICMP messages */
      if (getsockname (sock, (char *)&sockname, &socklen))
        p_error ("tracercv: getsockname");
      printf("\nPacket received at time %s", strtime);
      printf("protocol %d, from addr %s",protocol,inet_ntoa(frompt->sin_addr));
      printf(", on interface addr %s\n", inet_ntoa(sockname.sin_addr));
      printf("Data octets (decimal):\n");
      for (i = 0; i < length; i++)
        printf ("%d ", packet[i]);
      printf ("\n");
      break;
  }
  (void) fflush(stdout);
  return;
}

/*
 * Trace EGP packet
 */
traceegp(comment, src, dst, egp, length)
	char  *comment;
	struct in_addr	src, dst;
	struct	egppkt	*egp;
	int	length;
{
  register u_char *data, *dataf;
  struct egppkt	*ep;

  printf("\n%s", comment);
  if (src.s_addr != INADDR_ANY)
    printf(" %s", inet_ntoa(src));
  printf(" -> %s %s", inet_ntoa( dst), strtime);

  printf("vers %d, typ %d, code %d, status %d, AS# %d, id %d",
		egp->egp_ver, egp->egp_type, egp->egp_code, egp->egp_status,
		ntohs(egp->egp_system), ntohs(egp->egp_id));
  if (length > sizeof( struct egppkt))
    switch (egp->egp_type) {
      case EGPACQ:
        printf(", hello int %d, poll int %d", 
                       ntohs(((struct egpacq *)egp)->ea_hint),
                       ntohs(((struct egpacq *)egp)->ea_pint));
        break;
      case EGPPOLL:
        printf(", src net %s", inet_ntoa(((struct egppoll *)egp)->ep_net));
        break;
      case EGPNR:
        printf(", src net %s, #int %d, #ext %d\n",
                       inet_ntoa(((struct egpnr *)egp)->en_net),
                       ((struct egpnr *)egp)->en_igw,
                       ((struct egpnr *)egp)->en_egw);
        if (length > sizeof(struct egpnr)) {
          data = (u_char *)egp + sizeof(struct egpnr);
          dataf = (u_char *)egp + length;
          for ( ; data < dataf; data++) 
             printf( "%d ", *data);
        }
        break;
      case EGPERR:
        printf(", error %d\nreturned header: ",
                       ntohs(((struct egperr *)egp)->ee_rsn));
        ep = (struct egppkt *)((struct egperr*)egp)->ee_egphd;
        printf("vers %d, typ %d, code %d, status %d, AS# %d,",
                       ep->egp_ver, ep->egp_type, ep->egp_code,
                       ep->egp_status, ntohs(ep->egp_system));
        printf(" id %d, %d, %d", ntohs(ep->egp_id),
                       ((struct egperr *)egp)->ee_egphd[10],
                       ((struct egperr *)egp)->ee_egphd[11]);
        break;
    }
  printf("\n");
  (void) fflush(stdout);
  return;
}

tracerip(dir, who, cp, size)
	struct sockaddr_in *who;		/* should be sockaddr */
	char *dir, *cp;
	register int size;
{
  register struct rip *rpmsg = (struct rip *)cp;
  register struct netinfo *n;
  u_long intempadd = who->sin_addr.s_addr;

  if (rpmsg->rip_cmd && rpmsg->rip_cmd < RIPCMD_MAX)
    printf("%s %s %s.%d", ripcmds[rpmsg->rip_cmd],
                   dir, inet_ntoa(who->sin_addr), ntohs(who->sin_port));
  else {
    printf("Bad cmd 0x%x %s %x.%d\n", rpmsg->rip_cmd,
                   dir, inet_ntoa(who->sin_addr), ntohs(who->sin_port));
    printf("size=%d cp=%x packet=%x\n", size, cp, rip_packet);
    return;
  }
  switch (rpmsg->rip_cmd) {
    case RIPCMD_REQUEST:
    case RIPCMD_RESPONSE:
      printf(":\n");
      size -= 4 * sizeof (char);
      n = rpmsg->rip_nets;
      for (; size > 0; n++, size -= sizeof (struct netinfo)) {
        if (size < sizeof (struct netinfo))
          break;
        printf("\tdst %s metric %d from %lx size %d\n",
                      inet_ntoa(sock_inaddr(&n->rip_dst)),
                      ntohl((u_long)n->rip_metric), intempadd, size);
      }
      break;
    case RIPCMD_TRACEON:
      printf(", file=%*s\n", size, rpmsg->rip_tracefile);
      break;
    case RIPCMD_TRACEOFF:
      printf("\n");
      break;
  }
  (void) fflush(stdout);
}

/* ARGSUSED */
tracehello(comment, src, dst, hello, length, nets)
	char  *comment;
	struct in_addr	src, dst;
	char	*hello;
	int	length, nets;
{
  printf("%s %d nets %s ",
	comment, nets,
	src.s_addr ? inet_ntoa(src) : "");
  printf("-> %s %s\n",
	inet_ntoa(dst), strtime);
  (void) fflush(stdout);
  return;
}

dumpinfo()
{
  FILE *fd;
  register struct interface *ifp;
  static struct bits {
		int	t_bits;
		char	*t_name;
  } flagbits[] = {
		{ RTF_UP,	"UP" },
		{ RTF_GATEWAY,	"GATEWAY" },
		{ RTF_HOST,	"HOST" },
		{ 0 }
  }, statebits[] = {
		{ RTS_PASSIVE,	   "PASSIVE" },
		{ RTS_REMOTE,	   "REMOTE" },
		{ RTS_INTERFACE,   "INTERFACE" },
		{ RTS_CHANGED,	   "CHANGED" },
		{ RTS_NOTINSTALL,  "NOTINSTALL" },
		{ RTS_NOTADVISENR, "NOTADVISENR" },
		{ RTS_SUBNET,      "SUBNET" },
		{ RTS_POINTOPOINT, "POINTOPOINT" },
		{ RTS_STATIC,      "STATIC" },
		{ RTS_HOSTROUTE,   "HOSTROUTE" },
		{ RTS_INTERIOR,    "INTERIOR" },
		{ RTS_EXTERIOR,    "EXTERIOR" },
		{ 0 }
  }, intfbits[] = {
		{ IFF_UP,          "UP" },
		{ IFF_BROADCAST,   "BROADCAST" },
		{ IFF_DEBUG,       "DEBUG" },
		{ IFF_ROUTE,       "ROUTE" },
		{ IFF_POINTOPOINT, "POINTOPOINT" },
		{ IFF_SUBNET,      "SUBNET" },
		{ IFF_PASSIVE,     "PASSIVE" },
		{ IFF_INTERFACE,   "INTERFACE" },
		{ IFF_REMOTE,      "REMOTE" },
                { IFF_NORIPOUT,    "NORIPOUT" },
                { IFF_NORIPIN,     "NORIPIN" },
                { IFF_NOHELLOOUT,  "NOHELLOOUT" },
                { IFF_NOHELLOIN,   "NOHELLOIN" },
                { IFF_NOAGE,       "NOAGE" },
                { IFF_RECONST,     "RECONST" },
                { IFF_ANNOUNCE,    "ANNOUNCE" },
                { IFF_NOANNOUNCE,  "NOANNOUNCE" },
		{ 0 }
  }, protobits[] = {
		{ RTPROTO_EGP,	    "EGP" },
		{ RTPROTO_RIP,	    "RIP" },
		{ RTPROTO_HELLO,    "HELLO" },
		{ RTPROTO_KERNEL,   "KERNEL" },
		{ RTPROTO_REDIRECT, "REDIRECT" },
		{ RTPROTO_DIRECT,   "DIRECT" },
		{ 0 }
  };

#define HELLO_REPORT	8	/* how far back we will report */

  register struct bits *p;
  register int first;
  char *cp, *ch;
  struct rthash	*rh;
  struct active_gw *agp;
  struct rthash	*base = hosthash;
  struct rt_entry *rt;
  int doinghost = 1, cnt, ind, cntr;
  struct in_addr tmpos;

  if ((fd = fopen(DUMPFILE, "a")) <= (FILE *)0) {
    perror("dumpinfo:");
    return;
  }
  fprintf(fd, "\n\tGATED version %s memory dump at %s\n\n", VERSION, strtime);
  /*
   * Print out all of the interface stuff.
   */
  fprintf(fd, "\tINTERFACE STATS:\n\n");
  for (ifp = ifnet; ifp; ifp = ifp->int_next) {
    fprintf(fd, "\tInterface: %s\t\t\tmetric: %d\n", ifp->int_name,
                ifp->int_metric);
    fprintf(fd, "\tAddress: %s\n", inet_ntoa(sock_inaddr(&ifp->int_addr)));
    fprintf(fd, "\tUp-down transitions: %u\n", ifp->int_transitions);
    if (ifp->int_flags & IFF_BROADCAST)
      fprintf(fd, "\tBroadcast Address:   %s\n",
                       inet_ntoa(sock_inaddr(&ifp->int_broadaddr)));
    if (ifp->int_flags & IFF_POINTOPOINT)
      fprintf(fd, "\tDestination Address: %s\n",
                       inet_ntoa(sock_inaddr(&ifp->int_dstaddr)));
    tmpos.s_addr = ntohl(ifp->int_net);
    fprintf(fd, "\tNet Number:    %s\t\tNet Mask:    %lx\n",
                       inet_ntoa(tmpos),
                       ifp->int_netmask);
    tmpos.s_addr = ntohl(ifp->int_subnet);
    fprintf(fd, "\tSubnet Number: %s\t\tSubnet Mask: %lx\n",
                       inet_ntoa(tmpos),
                       ifp->int_subnetmask);
    fprintf(fd, "\tReconstituted Metric (RIP ONLY): ");
    if (ifp->int_reconstmetric == -1)
      fprintf(fd, "NOT APPLIED (uses true metric)\n");
    else
      fprintf(fd, "%d\n", ifp->int_reconstmetric);
    fprintf(fd, "\tFlags: <");
    cp = "%s";
    for (first = 1, p = intfbits; p->t_bits > 0; p++) {
      if ((ifp->int_flags & p->t_bits) == 0)
        continue;
      fprintf(fd, cp, p->t_name);
      if (first) {
        cp = "|%s";
        first = 0;
      }
    }
    fprintf(fd, ">\n");
    fprintf(fd, "\tFixed Metrics:\n");
    fprintf(fd, "\t\t\tRIP:\t");
    if (ifp->int_flags & IFF_RIPFIXEDMETRIC) {
      fprintf(fd, "%d\n", ifp->int_ripfixedmetric);
    } else {
      fprintf(fd, "NONE\n");
    }
    fprintf(fd, "\t\t\tHELLO:\t");
    if (ifp->int_flags & IFF_HELLOFIXEDMETRIC) {
      fprintf(fd, "%d\n", ifp->int_hellofixedmetric);
    } else {
      fprintf(fd, "NONE\n");
    }
    fprintf(fd, "\tActive gateways on interface:");
    if (ifp->int_active_gw == NULL)
      fprintf(fd, "\tNONE\n\n");
    else {
      fprintf(fd, "\n");
      for (agp = ifp->int_active_gw; agp; agp = agp->next) {
         tmpos.s_addr = agp->addr;
         fprintf(fd, "\t\t\t%s\t<", inet_ntoa(tmpos));
         cp = "%s";
         for (first = 1, p = protobits; p->t_bits > 0; p++) {
           if ((agp->proto & p->t_bits) == 0)
             continue;
           fprintf(fd, cp, p->t_name);
           if (first) {
             cp = "|%s";
             first = 0;
           }
         }
         fprintf(fd, ">\t%d\n", agp->timer);
      }
    }
    fprintf(fd, "\n");
  }
  fprintf(fd, "-----------------------------------------------------------\n");
  /*
   * Dump all the routing information
   */
  fprintf(fd, "\tROUTING TABLES:\n\n");
onemoretime:
  for (rh = base; rh < &base[ROUTEHASHSIZ]; rh++) {
    for (rt = rh->rt_forw; rt != (struct rt_entry *)rh; rt = rt->rt_forw) {
      fprintf(fd,"\tDestination: %s\n", inet_ntoa(sock_inaddr(&rt->rt_dst)));
      fprintf(fd,"\tGateway:     %s\n", inet_ntoa(sock_inaddr(&rt->rt_router)));
      fprintf(fd,"\tInterface:   %s\t\tAge (in seconds): %d\n",
                 rt->rt_ifp->int_name, rt->rt_timer);
      switch (rt->rt_proto) {
        case RTPROTO_RIP:
                         ch = "RIP";
                         break;
        case RTPROTO_HELLO:
                         ch = "HELLO";
                         break;
        case RTPROTO_EGP:
                         ch = "EGP";
                         break;
        case RTPROTO_REDIRECT:
                         ch = "REDIRECT";
                         break;
        case RTPROTO_KERNEL:
                         ch = "KERNEL";
                         break;
        case RTPROTO_DIRECT:
                         ch = "DIRECT";
                         break;
        default:
                         ch = "UNKNOWN";
                         break;
      }
      fprintf(fd, "\tProtocol:    %s\t\tMetric (in milliseconds): %d\n",
                 ch, rt->rt_metric);
      fprintf(fd, "\tFlags: <");
      cp = "%s";
      for (first = 1, p = flagbits; p->t_bits > 0; p++) {
        if ((rt->rt_flags & p->t_bits) == 0)
          continue;
        fprintf(fd, cp, p->t_name);
        if (first) {
          cp = "|%s";
          first = 0;
        }
      }
      fprintf(fd, ">\n");
      fprintf(fd, "\tState: <");
      cp = "%s";
      for (first = 1, p = statebits; p->t_bits > 0; p++) {
        if ((rt->rt_state & p->t_bits) == 0)
          continue;
        fprintf(fd, cp, p->t_name);
        if (first) {
          cp = "|%s";
          first = 0;
        }
      }
      fprintf(fd, ">\n");
      fprintf(fd, "\tFrom Protocol: <");
      cp = "%s";
      for (first = 1, p = protobits; p->t_bits > 0; p++) {
        if ((rt->rt_fromproto & p->t_bits) == 0)
          continue;
        fprintf(fd, cp, p->t_name);
        if (first) {
          cp = "|%s";
          first = 0;
        }
      }
      fprintf(fd, ">\n");
      if (rt->rt_proto & RTPROTO_HELLO) {
        fprintf(fd, "\tMinimum HELLO time delay in last %d minutes: %d\n",
                      WINDOW_INTERVAL, rt->rt_hwindow.h_min);
        fprintf(fd, "\tLast %d HELLO time delays:\n\t\t", HELLO_REPORT);
        cnt = 0;
        ind = rt->rt_hwindow.h_index;
        while (cnt < HELLO_REPORT) {
          fprintf(fd, "%d ", rt->rt_hwindow.h_win[ind]);
          cnt++;
          ind++;
          if (ind >= HWINSIZE)
            ind = 0;
        }
        fprintf(fd, "\n");
      }
      fprintf(fd, "\tRESTRICTIONS:\n");
      if (rt->rt_announcelist != NULL) {
        fprintf(fd, "\t\tANNOUNCED via <");
        cp = "%s";
        for (first = 1, p = protobits; p->t_bits > 0; p++) {
          if ((rt->rt_announcelist->rproto & p->t_bits) == 0)
             continue;
           fprintf(fd, cp, p->t_name);
           if (first) {
             cp = "|%s";
             first = 0;
           }
         }
         fprintf(fd, "> ");
         if (rt->rt_announcelist->rproto & RTPROTO_EGP)
           fprintf(fd, "EGP metric %d ", rt->rt_announcelist->regpmetric);
         cntr = 0;
         if (rt->rt_announcelist->rintf[cntr] == 0)
           fprintf(fd, "OUT ALL INTERFACES\n");
         else {
           fprintf(fd, "OUT INTERFACES:\n");
           while ((rt->rt_announcelist->rintf[cntr] != 0) ||
                  (cntr >= MAXINTERFACE)) {
             fprintf(fd, "\t\t\t\t");
             tmpos.s_addr = rt->rt_announcelist->rintf[cntr];
             fprintf(fd, "%s\n", inet_ntoa(tmpos));
             cntr++;
           }
         }
      }
      if (rt->rt_noannouncelist != NULL) {
        fprintf(fd, "\t\tNOT ANNOUNCED via <");
        cp = "%s";
        for (first = 1, p = protobits; p->t_bits > 0; p++) {
          if ((rt->rt_noannouncelist->rproto & p->t_bits) == 0)
             continue;
           fprintf(fd, cp, p->t_name);
           if (first) {
             cp = "|%s";
             first = 0;
           }
         }
         fprintf(fd, "> ");
         if (rt->rt_noannouncelist->rproto & RTPROTO_EGP)
           fprintf(fd, "EGP metric %d ", rt->rt_noannouncelist->regpmetric);
         cntr = 0;
         if (rt->rt_noannouncelist->rintf[cntr] == 0)
           fprintf(fd, "OUT ALL INTERFACES\n");
         else {
           fprintf(fd, "OUT INTERFACES:\n");
           while ((rt->rt_noannouncelist->rintf[cntr] != 0) ||
                  (cntr >= MAXINTERFACE)) {
             fprintf(fd, "\t\t\t\t");
             tmpos.s_addr = rt->rt_noannouncelist->rintf[cntr];
             fprintf(fd, "%s\n", inet_ntoa(tmpos));
             cntr++;
           }
         }
      }
      if (rt->rt_listenlist != NULL) {
        fprintf(fd, "\t\tNOTLISTENEDTO via <");
        cp = "%s";
        for (first = 1, p = protobits; p->t_bits > 0; p++) {
          if ((rt->rt_listenlist->rproto & p->t_bits) == 0)
             continue;
           fprintf(fd, cp, p->t_name);
           if (first) {
             cp = "|%s";
             first = 0;
           }
         }
         fprintf(fd, "> ");
         if (rt->rt_listenlist->rproto & RTPROTO_EGP)
           fprintf(fd, "EGP metric %d ", rt->rt_listenlist->regpmetric);
         cntr = 0;
         if (rt->rt_listenlist->rintf[cntr] == 0)
           fprintf(fd, "FROM ALL INTERFACES\n");
         else {
           fprintf(fd, "FROM INTERFACES:\n");
           while ((rt->rt_listenlist->rintf[cntr] != 0) ||
                  (cntr >= MAXINTERFACE)) {
             fprintf(fd, "\t\t\t\t");
             tmpos.s_addr = rt->rt_listenlist->rintf[cntr];
             fprintf(fd, "%s\n", inet_ntoa(tmpos));
             cntr++;
           }
         }
      }
      if (rt->rt_srclisten != NULL) {
        fprintf(fd, "\t\tLISTENEDTO via <");
        cp = "%s";
        for (first = 1, p = protobits; p->t_bits > 0; p++) {
          if ((rt->rt_srclisten->rproto & p->t_bits) == 0)
             continue;
           fprintf(fd, cp, p->t_name);
           if (first) {
             cp = "|%s";
             first = 0;
           }
         }
         fprintf(fd, "> ");
         if (rt->rt_srclisten->rproto & RTPROTO_EGP)
           fprintf(fd, "EGP metric %d ", rt->rt_srclisten->regpmetric);
         cntr = 0;
         if (rt->rt_srclisten->rintf[cntr] == 0)
           fprintf(fd, "FROM ALL GATEWAYS\n");
         else {
           fprintf(fd, "FROM GATEWAYS:\n");
           while ((rt->rt_srclisten->rintf[cntr] != 0) ||
                  (cntr >= MAXINTERFACE)) {
             fprintf(fd, "\t\t\t\t");
             tmpos.s_addr = rt->rt_srclisten->rintf[cntr];
             fprintf(fd, "%s\n", inet_ntoa(tmpos));
             cntr++;
           }
         }
      }
      fprintf(fd, "\n");
    }
  }
  if (doinghost) {
    base = nethash;
    doinghost = 0;
    goto onemoretime;
  }
  (void) fflush(fd);
  (void) fclose(fd);
}
