/*
 *   CENTER FOR THEORY AND SIMULATION IN SCIENCE AND ENGINEERING
 *			CORNELL UNIVERSITY

 *      Portions of this software may fall under the following
 *      copyrights: 
 *
 *	Copyright (c) 1983 Regents of the University of California.
 *	All rights reserved.  The Berkeley software License Agreement
 *	specifies the terms and conditions for redistribution.
 *
 *	The copyright (c) of the EGPUP User Process, Paul Kirton and
 *	ISI, August 23, 1984
 *
 */

/*
 *	The following acknowledgments and thanks apply:
 *
 *	Mike Petry (petry@trantor.umd.edu), University of Maryland, for
 *	development of the HELLO routines used in this program.
 *
 *	Craig Partridge (craig@loki.bbn.com), BBN Laboratories, for
 *	thorough linting, profiling, and performance enhancements to
 *	this program.
 *
 *	Charles Hedrick (hedrick@topaz.rutgers.edu),
 *	Chris Maio (chris@columbia.edu), Sergio Heker (heker@jvnca.csc.org),
 *	and any other BETA testers who have helped in finding bugs.
 *
 *	Mark Fedor (now fedor@nisc.nyser.net) for the development and
 *	maintenance up to release 1.3.1.  Also for his continuing
 *	advice.
 *
 *	Thomas Narten <narten@purdue.edu> for his analysis and
 *	modifications to the EGP and other helpful improvements.
 *
 *
 *
 *  GATED - based on Kirton's EGP, UC Berkeley's routing daemon (routed),
 *	    and DCN's HELLO routing Protocol.  HELLO output routines
 *	    were developed and originally written at the University of
 *	    Maryland by Mike Petry.
 * 
 *  December 5, 1986 - Mark Fedor
 *
 * $Header: trace.h,v 1.3.1.18 88/02/10 20:05:19 jch Exp $
 *
 */

/*
 * trace.h
 */

				/* tracing levels */
#define	TR_INT		0x1	/* internal errors */
#define TR_EXT		0x2	/* external changes resulting from egp */
#define TR_RT		0x4	/* routing changes */
#define TR_PKT		0x8	/* all egp packets sent and received */
#define TR_UPDATE	0x10	/* trace update info sent */
#define TR_RIP		0x20	/* trace update info sent */
#define TR_HELLO	0x40	/* trace update info sent */

#define TRACE_TRC   if (tracing) printf
#define TRACE_INT   if (tracing & TR_INT) printf
#define TRACE_EXT   if (tracing & TR_EXT) printf
#define TRACE_RT    if (tracing & TR_RT) printf
#define TRACE_PKT   if (tracing & TR_PKT) printf
#define TRACE_UPD   if (tracing & TR_UPDATE) printf
#define TRACE_RIP   if (tracing & TR_RIP) printf
#define TRACE_HEL   if (tracing & TR_HELLO) printf

/*
 *  Trace Flags for traceon()
 */
#define RIP_TRACE	0x1
#define HELLO_TRACE	0x2
#define GEN_TRACE	0x4

#define	TRACE_ACTION(action, route) { \
	    if (tracing & TR_RT) \
		traceaction( stdout, "action", route); \
	}

#define TRACE_EGPPKT(comment, src, dst, egp, length) { \
	    if( tracing & TR_PKT) \
		traceegp( "comment", src, dst, egp, length); \
	}
 
#define TRACE_RIPOUTPUT(ifp, dst, size) { \
	    if ((tracing & TR_UPDATE) && (tracing & TR_RIP)) \
		tracerip("to", dst, rip_packet, size); \
	}

#define TRACE_RIPINPUT(ifp, dst, size, ripinfo) { \
	    if (tracing & TR_RIP) \
		tracerip("from", dst, ripinfo, size); \
	}
 
#define TRACE_HELLOPKT(comment, src, dst, hello, length, nets) { \
	    if ((tracing & TR_UPDATE) && (tracing & TR_HELLO)) \
		tracehello( "comment", src, dst, hello, length, nets); \
	}

